//#include "StdAfx.h"
#include "TreeMap.h"

#pragma once

// Constructor
TreeMap::TreeMap(const char* inputfile, const char* outputfile,double diff)
{
	infile = inputfile; // cg file (dimension cluster file)
	outfile = outputfile; // tm file (treemap file)
	Create(diff);
}

void TreeMap::Create(double diff)
{
	// 1. Read Input File
	ReadFile();

	// 2. Calculate Grid Size
	CalcGridSize(diff); 

	// 3. Create HCurve
	//HCurve* hc = new HCurve();
	HCurve* hc = new HCurve(gsizex,gsizey);

	// 4. Get Leafnodes - Depth first traversal of tree
	leafnodes = new int[ndata];
	leafcount=0;
	GetLeafNodes(0);

	// 5. Write Output file
	FILE *tmfile = fopen(outfile,"w");
	fprintf(tmfile, "%d %d\n", ndata,gsizex); //this needs changing to store y-value also.

	//-- Find positions of each leaf node
	int tempx,tempy;
	for (int i=0;i<leafcount;i++){
		tempx = hc->xco(i);
		tempy = hc->yco(i);
		fprintf(tmfile, "%d %d %d\n", leafnodes[i],tempx,tempy);
	}
	fclose(tmfile);

}

// Read the CG (Cluster) File.
void TreeMap::ReadFile()
{
    int i;
	//--open file
	FILE *cgfile = fopen(infile,"r");
	//--read header
	fscanf(cgfile, "%d %d\n", &nnodes, &ndata);
	//--read data
	int val1,val2,val3,val4;
	//initialise data
	dataset = new int*[nnodes];
	for (i=0;i<nnodes;i++){
		dataset[i] = new int[6];
		for (int j=0;j<6;j++)
			dataset[i][j]=-1;
	}
	//read data
	for (i=0;i<nnodes;i++){
		fscanf(cgfile, "%d %d %d %d\n", &val1, &val2, &val3, &val4);
		dataset[i][0] = val1; //node id
		dataset[i][1] = val2; //parent id
		dataset[i][2] = val3; //size 
		dataset[i][3] = val4; //data ref
		// if this is a child node (i.e. not the root) 
		// then store reference from the parent.
		// This is used for traversing the nodes.
		if (val2!=-1){ 
			int parentid = val2;
			// if this is the first child to be stored
			if (dataset[parentid][4]==-1)  
				dataset[parentid][4] = val1; // store reference to child 1
			else
				dataset[parentid][5] = val1; // store reference to child 2
		}
	}
	fclose(cgfile);
}

// Helper Function - is 'i' an odd number.
bool TreeMap::IsOdd(int i){
	if ((i%2)!=0) 
		return true;
	else 
		return false;
}



// Calculate Grid Size from datasize 
void TreeMap::CalcGridSize(double diff)
{
	int square,error,minerror,min,max;
	square = ceil(sqrt((double)ndata));
	//set default
	gsizex = square;
	gsizey = square;
	if (IsOdd(gsizex)) gsizex=gsizex+1;
	if (IsOdd(gsizey)) gsizey=gsizey+1;
	// If we can't represent it perfectly by an even square
	if ( (( square * square ) - ndata)!=0 || IsOdd(square) ){
		// search for a good rectangle
		min = (int)ceil(((double)square - (diff * (double)square)));
		max = (int)ceil(((double)square + (diff * (double)square)));
		minerror = (( square * square ) - ndata);
		for (int i=min;i<=max;i++){
			for(int j=i;j<=max;j++){
				error = (i * j) - ndata;
				if ((error<minerror) && (error>=0) && (!IsOdd(i)) && (!IsOdd(j)) ){
					gsizex=j;
					gsizey=i;
					minerror=error;
				}
			}
		}
	}
}

// Depth First
void TreeMap::GetLeafNodes(int i)
{
	// if leaf node
	if (dataset[i][2]==1){
		leafnodes[leafcount]=dataset[i][3]; // store dataref
		leafcount++;
	}
	// if parent node (i.e. size>1)
	if (dataset[i][2]>1){
		// child1
		int child1=dataset[i][4];
		GetLeafNodes(child1);
		// child2
		int child2=dataset[i][5];
		GetLeafNodes(child2);
	}
	
}




