//linklist class defined by the system

#include "CLinkList.h"

CLinkList::CLinkList()
{
	head = new CLinkListNode();
	tail = new CLinkListNode();
	head->next = tail;
	tail->prev = head;

	length = 0;
}


CLinkList::~CLinkList()
{
	CLinkListNode *node, *temp;
	node = head;

	while ( node != NULL )
	{
		temp = node->next;
		delete node;
		node = temp;
	}
}


void CLinkList::AddNodeAtEnd(void* ele)
{
	CLinkListNode *new_node, *node1;
	new_node = new CLinkListNode(ele);
	node1 = tail->prev;

	node1->next = new_node;

	new_node->prev = tail->prev;
	new_node->next = tail;

	tail->prev = new_node;

	length += 1;
}


void CLinkList::AddNodeAtStart(void* ele)
{
	CLinkListNode *new_node, *node1;
	new_node = new CLinkListNode(ele);
	node1 = head->next;

	node1->prev = new_node;

	new_node->prev = head;
	new_node->next = node1;

	head->next = new_node;

	length += 1;
}

void CLinkList::AddNodeAfter(void* new_ele, CLinkListNode *node)
{
	CLinkListNode *new_node;
	new_node = new CLinkListNode(new_ele);

	new_node->prev = node;
    new_node->next = node->next;

    node->next = new_node;
    
    new_node->next->prev = new_node;

	length += 1;
}


bool CLinkList::DeleteNode(CLinkListNode* del_node)
{
	if ( del_node == head || del_node == tail )
		return false;

	CLinkListNode *node1, *node2;
	node1 = head;
	node2 = node1->next;

	while ( node2 != tail && node2 != del_node )
	{
		node1 = node2;
		node2 = node2->next;
	}

	if ( node2 == tail )
		return false;

	node2 = del_node->next;

	delete del_node;
	
	node1->next = node2;
	node2->prev = node1;

	length -= 1;
	
	return true;
}


bool CLinkList::DeleteElement(void* ele)
{
	CLinkListNode *node1, *node2, *del_node;
	node1 = head;
	node2 = node1->next;

	while ( node2 != tail && node2->element != ele )
	{
		node1 = node2;
		node2 = node2->next;
	}

	if ( node2 == tail )
		return false;

    del_node = node2;
	node2 = del_node->next;

	delete del_node;
	
	node1->next = node2;
	node2->prev = node1;

	length -= 1;
	
	return true;
}

int CLinkList::GetLength()
{
	return length;
}


CLinkListNode* CLinkList::GetFirst()
{
    if ( head->next != tail )
        return head->next;
    else
        return NULL;
}


CLinkListNode* CLinkList::GetNext( CLinkListNode* node)
{
    if ( node->next != tail )
        return node->next;
    else
        return NULL;
}

CLinkListNode* CLinkList::Get( int N )
{
    if ( N >= length || N < 0 )
        return NULL;

    CLinkListNode* node;
    node = head;
    for ( int i = 0; i <= N; i++ )
    {
        node = node->next;
    }

    return node;
}

void* CLinkList::GetElement(int N)
{
    if ( N >= length || N < 0 )
        return NULL;

    CLinkListNode* node;
    node = head;
    for ( int i = 0; i <= N; i++ )
    {
        node = node->next;
    }

    return node->element;
}

void* CLinkList::TransToArray()
{
    if ( length <= 0 )
        return NULL;

    char** node_array = new char* [length];
    CLinkListNode* node = GetFirst();
    for ( int i = 0; i < length; i++ )
    {
        node_array[i] = (char*) node->element;
        node = GetNext(node);
    }
    return (void*) node_array;
}

bool CLinkList::Delete(int N)
{
    if ( N >= length || N < 0 )
        return false;

    CLinkListNode *node, *next_node;
    node = head;
    next_node = node->next;
    for ( int i = 1; i <= N; i++ )
    {
        node = next_node;
        next_node = node->next;
    }

    node->next = next_node->next;
    next_node->next->prev = node;
    delete next_node;
	length -= 1;

    return true;
}

void CLinkList::RemoveAll()
{
    int le = length;
    for (int i = 0; i < le; i++)
    {
        Delete(0);
    }
}

bool CLinkList::RemoveNode(CLinkListNode* del_node)
{
	CLinkListNode *node1, *node2;
	node1 = del_node->prev;
    node2 = del_node->next;

	node1->next = node2;
	node2->prev = node1;

	length -= 1;
	return true;
}

void CLinkList::DeleteNodeAtEnd()
{
	CLinkListNode *node1, *node2;
	node1 = tail->prev;
    if (node1 == head)
        return;

    node2 = node1->prev;
    node2->next = tail;
    tail->prev = node2;

    delete node1;

	length -= 1;
}

void CLinkList::AddNodeAtStart(CLinkListNode* new_node)
{
	CLinkListNode *node1;
	node1 = head->next;

	node1->prev = new_node;

	new_node->prev = head;
	new_node->next = node1;

	head->next = new_node;

	length += 1;
}

