//  Modify this file to set up default database connections. 
//  If you want to set up database automatically by using 
//  DBManager *db = new DBManager(), modify the url (database name), loginName, 
//  and loginPassword in the constructor without parameters 
//  according to the settings of your database. You can also 
//  explicitly set up the connection by using the constructor with parameters

//  In order to enable the db connection, you need to set an ODBC system data source 
//  in your machine. 

#include <iostream>
#include <string>
#include "DBManager.h"
#include "odbc.h"
#include "ConnectionFactory.h"


DBManager :: DBManager()
{
	this->url = "oracle_xmdv";
    this->loginName = "yj";
    this->loginPassword = "yj";
	con = NULL;
    stmt = NULL;
    bool isConnected = false;

	otl_connect::otl_initialize(); // initialize ODBC environment
	Connect();
}


/*
DBManager :: DBManager()
{
	this->url = "xmdv";
    this->loginName = "yj";
    this->loginPassword = "yj";
	con = NULL;
    stmt = NULL;
    bool isConnected = false;

	otl_connect::otl_initialize(); // initialize ODBC environment
	Connect();
}
*/

DBManager :: DBManager(std::string url, std::string loginName, std::string loginPassword)
{
    this->url = url;
    this->loginName = loginName;
    this->loginPassword = loginPassword;
	con = NULL;
    stmt = NULL;
	bool isConnected = false;

	otl_connect::otl_initialize(); // initialize ODBC environment
	Connect();
}

DBManager :: ~DBManager()
{
    Disconnect();
}

bool DBManager :: IsConnected()
{
	return isConnected;
}

bool DBManager :: Connect()
{
	try{
		if (stmt != NULL)
        {
            delete stmt;
            stmt = NULL;
        }
		if (con != NULL)
        {
            delete con;
            con = NULL;
        }
        con = ConnectionFactory::getInstance()->makeConnection(loginName, loginPassword, url);
        stmt = con->createStatement();
        isConnected = true;
        return true;
    }
	catch(otl_exception& p){ // intercept OTL exceptions
		std::cerr<<p.code<<std::endl; // print out error code
		std::cerr<<p.sqlstate<<std::endl; // print out error SQLSTATE
		std::cerr<<p.msg<<std::endl; // print out error message
		std::cerr<<p.stm_text<<std::endl; // print out SQL that caused the error
		std::cerr<<p.var_info<<std::endl; // print out the variable that caused the error
		isConnected = false;
        return false;
	}
}

bool DBManager :: Connect(std::string url, std::string loginName, std::string loginPassword)
{
    this->url = url;
    this->loginName = loginName;
    this->loginPassword = loginPassword;
    
    return Connect();
}

void DBManager :: Disconnect()
{
    if (stmt != NULL)
    {
        delete stmt;
        stmt = NULL;
    }
    if (con != NULL)
    {
        delete con;
        con = NULL;
    }

	isConnected = false;
}

bool DBManager :: ExecuteUpdate(std::string sqlString)
{
    try 
    {
        stmt->execute(sqlString);
        return true;
    }
    catch(otl_exception& p){ // intercept OTL exceptions
		std::cerr<<p.code<<std::endl; // print out error code
		std::cerr<<p.sqlstate<<std::endl; // print out error SQLSTATE
		std::cerr<<p.msg<<std::endl; // print out error message
		std::cerr<<p.stm_text<<std::endl; // print out SQL that caused the error
		std::cerr<<p.var_info<<std::endl; // print out the variable that caused the error
        return false;
	}
}

ResultSet* DBManager :: ExecuteQuery(std::string query)
{
    try 
    {
        ResultSet* rset = stmt->executeQuery(query);
        return rset;
    }
    catch(otl_exception& p){ // intercept OTL exceptions
		std::cerr<<p.code<<std::endl; // print out error code
		std::cerr<<p.sqlstate<<std::endl; // print out error SQLSTATE
		std::cerr<<p.msg<<std::endl; // print out error message
		std::cerr<<p.stm_text<<std::endl; // print out SQL that caused the error
		std::cerr<<p.var_info<<std::endl; // print out the variable that caused the error
        return NULL;
	}
}

void DBManager :: Commit()
{
    if (con)
        con->commit();
}