//  The usage of database is simple in this sytem, the steps are:
//  1. connect to a db server
//  2. set the ODBC data source in your machine
//  3. new a DBManager using a proper constructor
//     such as DBManager* db = new DBManager() to use the default constructors and user
//  4. call the function Connect() to connect to the database. It returns true if db is correctly connected.
//  5. You can test the connect later using the function IsConnected()
//  6. To update your database 
//       ExecuteUpdate(std::string sqlString);
//       Commit();   for database update
//       You can Commit once after multiple ExecuteUpdate(std::string sqlString) calls for efficiency
//  7. To Query the database 
//       ResultSet* re = ExecuteQuery(std::string query);
//      see ResultSet.h for how to get results out of re. 
//  8. Call Disconnect for disconnect with the db. 



#ifndef _DBMANAGER_H
#define _DBMANAGER_H

#include <string>
#include "connection.h"
#include "Statement.h"
#include "Resultset.h"


class DBManager
{
protected:
    std::string url;
    std::string loginName;
    std::string loginPassword;
    Connection* con;
    Statement* stmt;
 	bool isConnected;


public:
	DBManager();
	DBManager(std::string url, std::string loginName, std::string loginPassword);
	~DBManager();
	
    bool IsConnected();
    bool Connect();
    bool Connect(std::string url, std::string loginName, std::string loginPassword);
    void Disconnect();
    bool ExecuteUpdate(std::string sqlString);
    ResultSet* ExecuteQuery(std::string query);
    void Commit();

};

#endif

