#include<string>
#include<math.h>
#include <fstream>
#include <sstream>

#include "BinarySimilarityCalculator.h"

using namespace std;

BinarySimilarityCalculator :: BinarySimilarityCalculator()
{
    similarityMatrix = NULL;
    minor_weight = 0.5;
}

BinarySimilarityCalculator :: BinarySimilarityCalculator(int numDims)
{
    similarityMatrix = NULL;
    minor_weight = 0.5;

    init(numDims, 2);
}

BinarySimilarityCalculator :: ~BinarySimilarityCalculator()
{
    int i;

    if (similarityMatrix != NULL)
    {
        for (i = 0; i < numDims; i++)
        {
            delete [] similarityMatrix[i];
        }
        delete [] similarityMatrix;
    }
    if (counts != NULL)
        delete [] counts;
}

bool BinarySimilarityCalculator :: init(int numDims, int numBins)
{
    //set numDims and numBins;
    if (numDims < 1)
        return false;
    this->numDims = numDims;

    int i, j;

   //init similarityMatrix;
    similarityMatrix = new double*[numDims];
    for (i = 0; i < numDims; i++)
    {
        similarityMatrix[i] = new double[numDims];
    }

    for (i = 0; i < numDims; i++)
    {
        for (j = 0; j < numDims; j++)
        {
            similarityMatrix[i][j] = 0;
        }
    }
    counts = new double[numDims];
    for (i = 0; i < numDims; i++)
    {
        counts[i] = 0;
    }
    return true;
}

bool BinarySimilarityCalculator :: Update(double* values)
{
    int i, j;

    for (i = 0; i < numDims; i++)
    {
        if (fabs(values[i]) >= 0.5)
        {
            counts[i] += 1;
            for (j = i+1; j < numDims; j++)
            {
                if (fabs(values[j]) >= 0.5)
                {
                    similarityMatrix[i][j]++;
                    similarityMatrix[j][i]++;
                }
            }

        }
    }
    return true;
}

bool BinarySimilarityCalculator :: Calculate()
{
    //load rows
    int j, k;

    for (j = 0; j < numDims; j++)
    {
        for (k = j+1; k < numDims; k++)
        {
            if (counts[j] > 0.1 && counts[k] > 0.1) 
            {
//                similarityMatrix[j][k] *= similarityMatrix[j][k]/counts[j]/counts[k];
                if (counts[j] < counts[k])
                     similarityMatrix[j][k] = minor_weight * similarityMatrix[j][k]/counts[j]
                     + (1.-minor_weight) * similarityMatrix[j][k]/counts[k];
                else
                     similarityMatrix[j][k] = (1.-minor_weight) * similarityMatrix[j][k]/counts[j]
                     + minor_weight * similarityMatrix[j][k]/counts[k];

                similarityMatrix[j][k] = 1-similarityMatrix[j][k];
            }
            else if (counts[j] < 0.1 && counts[k] < 0.1)
                similarityMatrix[j][k] = 0;
            else
                similarityMatrix[j][k] = 1;
            similarityMatrix[k][j] = similarityMatrix[j][k];

        }
    }
    return true;
}


void BinarySimilarityCalculator::SetMinorWeight(double minor_weight)
{
    if (minor_weight > 1)
        minor_weight =1;
    else if (minor_weight < 0)
        minor_weight = 0; 

    this->minor_weight = minor_weight;
}


