#include <string>
#include <iostream>
#include <fstream>
#include <sstream>
#include <ctime>
#include <stdio.h>


#include "DBFlatDataSetLoader.h"
#include "SimilarityCalculator.h"
#include "NumericSimilarityCalculator.h"
#include "BinarySimilarityCalculator.h"
#include "../../Interface/UserNotifier.h"


DBFlatDataSetLoader :: DBFlatDataSetLoader()
{
}

DBFlatDataSetLoader :: ~DBFlatDataSetLoader()
{
}

bool DBFlatDataSetLoader :: Load(DBManager* db, std::string OKCFileName, bool binaryMode)
{
    std::string DBFileName = OKCFileName.substr(0, OKCFileName.length()-3) + "tmp";
    std::string DBFileName_final = OKCFileName.substr(0, OKCFileName.length()-3) + "db";

	int i, j;
    //get reader to OKCFILE
    std::ifstream infile (OKCFileName.c_str());

    //get # dimensions, #data items
	int numDims, numDataItems;
    bool hasDescription;
    std::string destr;
    std::string itemDestr;
//	infile >> numDims;
//	infile >> numDataItems;
  
    ReadFirstLine(&infile, numDims, numDataItems, hasDescription, destr);

    //construct all dimension space of the okc file
    //construct all dimension space of the okc file
	DimensionSpace *dimSpace = new DimensionSpace();
	if (!dimSpace->LoadDimensionsFromFile(&infile, numDims))
    {    
        delete dimSpace;
        dimSpace = NULL;
        
        infile.close();
        return false;
    }

    std::string dataTableName = OKCFileName.substr(OKCFileName.rfind('/')+1, OKCFileName.rfind('.') - OKCFileName.rfind('/') - 1);
    dataTableName = "wx" + dataTableName.substr(dataTableName.rfind('\\')+1, dataTableName.rfind('.') - dataTableName.rfind('\\') - 1);


    SimilarityCalculator* similarityCalculator;
    if (binaryMode)
        similarityCalculator = new BinarySimilarityCalculator();
    else
        similarityCalculator = new NumericSimilarityCalculator();
    
    if (!dimSpace->IsMappingUncertain())
    {
        UserNotifier::UpdateProgressDialog(1, "Inserting dataset into the database and building the dimmension similarity table");
        //get writer to DBfile
        std::ofstream outfile(DBFileName.c_str());

        WriteFirstLine(&outfile, numDims, numDataItems, hasDescription, destr);

        //write dimensions of the all dimension space to the DBFile
	    dimSpace->WriteDimensionsToFile(&outfile);

//        int numBins = 100;
        int numBins = 20;
	    similarityCalculator->init(numDims, numBins); 

        //create db table
	    db->ExecuteUpdate(GetDropTableString(dataTableName));
	    db->ExecuteUpdate(GetCreateTableString(dataTableName, dimSpace));
    
	    double *nordata;
        std::string *strs;
        Dimension** dims;

	    nordata = new double [numDims];
        strs = new std::string [numDims];
        dims = new Dimension* [numDims];

        for(i = 0; i < numDims; i++)
        {
            dims[i] = dimSpace->GetDimension(i);
        }

//	    srand( (unsigned)time( NULL ));
        //for all data items

        char *buf = new char[262144];
        char *separator = ", \t\n";  // separators required by strtok() function
    
        int dimType;
        for (i = 0; i < numDataItems; i++)
        {
            infile.getline(buf, 262144); // read all the dimension names
            char *token = strtok(buf, separator);  // get the first token name

            if (hasDescription)
            {
                itemDestr = token;
                token = strtok( NULL, separator);
            }
		    for (j = 0; j < numDims; j++)
		    {
                strs[j] = token;
                token = strtok( NULL, separator);

                nordata[j] = dims[j]->GetNorValue(strs[j]);

                dimType = dims[j]->GetType();
                if (dimType == NORMINAL_DIMENSION || dimType == DATE_DIMENSION)
                    strs[j] = "'" + strs[j] + "'";
            } 
		    // insert the data item into the data base
		    db->ExecuteUpdate(GetInsertTableString(dataTableName, numDims, strs, i, hasDescription, itemDestr));

            // update similarity table in the similarity calculator
            similarityCalculator->Update(nordata);
        }
        delete [] buf;

        db->Commit();

        delete [] nordata;
        delete [] strs;
        delete [] dims;
        //end

        UserNotifier::UpdateProgressDialog(15, "Inserting dataset into the database succeeded. Inserting dimension similarity table"); 
            
        //write db table information into db file
        outfile << dataTableName.c_str() << std::endl;

        //create and populate similarity table
        //std::string sTableName = dataTableName + "_simi";
	    //similarityCalculator->StoreSimilarityTableToDB(db, sTableName);
 
        //write similarity table information into db file
	    //outfile << sTableName << " " << numBins;
        similarityCalculator->Calculate();
        std::string sFileName =  OKCFileName.substr(0, OKCFileName.length()-3) + "cov";
        similarityCalculator->WriteCovMatrix(sFileName);
        outfile << sFileName;

	    //close files
	    infile.close();
	    outfile.close();
        rename(DBFileName.c_str(), DBFileName_final.c_str());

        UserNotifier::UpdateProgressDialog(20, "Inserting dimension similarity table succeeded"); 
    }
    else
    {
        //*************First scan - put the dataset into the database and 
        //build the dimension - value-normalized value hashtable in each dimension
        
        UserNotifier::UpdateProgressDialog(1, "First scan: Inserting dataset into the database and finding Min Max"); 

        //create db table
	    db->ExecuteUpdate(GetDropTableString(dataTableName));
	    db->ExecuteUpdate(GetCreateTableString(dataTableName, dimSpace));
    
	    double *nordata;
        std::string *strs;
        Dimension** dims;

	    nordata = new double [numDims];
        strs = new std::string [numDims];
        dims = new Dimension* [numDims];

        for(i = 0; i < numDims; i++)
        {
            dims[i] = dimSpace->GetDimension(i);
        }

//	    srand( (unsigned)time( NULL ));
        //for all data items

        char *buf = new char[262144];
        char *separator = ", \t\n";  // separators required by strtok() function
    
//        infile.getline(buf, 2);
        int dimType;
        for (i = 0; i < numDataItems; i++)
        {
            infile.getline(buf, 262144); // read all the dimension names
            char *token = strtok(buf, separator);  // get the first token name

            if (hasDescription)
            {
                itemDestr = token;
                token = strtok( NULL, separator);
            }
		    for (j = 0; j < numDims; j++)
		    {
                strs[j] = token;
                token = strtok( NULL, separator);

 //               nordata[j] = dims[j]->GetNorValue(strs[j]);
                if (dims[j]->IsMappingUncertain())
                {
                    dims[j]->ChangeMinMax(strs[j]);
                }

                dimType = dims[j]->GetType();
                if (dimType == NORMINAL_DIMENSION || dimType == DATE_DIMENSION)
                    strs[j] = "'" + strs[j] + "'";

            } 
		    // insert the data item into the data base
		    db->ExecuteUpdate(GetInsertTableString(dataTableName, numDims, strs, i, hasDescription, itemDestr));
        }

        db->Commit();

 	    //close files
	    infile.close();

        //*************Second scan - build similarity table

        UserNotifier::UpdateProgressDialog(10, "Second scan: Building dimension similarity table"); 
        //get reader to OKCFILE
        infile.open(OKCFileName.c_str());

	    infile >> numDims;
	    infile >> numDataItems;

        //a fake dimSpace is created into to run infile pointer to proper position
	    DimensionSpace *dimSpace1 = new DimensionSpace();
	    dimSpace1->LoadDimensionsFromFile(&infile, numDims);
        delete dimSpace1;


        //get writer to DBfile
        std::ofstream outfile(DBFileName.c_str());

        //write # dimensions, #data items to the DBFile
        WriteFirstLine(&outfile, numDims, numDataItems, hasDescription, destr);

        //write dimensions of the all dimension space to the DBFile
	    dimSpace->WriteDimensionsToFile(&outfile);

        int numBins = 20;
	    similarityCalculator->init(numDims, numBins); 

  //      infile.getline(buf, 2);
        for (i = 0; i < numDataItems; i++)
        {
            infile.getline(buf, 262144); // read all the dimension names
            char *token = strtok(buf, separator);  // get the first token name

            if (hasDescription)
            {
                itemDestr = token;
                token = strtok( NULL, separator);
            }

            for (j = 0; j < numDims; j++)
		    {
                strs[j] = token;
                token = strtok( NULL, separator);

                nordata[j] = dims[j]->GetNorValue(strs[j]);
            } 
            // update similarity table in the similarity calculator
            similarityCalculator->Update(nordata);
        }

        delete [] buf;

		for (j = 0; j < numDims; j++)
		{
            dims[j]->SetMappingUncertain(false);
        }

        delete [] nordata;
        delete [] strs;
        delete [] dims;
        //end

        //write db table information into db file
        outfile << dataTableName.c_str() << std::endl;

        UserNotifier::UpdateProgressDialog(15, "Second scan: Inserting dimension similarity table into database"); 

        //create and populate similarity table
        //std::string sTableName = dataTableName + "_simi";
	    //similarityCalculator->StoreSimilarityTableToDB(db, sTableName);
        //write similarity table information into db file
	    //outfile << sTableName << " " << numBins;
        similarityCalculator->Calculate();
        std::string sFileName =  OKCFileName.substr(0, OKCFileName.length()-3) + "cov";
        similarityCalculator->WriteCovMatrix(sFileName);
        outfile << sFileName;

	    //close files
	    infile.close();
	    outfile.close();
        rename(DBFileName.c_str(), DBFileName_final.c_str());
    }

    delete similarityCalculator;
	delete dimSpace;
    return true;
}

/*
bool DBFlatDataSetLoader :: LoadGroupInformation(DBManager* db, std::string DBFileName)
{
    std::string groupFileName = OKCFileName.substr(0, OKCFileName.length()-3) + "grp";

	int i, j;
    //get reader to OKCFILE
    std::ifstream infile (DBFileName.c_str());

    //get # dimensions, #data items
	int numDims, numDataItems;

	infile >> numDims;
	infile >> numDataItems;

    //construct all dimension space of the okc file
    //construct all dimension space of the okc file
	DimensionSpace *dimSpace = new DimensionSpace();
	if (!dimSpace->LoadDimensionsFromFile(&infile, numDims))
    {    
        delete dimSpace;
        dimSpace = NULL;
        
        infile.close();
        return false;
    }

    infile >> dataTableName;
    infile.close();

    std::string dataTableName = OKCFileName.substr(OKCFileName.rfind('/')+1, OKCFileName.rfind('.') - OKCFileName.rfind('/') - 1);
    dataTableName = "wx" + dataTableName.substr(dataTableName.rfind('\\')+1, dataTableName.rfind('.') - dataTableName.rfind('\\') - 1);
	db->ExecuteUpdate(GetDropTableString(dataTableName));
	db->ExecuteUpdate(GetCreateTableString(dataTableName, dimSpace));

	double *nordata;
    std::string *strs;
    Dimension** dims;

	nordata = new double [numDims];
    strs = new std::string [numDims];
    dims = new Dimension* [numDims];

    for(i = 0; i < numDims; i++)
    {
        dims[i] = dimSpace->GetDimension(i);
    }

	srand( (unsigned)time( NULL ));
    //for all data items

    char buf[25000];
    char *separator = ", \t\n";  // separators required by strtok() function
    
//        infile.getline(buf, 2);
        for (i = 0; i < numDataItems; i++)
        {
            infile.getline(buf, 25000); // read all the dimension names
            char *token = strtok(buf, separator);  // get the first token name

		    for (j = 0; j < numDims; j++)
		    {
                strs[j] = token;
                token = strtok( NULL, separator);

                nordata[j] = dims[j]->GetNorValue(strs[j]);
                if (dims[j]->GetType() == NORMINALDIMENSION)
                    strs[j] = "'" + strs[j] + "'";

            } 
		    // insert the data item into the data base
		    db->ExecuteUpdate(GetInsertTableString(dataTableName, numDims, strs));
        }

        db->Commit();

 	    //close files
	    infile.close();

        //*************Second scan - build similarity table

        //get reader to OKCFILE
        infile.open(OKCFileName.c_str());

	    infile >> numDims;
	    infile >> numDataItems;

        //a fake dimSpace is created into to run infile pointer to proper position
	    DimensionSpace *dimSpace1 = new DimensionSpace();
	    dimSpace1->LoadDimensionsFromFile(&infile, numDims);
        delete dimSpace1;


        //get writer to DBfile
        std::ofstream outfile(DBFileName.c_str());

        //write # dimensions, #data items to the DBFile
        outfile << numDims << " ";
        outfile << numDataItems << std::endl;

        //write dimensions of the all dimension space to the DBFile
	    dimSpace->WriteDimensionsToFile(&outfile);

        int numBins = 100;
	    //initate a similarity calculator
	    DBSimilarityCalculator* similarityCalculator = new DBSimilarityCalculator(numDims, numBins); 

  //      infile.getline(buf, 2);
        for (i = 0; i < numDataItems; i++)
        {
            infile.getline(buf, 25000); // read all the dimension names
            char *token = strtok(buf, separator);  // get the first token name

		    for (j = 0; j < numDims; j++)
		    {
                strs[j] = token;
                token = strtok( NULL, separator);

                nordata[j] = dims[j]->GetNorValue(strs[j]);
            } 
            // update similarity table in the similarity calculator
            similarityCalculator->Update(nordata);
        }

		for (j = 0; j < numDims; j++)
		{
            dims[j]->SetMappingUncertain(false);
        }

        delete [] nordata;
        delete [] strs;
        delete [] dims;
        //end

        //write db table information into db file
        outfile << dataTableName.c_str() << std::endl;

        //create and populate similarity table
        std::string sTableName = dataTableName + "_simi";
	    similarityCalculator->StoreSimilarityTableToDB(db, sTableName);
        //write similarity table information into db file
	    outfile << sTableName << " " << numBins;

	    //close files
	    infile.close();
	    outfile.close();

	    delete similarityCalculator;
	    delete dimSpace;

        return true;
    }
}
*/

std::string DBFlatDataSetLoader ::  GetCreateTableString(std::string dataTableName, DimensionSpace* dimSpace)
{
    std::ostringstream oss;

	oss << "create table " << dataTableName << "(";
	int numDims = dimSpace->GetSize();

	int i;
	for (i = 0; i < numDims; i++)
	{
        int type = dimSpace->GetDimension(i)->GetType();
        if (type == DOUBLE_DIMENSION|| type == BINARY_DIMENSION)
    		oss << "t" << i << " number(30,15),";
//    		oss << "t" << i << " DOUBLE(5,2),";
        else
    		oss << "t" << i << " char(40),";
	}
	oss << "randomColumn number(15,13), itemID number(20), description char(100))";
//	oss << "randomColumn DOUBLE(5,2), itemID INTEGER(20))";

	return oss.str();

}


/*for mysql
std::string DBFlatDataSetLoader ::  GetCreateTableString(std::string dataTableName, DimensionSpace* dimSpace)
{
    std::ostringstream oss;

	oss << "create table " << dataTableName << "(";
	int numDims = dimSpace->GetSize();

	int i;
	for (i = 0; i < numDims; i++)
	{
        if ((dimSpace->GetDimension(i))->GetType() == DOUBLEDIMENSION)
    		oss << "t" << i << " DOUBLE(30,15),";
//    		oss << "t" << i << " DOUBLE(5,2),";
        else
    		oss << "t" << i << " char(40),";
	}
	oss << "randomColumn DOUBLE(15,13), itemID INTEGER(20))";
//	oss << "randomColumn DOUBLE(5,2), itemID INTEGER(20))";

	return oss.str();

}
*/

std::string DBFlatDataSetLoader ::  GetDropTableString(std::string dataTableName)
{
    std::ostringstream oss;

	oss << "drop table " << dataTableName;
	return oss.str();

}

std::string DBFlatDataSetLoader ::  GetInsertTableString(std::string dataTableName, 
                                    int numDims, std::string* strs, int itemID, bool hasDescription, std::string itemDestr)
{
    std::ostringstream oss;

    itemDestr = "'" + itemDestr + "'";

	oss << "insert into " << dataTableName << " (";

	int i;
	for (i = 0; i < numDims; i++)
	{
		oss << "t" << i << ",";
	}
    if (hasDescription)
    	oss << "randomColumn,itemID,description) values (";
    else
    	oss << "randomColumn,itemID) values (";

	for (i = 0; i < numDims; i++)
	{
		oss << strs[i].c_str() << ",";
	}
    if (hasDescription)
        oss << (double)rand()/RAND_MAX << "," << itemID << "," << itemDestr << ")";
    else
        oss << (double)rand()/RAND_MAX << "," << itemID << ")";
	return oss.str();     

}

void DBFlatDataSetLoader::ReadFirstLine(std::ifstream *infile, int &numDims, int &numDataItems, bool &hasDescription, std::string &destr)
{
    char buf1[2500];
    char *separator = ", \t\n";  // separators required by strtok() function
    (*infile).getline(buf1, 2500); // read all the dimension names
    char *p1 = strtok(buf1, separator);  // get the first token name
    char *p2 = strtok(NULL, separator);  // get the first token name
    char *p3 = strtok(NULL, separator);  // get the first token name
    numDims = atoi(p1);
    numDataItems = atoi(p2);
    if (p3 != NULL)
    {
        hasDescription = true;
        destr = p3;
    }
    else 
        hasDescription = false;
}

void DBFlatDataSetLoader::WriteFirstLine(std::ofstream *outfile, int numDims, int numDataItems, bool hasDescription, std::string destr)
{
    //write # dimensions, #data items to the DBFile
    *outfile << numDims << " ";
    *outfile << numDataItems;
    if (hasDescription)
        *outfile << " " << destr << std::endl;
    else
        *outfile << std::endl;
}