#include <sstream>
#include "DBRowIterator.h"

DBRowIterator::DBRowIterator(DBManager* dbManager, std::string dataTableName, 
                             DimensionSpace* allDimSpace, DimensionSpace* dimSpace, 
                             int rrowNum, Query* query, bool ordering, int outputMode,
                             bool hasDescription, ISHash *descriptionHash, bool storeDescriptionInHash)
{
    this->dbManager = dbManager;
    this->rowNum = rrowNum;
    this->dataTableName = dataTableName;
    this->dimSpace = dimSpace;
    this->samplingMode = query->GetSamplingMode();
    this->isIncreasingOrder = query->IsIncreasingOrder();
    this->outputMode = outputMode;
    this->hasDescription = hasDescription;
    this->descriptionHash = descriptionHash;
    this->storeDescriptionInHash = storeDescriptionInHash;
    this->query = query;

    int approximateSize = query->GetApproximateSampleSize();
    if (rowNum <= approximateSize)
        this->sampling = false;
    else
        this->sampling = query->IsSampling();

    if (ordering == true)
        orderbyDimID = query->GetOrderByDimensionID();
    else 
        orderbyDimID = -1;

    dimNum = dimSpace->GetSize();

    
    if (dimNum != 0)
    {
        rowNum = GetRowNumber(approximateSize);
        dims = new Dimension *[dimNum];
        for (int i = 0; i < dimNum; i++)
        {
            dims[i] = dimSpace->GetDimension(i);
        }

        LoadData();
    }
    else
    {
        dims = NULL;
        re = NULL;
    }
    count = 0;

    docRow = new DocRow(dimSpace, false, false);
}	


DBRowIterator::~DBRowIterator()
{
    if (re != NULL)
        delete re;
    if (dims != NULL)
        delete [] dims;
    delete docRow;
}


bool DBRowIterator::LoadData()
{
    int i;

    std::ostringstream oss;
    oss << "select ";
    for (i = 0; i < dimNum; i ++)
    {
        oss << "t" << dimSpace->GetDimension(i)->GetDimensionID() << ", ";
    }
    oss << "itemID";
    if (hasDescription)
        oss << ", description"; 
    oss << " from " << dataTableName;

    if (sampling)
        oss << " ta " << "where ta.randomColumn > " << query->GetStart() << " and ta.randomColumn < " << query->GetEnd();
    
    if ( orderbyDimID >= 0 )
    {
        oss << " order by t" << orderbyDimID;
        if (!isIncreasingOrder)
            oss << " DESC";
    }

    re = dbManager->ExecuteQuery(oss.str());

    return true;
}

int DBRowIterator::GetRowNumber(int approximateSize)
{
    if (sampling == false)
        return rowNum;
    else //if (samplingMode == RANDOM_SAMPLING)
        return GetRandomSamplingRowNumber(approximateSize);
//    else if (samplingMode == BIASED_SAMPLING)
//        return GetBiasedSamplingColumnSize(approximateSize);
}

int DBRowIterator::GetRandomSamplingRowNumber(int approximateSize)
{
    int size;
    if (rowNum > approximateSize)
    {
        std::ostringstream oss;
        oss << "select count(*)" << " from " << dataTableName << " ta "
            << "where ta.randomColumn > " << query->GetStart() << " and ta.randomColumn < " << query->GetEnd();
        ResultSet* re = dbManager->ExecuteQuery(oss.str());
	    re->next();
        size = re->getInt(0);

        delete re;
    }
    else
        size = rowNum;

    return size;
}


void* DBRowIterator::GetNext()
{
    if (re->isDone())
        return NULL;

	re->next();
	for (int j = 0; j < dimNum; j++)
	{
        if (outputMode == ROW_NORVALUEONLY)
        {
           if (dims[j]->GetType() == DOUBLE_DIMENSION || dims[j]->GetType() == BINARY_DIMENSION )
            {
            //           docRow->SetNorValue(0, 0, dims[j]->GetDimensionID(), dims[j]->GetNorValue(re->getDouble(j)));
                docRow->SetNorValue(0, 0, j, dims[j]->GetNorValue(re->getDouble(j)));
            }
            else
            {
            //           docRow->SetNorValue(0, 0, dims[j]->GetDimensionID(), dims[j]->GetNorValue(re->getString(j)));
                docRow->SetNorValue(0, 0, j, dims[j]->GetNorValue(re->getString(j)));
            }
        }
        else
        {
           if (dims[j]->GetType() == DOUBLE_DIMENSION || dims[j]->GetType() == BINARY_DIMENSION )
            {
            //           docRow->SetNorValue(0, 0, dims[j]->GetDimensionID(), dims[j]->GetNorValue(re->getDouble(j)));
                docRow->SetCard(j, dims[j]->GetCard(re->getDouble(j)));
            }
            else
            {
            //           docRow->SetNorValue(0, 0, dims[j]->GetDimensionID(), dims[j]->GetNorValue(re->getString(j)));
                docRow->SetCard(j, dims[j]->GetCard(re->getString(j)));
            }
        }
    }
   // int id = re->getInt(j);
    docRow->SetItemID(re->getInt(j));
//    docRow->SetItemID(re->getDouble(j));
    if (hasDescription)
    {
        dstr = re->getString(j+1);
        docRow->SetItemDescription(dstr);
        if (storeDescriptionInHash)
            (*descriptionHash)[docRow->GetItemID()] = dstr;
    }
    return docRow;
}
    

bool DBRowIterator::IsDone()
{
    return re->isDone();
}

