#include <string>
#include <fstream>

#include "Dimension.h"

//constructor
Dimension::Dimension() 
{
	dimName = "";
    dimID = -1;

    rimFactor = 0;

    cardinality = 5;

    origMinString = "";
    origMaxString = "";

    minString = "";
    maxString = "";

    cardValues = NULL;

}
    
//deconstructor
Dimension::~Dimension() 
{
    if (cardValues != NULL)
        delete [] cardValues;
}

void Dimension::AddRim() 
{
    double rim = (max - min) * rimFactor;
    max = max + rim;
    min = min - rim;
}

double Dimension::GetRimFactor()
{
    return rimFactor;
}

void Dimension::SetRimFactor(double rimFac)
{
    this->rimFactor = rimFac;
}

bool Dimension::IsLeafDimension()
{
    if (dimID < 0)
        return false;
    else 
        return true;
}


void Dimension::SetDimensionID(int id)
{
    dimID = id;
}

int Dimension::GetDimensionID()
{
    return dimID;
}

std::string Dimension::GetOrigMaxString() 
{
    return origMaxString;
}
    
std::string Dimension::GetOrigMinString() 
{
    return origMinString;
}


/**
 * Sets the name of this dimension. 
 * @param name the name to be set
 */
void Dimension::SetDimensionName(std::string nameString) 
{
	this->dimName = nameString.substr(0, nameString.find_first_of('('));
}

/**
 * Gets the name of this dimension. 
 * @return the name of this dimension. 
 * It is not a copy, so you should not delete it.
 */
std::string Dimension::GetDimensionName() {
    return dimName;
}


void Dimension::SetCardinality(int cardinality) {
    if (type == DATE_DIMENSION)
        int i = 0;
    if (cardinality < 1)
        cardinality = 1;
    this->cardinality = cardinality;
    
    if (cardValues != NULL)
        delete [] cardValues;

    cardValues = new double [cardinality + 1];

    cardValues[0] = rimFactor;
    double interval = (1.-2*rimFactor)/cardinality;
    for (int i = 1; i < cardinality; i++)
    {
        cardValues[i] = cardValues[i-1] + interval;
    }
    cardValues[cardinality] = 1.;
    cardValues[0] = 0;
}

/**
 * Gets the cardinality of this dimension.  
 * @return the cardinality of this dimension
 */
int Dimension::GetCardinality() {
    return cardinality;
}

double Dimension::GetNorValue(std::string valueString)
{
	return 0;
}

double Dimension::GetNorValue(Data data)
{
	return 0;
}

void Dimension::GetNorValue(Data* data, double &norValue)
{
    norValue = GetNorValue(data[dimID]);
}

void Dimension::GetNorValueWithMinMax(Data* data, double &norValue, double &norMin, double &norMax)
{
}

void Dimension::GetNorValueWithMinMax(double value, double &norValue, double &norMin, double &norMax)
{
}

double Dimension::GetNorValue(double value)
{
    if (min == max)
    {
        if (value == min)
            return 1;
        else
            return 0;
    }
    
    double v = (value - min) / (max - min);

    if (v < 0)
        v = 0;
    else if (v > 1)
        v = 1;

    return v;
}

int Dimension::GetType()
{
    return type;
}

void Dimension::GetLabels(std::string* &labels, double* &values, int &labelNum)
{
}

double Dimension::GetValue(std::string valueString)
{
    return 0;
}


void Dimension::WriteMetaInformation(std::ofstream *outfile)
{
    (*outfile) << GetOrigMinString() << " "
		    << GetOrigMaxString() << " "
            << GetCardinality() << std::endl;
}

int Dimension::GetCard(Data* data)
{
    double norValue;
    GetNorValue(data, norValue);
    return GetCardFromNorValue(norValue);
}

int Dimension::GetCard(std::string valueString)
{
    double norValue;
    norValue = GetNorValue(valueString);
    return GetCardFromNorValue(norValue);
}

int Dimension::GetCard(double value)
{
    double norValue;
    norValue = GetNorValue(value);
    return GetCardFromNorValue(norValue);
}


//////////protected functions
int Dimension::GetCardFromNorValue(double norValue)
{
    int i = 1;
    while (i < cardinality && norValue >= cardValues[i])
        i ++;

    return (i-1); 
}

//////////protected functions