#ifndef _Dimension_H
#define _Dimension_H

#include <string>

#include "DataBuffer.h"

#define DOUBLE_DIMENSION 1
#define NORMINAL_DIMENSION 2
#define DATE_DIMENSION 3
#define BINARY_DIMENSION 5

class Dimension 
{
// Variables declaration
protected: 
    int type;

    double min, max;

	std::string dimName;
    int cardinality;

    //each dimension has a unique dimID. 
    int dimID;
    double rimFactor;

    std::string origMaxString, origMinString;
    std::string maxString, minString;
    bool mappingUncertain;

    std::string* groupValues;
    int* groupSizes;
    int groupNum;

    double* cardValues;


// End of variables declaration

public: 
    //constructor
    Dimension();

    //deconstructor
    virtual ~Dimension();

    // could be deleted - 
    bool IsLeafDimension();

    //dimensionID: a unique id for a dimension
    void SetDimensionID(int id);
    int GetDimensionID();

    //dimension name: the name of the dimension
    void SetDimensionName(std::string nameString);
	std::string GetDimensionName(); 
    
    //cardinality is used for discreting the values
    int GetCardinality();    
    void SetCardinality(int cardinality);
    
    //card: the position (from 0 to cardinality) of the interval the value of this dimension of this data falling in
    int GetCard(Data* data);
    int GetCard(std::string valueString);
    int GetCard(double value);
    
    //add rim so that the axis can be a little bit longer than the exact length
    void AddRim(); 
    double GetRimFactor();
    void SetRimFactor(double rimFac);

    //origmaxstring and origminstring: the string to be shown in the axes for max and min values before adding rim 
    std::string GetOrigMaxString(); 
    std::string GetOrigMinString(); 

    double GetNorValue(double value);

    void GetNorValue(Data* data, double &norValue);

    //return the type of the dimension: double dimenison, nominal dimension, or other types of dimenisions
    int GetType();

    //if is mapping uncertain, then you could not get proper normalized value from this dimension
    void SetMappingUncertain(bool unc) {mappingUncertain = unc;}
    bool IsMappingUncertain() {return mappingUncertain;}

    virtual double GetNorValue(std::string valueString);
    virtual double GetValue(std::string valueString);
    virtual double GetNorValue(Data data);
    //norMin and norMax are meaningful only for aggregated dimensions
    //for other kind of dimensions they should be omited. 
    virtual void GetNorValueWithMinMax(Data* data, double &norValue, double &norMin, double &norMax);
    virtual void GetNorValueWithMinMax(double value, double &norValue, double &norMin, double &norMax);


    virtual void GetLabels(std::string* &labels, double* &values, int &labelNum);

    virtual void WriteMetaInformation(std::ofstream *outfile);
    virtual void ReadMetaInformation(std::ifstream *infile) {}


    virtual void GetGroups(std::string* &groupValues, int* &groupSizes, int &groupNum) {}
    virtual void SetGroups(std::string* groupValues, int* groupSizes, int groupNum) {}

    virtual void ChangeMinMax(std::string valueString) {}

  
protected:
    int GetCardFromNorValue(double norValue);


};

#endif