#include <string>
#include <fstream>
#include <iostream>
#include <stdio.h>
#include "DimensionSpace.h"
#include "Dimension.h"
#include "DoubleDimension.h"
#include "BinaryDimension.h"
#include "NorminalDimension.h"
#include "DateDimension.h"
#include "../Basic/LinkListIterator.h"

DimensionSpace::DimensionSpace()
{
    dims = new CLinkList();
    spaceName = "";
    colorDim = NULL;
}

DimensionSpace::~DimensionSpace()
{
/*    int len = dims->GetLength();
    for (int i = 0; i < len; i++)
    {
        delete (Dimension *)(dims->GetElement(i));
    }
*/
    delete dims;
}

void DimensionSpace::AddRims(double rim)
{
    int len = dims->GetLength();
    Dimension *dim;
    for (int i = 0; i < len; i++)
    {
        dim = (Dimension *)(dims->GetElement(i));
//        dim->SetRimFactor(rim);
//        dim->AddRim();
    }
}

void DimensionSpace::AddDimension(Dimension* dim)
{
    dims->AddNodeAtEnd(dim);
}

void DimensionSpace::AddDimensionAtEnd(Dimension* dim)
{
    dims->AddNodeAtEnd(dim);
}

void DimensionSpace::RemoveDimensionAtEnd()
{
    dims->DeleteNodeAtEnd();
}

int DimensionSpace::GetSize()
{
    return dims->GetLength();
}

Dimension* DimensionSpace::GetColorDimension()
{
    return colorDim;
}

void DimensionSpace::SetColorDimension(Dimension* dim)
{
    colorDim = dim;
}

Dimension* DimensionSpace::GetDimension(int n)
{
    return (Dimension *)(dims->GetElement(n));
}

double DimensionSpace::GetDimensionDistance(int dimID1, int dimID2)
{
    return 1; //to be done
}

//copy the dimension composition of another dimension space
//para space - the dimension space to be copied.
//notice - the name of this space will not be changed.
//         the original dimension in this space will be removed.
/*void DimensionSpace::Copy(DimensionSpace* space)
{
    delete dims;
    dims = new CLinkList();
    int len = space->GetSize();
    for (int i = 0; i < len; i++)
    {
        AddDim(space->GetDim(i));
    }
}
*/
//set the name of the dimension space
//para nameString - the name to be set
//notice a copy of nameString will be made.
/*void DimensionSpace::SetSpaceName(std::string nameString)
{
	this->spaceName = nameString;
}*/

//get the name of the dimension space
//return - the name of the space
//notice it is not a copy, so don't delete
/*std::string DimensionSpace::GetSpaceName()
{
    return spaceName;
}
*/

LinkListIterator* DimensionSpace ::IterateDimensions()
{
	return new LinkListIterator(dims);
}

Dimension* DimensionSpace :: GetDimension(std::string dimName)
{
    if (dimName.compare("") == 0)
        return NULL;

    Dimension *temp = NULL;
    LinkListIterator* it = IterateDimensions();
	while (!it->IsDone())
	{
		Dimension* dim = (Dimension *)(it->GetNext());
        if (dim->GetDimensionName().compare(dimName) == 0)
        {
            temp = dim;
            break;
        }
	}
	delete it;

    return temp;
}

Dimension* DimensionSpace :: GetDimensionByID(int ID)
{
    Dimension *temp = NULL;
    LinkListIterator* it = IterateDimensions();
	while (!it->IsDone())
	{
		Dimension* dim = (Dimension *)(it->GetNext());
        if (dim->GetDimensionID() == ID)
        {
            temp = dim;
            break;
        }
	}
	delete it;

    return temp;
}


bool DimensionSpace :: LoadDimensionsFromFile(std::ifstream *infile, int numDims)
{
	int i, index1, index2, index3, index4;

	if (numDims < 1)
		return false;

    Dimension* dim;
	for (i = 0; i < numDims; i++)
	{
		char buf[300];
		(*infile) >> buf;
		std::string str;
		str = buf;

        //one or 0 of them will != npos
        index1 = str.find("N:", 0);
        index2 = str.find("T:", 0);
        index3 = str.find("B:", 0);
        index4 = str.find("D:", 0);
        if (index1 != std::string::npos)
        {
            dim = new NorminalDimension();
		    char buf1[300];
		    (*infile) >> buf1;
		    str = buf1;
        }
        else if (index2 != std::string::npos)
        {
            dim = new DateDimension();
		    char buf1[300];
		    (*infile) >> buf1;
		    str = buf1;
        }
        else if (index3 != std::string::npos)
        {
            dim = new BinaryDimension();
		    char buf1[300];
		    (*infile) >> buf1;
		    str = buf1;
        }
        else
        {
            dim = new DoubleDimension();
        }
        
        dim->SetDimensionName(str);
        dim->SetDimensionID(i);
		AddDimension(dim);
    }
	
    char buf[2000];
    mappingUncertain = false;
    (*infile).getline(buf, 2000); // read all the dimension names

	LinkListIterator* it = IterateDimensions();
	while (!it->IsDone())
	{
		Dimension* dim = (Dimension *)(it->GetNext());
        dim->ReadMetaInformation(infile);
        if (dim->IsMappingUncertain())
            mappingUncertain = true;
    } 
	delete it;

	return true;
}

bool DimensionSpace :: IsMappingUncertain()
{
    return mappingUncertain;
}

void DimensionSpace :: WriteDimensionsToFile(std::ofstream *outfile)
{
	LinkListIterator* it = IterateDimensions();
	while (!it->IsDone())
	{
		Dimension* dim = (Dimension *)(it->GetNext());
        if (dim->GetType() == NORMINAL_DIMENSION)
            (*outfile) << "N: " << dim->GetDimensionName() << std::endl; 
        else if (dim->GetType() == DATE_DIMENSION)
            (*outfile) << "T: " << dim->GetDimensionName() << std::endl; 
        else if (dim->GetType() == BINARY_DIMENSION)
            (*outfile) << "B: " << dim->GetDimensionName() << std::endl; 
        else
            (*outfile) << dim->GetDimensionName() << std::endl; 
	}
	delete it;

	it = IterateDimensions();
	while (!it->IsDone())
	{
		Dimension* dim = (Dimension *)(it->GetNext());
		dim->WriteMetaInformation(outfile);
/*		Dimension* dim = (Dimension *)(it->GetNext());
		(*outfile) << dim->GetOrigMinString() << " "
			    << dim->GetOrigMaxString() << " "
                << dim->GetCardinality() << std::endl;*/
	}
	delete it;

	return;
}


/*
std::string DimensionSpace :: GetCreateTableString(CDBManager* db, std::string dataTableName)
{
	std::string createString = "create table " + dataTableName + "( ";
	
	int i;
	for (i = 0; i < numDims; i++)
	{
		std::string dname = "t" + i;
		createString += dname + " float,"
	}
	createString += );

	return createString;
}
*/

/*
void DimensionSpace :: GetNorValues(std::string *valueStrings, double *norValues)
{
	CDimensionIterator* it = IterateDimensions();
	CDimension *dim;
	int i = 0; 
	while (!it->IsDone())
	{
		dim = (CDimension *)(it->GetDimension());
		norValues[i] = dim->GetNorValue(valueStrings[i]);
		i++;
	}
}

int DimensionSpace :: GetDimensionality()
{
	CDimensionIterator* it = IterateDimensions();
	int i = 0; 
	while (!it->IsDone())
	{
		i++;
	}
	return i;
}

*/