#ifndef _DIMENSIONSPACE_H
#define _DIMENSIONSPACE_H

#include <string>
#include <fstream>

#include "../Basic/CLinkList.h"
#include "../Basic/LinkListIterator.h"
#include "Dimension.h"
#include "DimensionIterator.h"

class DimensionSpace
{
protected:
    CLinkList* dims;
	std::string spaceName;
    Dimension* colorDim;
    bool mappingUncertain;

public:
	DimensionSpace();
	~DimensionSpace();

    void AddRims(double rim);
    void SetColorDimension(Dimension* dim);
    Dimension* GetColorDimension();

    //add a dimension into the dimension space
    //para: dim - the dimension to be added
    //      it will be the last dimension in the space.
    //Notice: Dim is not copied.
    int GetSize();
    Dimension* GetDimension(int n);
    double GetDimensionDistance(int dimID1, int dimID2);

    void AddDimension(Dimension* dimension);
    void WriteDimensionsToFile(std::ofstream *outfile);

    bool IsMappingUncertain();

    
    //get the total number of dimensions in the dimension space
    //return: the total number of dimensions in the dimension space
//    int GetDimensionality();

    //copy the dimension composition of another dimension space
    //para space - the dimension space to be copied.
    //notice - the name of this space will not be changed.
    //         the original dimension in this space will be removed.
//    void CopyDimensions(DimensionSpace* space);

    //set the name of the dimension space
    //para nameString - the name to be set
    //notice a copy of nameString will be made.
//    void SetSpaceName(std::std::string nameString);

    //Get the name of the dimension space
    //para nameString - the char array to store the name
    //para n - the length of the name + 1.
//	std::std::string GetSpaceName();

    //////// functions related to single dimensions //////////

    // Prepares to input dimensions
    // for example, if you want to access all dimension names,
    // you make the following calls: 
    // space->IterateDimensions();
    // while (space->NextDimension()) {space->GetDimensionName()}.
//    void IterateDimensions();

    // Moves to the next dimension in the dimension space
	// @return true if the cursor is in the dimension space
//	bool NextDimension();

    // Gets the pointer to the current dimension
    LinkListIterator* IterateDimensions();

    // Gets the position of the current data item in a glyph display in a 
    // ([0,0], [1,1]) space
    // @para pos - an array of length 2, 
//    void GetParallelCoordinatesAxisPosition(ARR<double> &pos);

    // Gets color of the current dimension
    // @ return color of the current dimension
//	RGBt GetColor();

    bool LoadDimensionsFromFile(std::ifstream *infile, int numDims);
//	void WriteDimensionsToFile(ofstream *outfile);
//    void GetNorValues(std::string *valueStrings, double *norValues);
//	int GetDimensionality();
    
    Dimension* GetDimension(std::string dimName);
    Dimension* GetDimensionByID(int ID);

    void RemoveDimensionAtEnd();
    void AddDimensionAtEnd(Dimension* dim);

};

#endif

