#include <math.h>
#include "InMemoryFlatColumnIterator.h"
#include "../DataBuffer.h"


InMemoryFlatColumnIterator::InMemoryFlatColumnIterator(
	DataBuffer* dataBuffer, DimensionSpace* allDimSpace, DimensionSpace* dimSpace, Query *query)
{
	this->dataBuffer = dataBuffer;
	this->dimSpace = dimSpace;
    this->query = query;

    queryMode = query->GetQueryMode();
    groupingMode = query->GetGroupingMode();
    isMaskable = query->IsMaskable();

    rowNum= dataBuffer->GetRowNum();
    docColumn = new DocColumn();
    docColumn->SetSize(rowNum);

    dimNum = dimSpace->GetSize();

    if (queryMode == SCATTER_QUERY)
    {
        docColumn->SetMode(DUAL);
        Dimension* scatterDim = allDimSpace->GetDimensionByID(query->GetScatterDimensionID());

        double value;
        for (int i = 0; i < rowNum; i++)
        {
            Data* data = dataBuffer->GetRow(i);
            scatterDim->GetNorValue(data, value);
            docColumn->SetBaseData(i, value);
        }
    }
    else if (queryMode == XY_QUERY)
    {
        docColumn->SetMode(TRIPLE);
        Dimension* xDim = allDimSpace->GetDimensionByID(query->GetXDimensionID());
        Dimension* yDim = allDimSpace->GetDimensionByID(query->GetYDimensionID());

        double value;
        for (int i = 0; i < rowNum; i++)
        {
            Data* data = dataBuffer->GetRow(i);
            xDim->GetNorValue(data, value);
            docColumn->SetXData(i, value);
            yDim->GetNorValue(data, value);
            docColumn->SetYData(i, value);
        }
    }

    if (isMaskable)
    {
        docColumn->SetMaskMode(true);
        Dimension* maskDim = allDimSpace->GetDimensionByID(query->GetGroupByDimensionID());
        double value1 = maskDim->GetNorValue(query->GetGroupValue());

        double value;
        for (int i = 0; i < rowNum; i++)
        {
            Data* data = dataBuffer->GetRow(i);
            maskDim->GetNorValue(data, value);
            if (fabs(value-value1) < 1e-5)
                docColumn->SetMaskData(i, 0);
            else
                docColumn->SetMaskData(i, 1);
        }
    }
    
    count = 0;
}	

InMemoryFlatColumnIterator:: ~InMemoryFlatColumnIterator()
{
	delete docColumn;
}

void* InMemoryFlatColumnIterator::GetNext()
{
    if (count == dimNum)
        return NULL;

    Dimension* dim = dimSpace->GetDimension(count);
	docColumn->SetDimensionID(dim->GetDimensionID());

    double value;
    for (int i = 0; i < rowNum; i++)
    {
        Data* data = dataBuffer->GetRow(i);
        dim->GetNorValue(data, value);
        docColumn->SetData(i, value); 
    }

    count ++;
	return docColumn;
}


bool InMemoryFlatColumnIterator::IsDone()
{
	if (count >= dimNum)
		return true;
	else
		return false;
}


