#include "InMemoryFlatDataSet.h"
#include "InMemoryFlatColumnIterator.h"
#include "DataStructure/NorminalDimension.h"

InMemoryFlatDataSet::InMemoryFlatDataSet()
{
    dataBuffer = NULL;
    descriptionArray = NULL;
    hasDescription = false;
}


InMemoryFlatDataSet::~InMemoryFlatDataSet()
{
    if (dataBuffer != NULL)
        delete dataBuffer;
    if (descriptionArray != NULL)
        delete [] descriptionArray;
}

/*
InMemoryFlatDataIterator<FlatDataEntry*>* InMemoryFlatDataSet::LoadData()
{
	return (new InMemoryFlatDataIterator(dataBuffer));
}*/

/*
bool InMemoryFlatDataSet::ReadData(std::ifstream* infile, int dimNum, int itemNum)
{
    dataBuffer = new DataBuffer(dimNum, itemNum);
    double value;

    int size = dimNum * itemNum;
    
    dataBuffer->ResetInputCounter(0);
    for (int i = 0; i < size; i ++)
    {
        (*infile) >> value;
        dataBuffer->GetInput(value);
    }
    return true;
}
*/


bool InMemoryFlatDataSet::ReadData(std::ifstream* infile, DimensionSpace* dimSpace, int itemNum, bool hasDescription)
{
    int i, j;
    allDimSpace = dimSpace;
    int dimNum = dimSpace->GetSize();

    dataBuffer = new DataBuffer(dimNum, itemNum);
    dataBuffer->ResetInputCounter(0);
    
    Dimension** dims = new Dimension * [dimNum];

    for (i = 0; i < dimNum; i++)
    {
        dims[i] = dimSpace->GetDimension(i);
    }

    double value;
    void* pointer;
    NorminalDimension *dim;

    this->hasDescription = hasDescription;
    if (hasDescription)
        descriptionArray = new std::string [itemNum];
    for (i = 0; i < itemNum; i++)
    {
        (*infile) >> descriptionArray[i];
        for (j = 0; j < dimNum; j++)
        {
           if (dims[j]->GetType() == DOUBLE_DIMENSION || dims[j]->GetType() == BINARY_DIMENSION )
            {
                (*infile) >> value;
                dataBuffer->GetInput(value);
            }
            else
            {
                dim = (NorminalDimension *)dims[j];
                char buf[300];
		        std::string str;
		        
		        (*infile) >> buf;
		        str = buf;
                pointer = dim->GetPointer(str);
                dataBuffer->GetInput(pointer);
            }
        }
    }
    return true;
}


Iterator* InMemoryFlatDataSet::LoadRows(DimensionSpace* dimSpace, Query* query, bool dimCluster, bool ordering, int outputMode, bool loadDescriptions, bool storeDescriptions)
{
	return (new InMemoryFlatRowIterator(dataBuffer, dimSpace, dimCluster, outputMode));
}

Iterator* InMemoryFlatDataSet::LoadColumns(DimensionSpace* dimSpace, Query *query)
{
	return (new InMemoryFlatColumnIterator(dataBuffer, allDimSpace, dimSpace, query));
}

std::string InMemoryFlatDataSet::GetItemDescription(int itemID)
{
    if (hasDescription)
        return descriptionArray[itemID];
    else
        return "";
}

/*

InMemoryFlatDataIterator<FlatDataEntry*>* InMemoryFlatDataSet::LoadUnbrushedData(bool extents)
{
	return (new InMemoryFlatDataIterator(
        marks, -1, dataBuffer, coverages, dimManager, extents));
}


InMemoryFlatDataIterator<FlatDataEntry*>* InMemoryFlatDataSet::LoadBrushedData(int brushOperID, bool extents)
{
	return (new InMemoryFlatDataIterator(
        marks, brushOperID, dataBuffer, coverages, dimManager, extents));
}


void InMemoryFlatDataSet::ResetMarksAndCoverage()
{
    int i, j, brushOperNum; 
    Data* data;
    BrushOperation* brushOper;
    double coverage;

    brushOperNum = brushMananger->GetBrushOperationNumber();
    for (i = 0; i < numOfRows; i++)
    {
        marks[i] = -1;
        coverages[i] = 0;
        data = GetRow(i);
        if (dimManager->GetCurrentDimSpace()->ExceedsBoundary(data))
        {
            marks[i] = -2;
        }
        j = 0;
        while (marks[i] == -1 && j < brushOperNum)
        {
            brushOper = brushManager->GetBrushOperation(j);
            coverage = brushOper->GetCoverage(data);
            if (coverage > 1e10)
            {
                marks[i] = j;
                coverages[i] = coverage;
            }
        }
    }
}

int InMemoryFlatDataSet::BrushOperCompAverage(int brushOperID, BrushOperation* brushOper, ARR<double> &averages)
{
    int count = 0;
    int i, j; 
    Data* data;
    double* norValues;

    for (i = 0; i < dimNum; i++)
        averages[i] = 0;

    bool covered;
    for (i = 0; i < numOfRows; i++)
    {
        covered = false;
        if (marks[i] == brushOperID)
            covered = true;
        else if (marks[i] >= 0 && marks[i] < brushOperID)
        {
            data = GetRow(i);
            if (brushOper->IsPointCovered(data))
                covered = true;
        }
        
        if (covered == true)
        {
            count ++;
      		dimManager->GetNormalizedData(data, norValues);
            for (j = 0; j < dimNum; j <0)
            {
                averages[j] += norValues;
            }
        }
    }

    for (i = 0; i < dimNum; i++)
        averages[i] /= count;

    return count;
}

*/