#include "Query.h"

Query::Query()
{
    queryMode = SINGLE_QUERY;

    orderByDimID = -1;
    scatterDimID = -1;
    xDimID = -1;
    yDimID = -1;
    groupByDimID = -1;
    groupValue = "";

    timeDimID = -1;

    isSampling = false;
    samplingMode = RANDOM_SAMPLING;
    approximateSize = 500;

    isIncreasingOrder = false;
}

Query:: ~Query()
{
}

void Query::Reset()
{
    queryMode = SINGLE_QUERY;

    orderByDimID = -1;
    scatterDimID = -1;
    xDimID = -1;
    yDimID = -1;
    groupByDimID = -1;
    groupValue = "";

    timeDimID = -1;
}

int Query::GetQueryMode()
{
    if (queryMode == SCATTER_QUERY)
    {
        if (scatterDimID == -1)
            return SINGLE_QUERY;
    }
    else if (queryMode == XY_QUERY)
    {
        if (xDimID == -1 || yDimID == -1)
            return SINGLE_QUERY;
    }

    return queryMode;
}

int Query::GetGroupingMode()
{
    if (groupByDimID == -1 || groupValue == "")
        return NONE_GROUP;
    else
        return groupingMode;
}

bool Query::IsMaskable()
{
    if (groupByDimID == -1 || groupValue == "")
        return false;

    if (groupingMode == NONE_GROUP || groupingMode == MASK_GROUP)
        return true;
    else
        return false;
}


void Query::SetApproximateSampleSize(int size)
{
    approximateSize = size;

    double port;
    if (rowNum == 0)
        port = 0.5;
    else
        port = ((double)approximateSize)/rowNum;
    
    start = (double)rand()/RAND_MAX * (1-port);
    end = start + port;
}

void Query::SetRowNum(int rowNum)
{
    this->rowNum = rowNum;

    double port = ((double)approximateSize)/rowNum;
    start = (double)rand()/RAND_MAX * (1-port);
    end = start + port;
}

void Query::Resample()
{
    double port = ((double)approximateSize)/rowNum;
    start = (double)rand()/RAND_MAX * (1-port);
    end = start + port;
}

