#include "SBBrush.h"

SBBrush::SBBrush()
{
    unbrushedRadius = 0.5;
    brushedRadius = 0.5;

    brushType = POSITION_BRUSH;
    semantic = BRUSH_ALL;
    
    brushedRegionL = 0.25;
    brushedRegionR = 0.75;

    dataset = NULL;

}

SBBrush:: ~SBBrush()
{
}

void SBBrush::SetHierDataSet(HierDataSet* dataset)
{
    this->dataset = dataset;
    dataset->MarkBrushedClusters(this);
    double rootRadius = dataset->GetRootSqrtRadius();
    dataset->MarkContourClusters(rootRadius * unbrushedRadius, IS_CONTOUR_UNBRUSHED);
    dataset->MarkContourClusters(rootRadius * brushedRadius, IS_CONTOUR_BRUSHED);
}

void SBBrush::SetBrushedRegionL(double L)
{
    brushedRegionL = L;
    if (dataset != NULL)
    {
        dataset->MarkBrushedClusters(this);
    }
}

void SBBrush::SetBrushedRegionR(double R)
{
    brushedRegionR = R;
    if (dataset != NULL)
    {
        dataset->MarkBrushedClusters(this);
    }
}

void SBBrush::ShiftBrushedRegion(double shift)
{
    brushedRegionL += shift;
    brushedRegionR += shift;
    if (dataset != NULL)
    {
        dataset->MarkBrushedClusters(this);
    }
}

void SBBrush::SetUnbrushedRadius(double radius)
{
    unbrushedRadius = radius;
    double rootRadius = dataset->GetRootSqrtRadius();
    dataset->MarkContourClusters(rootRadius * unbrushedRadius, IS_CONTOUR_UNBRUSHED);
}

void SBBrush::SetBrushedRadius(double radius)
{
    brushedRadius = radius;
    double rootRadius = dataset->GetRootSqrtRadius();
    dataset->MarkContourClusters(rootRadius * brushedRadius, IS_CONTOUR_BRUSHED);
}

/*
void SBBrush::SetBrushType()
{
}
*/

bool SBBrush::IsClusterCovered(Cluster* cluster)
{
    bool brushed;
    if (brushType == EXTENT_BRUSH)
    {
/*        brushed = true;
        for (int k = 0; k < dimNum; k++) 
        {
            if (node->GetMean(k) < min[k] || node->GetMean[k] > max[k]) 
            {
                brushed = false;
                break;
            }
        }
*/    }
    else if (brushType == POSITION_BRUSH)
    {
        double pos = cluster->GetNorPosition();
        if (pos >= brushedRegionL && pos <= brushedRegionR)
            brushed = true;
        else
            brushed = false;
    }
    return brushed;
}

int SBBrush::GetSemantic()
{
    return semantic;
}

void SBBrush::SetSemantic(int semantic)
{
    this->semantic = semantic;
    if (dataset != NULL)
    {
        dataset->MarkBrushedClusters(this);
    }

}


