#include "TimeHistogramIterator.h"
#include "DocRow.h"
#include "../Interface/UserNotifier.h"


TimeHistogramIterator::TimeHistogramIterator(Iterator* rowIter, DimensionSpace* dimSpace, Dimension* timeDim)
{
    this->dimSpace = dimSpace;

//    this->intervalValues = timeDim->GetHistogramIntervalValues();
//    this->intervalNum = timeDim->GetHistogramIntervalNum();
    this->intervalNum = timeDim->GetCardinality();

    docTimeHistogram = NULL;
    histograms = NULL;
    itemHashs = NULL;
    
    //the last dimension is the time dimension for building timehistogram
    dimNum = dimSpace->GetSize() - 1;

    if (dimNum == 0)
        return;

    int i, j;

    histograms = new int *[dimNum];
    for (i = 0; i < dimNum; i++)
    {
        histograms[i] = new int [intervalNum];
        for (j = 0; j < intervalNum; j++)
            histograms[i][j] = 0;
    }

    itemHashs = new ISHash * [dimNum];

    for (i = 0; i < dimNum; i++)
        itemHashs[i] = new ISHash;

    DocRow* row;
    int index;
    int totalNum = 0;
    while (!rowIter->IsDone())
    {
        totalNum ++;
        row = (DocRow*)(rowIter->GetNext());
//        index = FindIndex(row->GetNorValue(0, 0, dimNum)); 
        index = row->GetCard(dimNum); 
        if (index > -1)
        {
            for (i = 0; i < dimNum; i++)
            {
                if (row->GetCard(i) > 0)
                {
                    histograms[i][index] ++;
                    AddRepresentativeItems(itemHashs[i], row->GetItemID(), row->GetItemDescription());
                }
            }
        }
    }

    int maxNum = histograms[0][0];
    for (i = 0; i < dimNum; i++)
    {
        for (j = 0; j < intervalNum; j++)
        {
            if (maxNum < histograms[i][j])
                maxNum = histograms[i][j];
        }
    }

    count = 0;
    docTimeHistogram = new DocTimeHistogram(dimSpace);
    docTimeHistogram->SetBinNum(intervalNum);
//    docTimeHistogram->SetMaxNum(maxNum);
    docTimeHistogram->SetMaxNum(totalNum);
   
}	

void TimeHistogramIterator::AddRepresentativeItems(ISHash *hash, int id, std::string destr)
{
    if (hash->size() < 16)
        (*hash)[id] = destr;
}


TimeHistogramIterator::~TimeHistogramIterator()
{
    if (histograms != NULL)
    {
        for (int i = 0; i < dimNum; i++)
            delete [] histograms[i];
        delete [] histograms;
    }

    if (docTimeHistogram != NULL)
        delete docTimeHistogram;

    if (itemHashs != NULL)
        delete [] itemHashs;
}


void* TimeHistogramIterator::GetNext()
{
    if (count < dimNum)
    {
        docTimeHistogram->SetHistogram(histograms[count]);
        int dimID = dimSpace->GetDimension(count)->GetDimensionID();
	    docTimeHistogram->SetDimensionID(dimID);
        
        int maxNum = histograms[count][0];
        for (int j = 0; j < intervalNum; j++)
        {
            if (maxNum < histograms[count][j])
                maxNum = histograms[count][j];
        }
        
        docTimeHistogram->SetMaxNum(maxNum);
        docTimeHistogram->SetItemHash(itemHashs[count]);

        count ++;
        return docTimeHistogram;
    }
    else
        return NULL;
}
    

bool TimeHistogramIterator::IsDone()
{
    if (count >= dimNum)
		return true;
	else
		return false;
}

int TimeHistogramIterator::FindIndex(double norValue)
{
    int i = -1;
    if (i < intervalValues[0] || i > intervalValues[intervalNum])
        return i;

    i = 1; 
    while (i <= intervalNum && norValue >= intervalValues[i])
        i ++;

    return (i-1); 
}