#include <wx/filedlg.h>
#include <wx/dir.h>
#include "VideoProcessFrame.h"
#include "KeywordProcess.h"

BEGIN_EVENT_TABLE(VideoProcessFrame, wxFrame)
    EVT_MENU(MENU_OPENFREQFILE, VideoProcessFrame::OpenFreqFile)
    EVT_MENU(MENU_OPENCCFILES, VideoProcessFrame::OpenCCFiles)
END_EVENT_TABLE()

VideoProcessFrame::VideoProcessFrame(const wxChar *title, int xpos, int ypos, int width, int height)
        :wxFrame((wxFrame *) NULL, -1, title, wxPoint(xpos, ypos), wxSize(width, height))
{
	//set menu bar
	menuBar = new wxMenuBar;
	SetMenuBar(menuBar);

    wxMenu *fileMenu = new wxMenu;
	fileMenu->Append(MENU_OPENFREQFILE,  "Open freq file");
	fileMenu->Append(MENU_OPENCCFILES,  "Open cc files");
	menuBar->Insert(0, fileMenu, "&File");
}

VideoProcessFrame::~VideoProcessFrame()
{
}

void VideoProcessFrame::OpenFreqFile(wxCommandEvent& event)
{
    wxFileDialog* dlg = new wxFileDialog(this);
    dlg->SetStyle(wxOPEN | wxMULTIPLE);
    dlg->SetWildcard("freq log files(*.freq.log)|*.freq.log");

    if (dlg->ShowModal() == wxID_CANCEL)
    {
        delete dlg;
        return;
    }

    wxArrayString paths;
    dlg->GetPaths(paths);
    delete dlg;

    int num = paths.Count();
    KeywordProcess *pro = new KeywordProcess();
    for (int i = 0; i < num; i++)
    {
        std::string fileName = paths[i].c_str();
        pro->AddFreqFile(fileName);
    }
    pro->Output();
    delete pro;

    return;
}

void VideoProcessFrame::OpenCCFiles(wxCommandEvent& event)
{
    wxDirDialog* dlg = new wxDirDialog(this, "Choose a directory containing cc files", "c:\\yangjing\\Fan\\video\\Multichannel");
    if (dlg->ShowModal() == wxID_CANCEL)
    {
        delete dlg;
        return;
    }

    wxString path = dlg->GetPath();
    wxDir dir(path);
    
    if ( !dir.IsOpened() )
    {
        return;
    }

    wxString filename;
    wxArrayString filenames;
    bool cont = dir.GetFirst(&filename, "*.cc", wxDIR_FILES);
    while ( cont )
    {
        filenames.Add(filename.c_str());
        cont = dir.GetNext(&filename);
    }
    delete dlg;

    KeywordProcess *pro = new KeywordProcess();
    pro->MakeOKCFile(path, &filenames);
    delete pro;
    return;
}