#ifndef _DEMO_DRAWER_H
#define _DEMO_DRAWER_H

#include "Drawer.h"
#include "../Basic/CLinkList.h"
#include "../Basic/ARR.h"
#include "../Basic/Vec2.h"

struct Line {
    double x1;
    double y1;
    double x2;
    double y2;
};

struct Circle {
    double centerx;
    double centery;
    double radius;
    double offset;
};

struct ArcType {
    double centerx;
    double centery;
    double radius;
    double start_angle;
    double sweep_angle;
};

struct PartialDisk {
    double centerx;
    double centery;
    double inner_radius;
    double outer_radius;
    double start_angle;
    double sweep_angle;
};

struct Lines {
    ARR<Vec2> pts;
};

struct RectangleType {
    double x;
    double y;
    double width;
    double ht;
};

struct Rectangle2 {
    double x1;
    double x2;
    double y1;
    double y2;
};

struct Point {
    double x;
    double y;
    double size;
};

struct Points {
    ARR<Vec2> pts;
    double size;
};

struct PolygonType {
    ARR<Vec2> pts;
};

struct FilledBand {
    ARR<Vec2> pts;
    double ratio;
};

struct TransparentSquare {
    ARR<Vec2> pts;
    double opacity_bottom;
    double opacity_top;
    RGBt color_bottom;
    RGBt color_top;
};

struct TransparentSquare2 {
    ARR<Vec2> pts;
    double opacity_bottom;
    double opacity_top;
    double frac;
};

struct TextType {
    std::string str;
    float baseX;
    float baseY;
    float angle;
    float scale;
};

class DemoDrawer : public Drawer
{
protected:
    CLinkList *lineList;
    CLinkList *circleList;
    CLinkList *arcList;
    CLinkList *partialDiskList;
    CLinkList *linesList;
    CLinkList *rectangleList;
    CLinkList *rectangle2List;
    CLinkList *filledRectangleList;
    CLinkList *pointList;
    CLinkList *pointsList;
    CLinkList *polygonList;
    CLinkList *filledPolygonList;
    CLinkList *filledBandList;
    CLinkList *transparentBandList;
    CLinkList *transparentBand2List;
    CLinkList *transparentSquareList;
    CLinkList *transparentSquare2List;
    CLinkList *textList;

public:
	DemoDrawer();
	virtual ~DemoDrawer();
	
    ColorManager* GetColorManager();

	virtual void RefreshCanvas(grCanvas* canvas);
    virtual void ButtonDown(grCanvas* canvas, int button, Vec2 p);                                     

	virtual void PrepareCanvas(grCanvas* canvas);

    void AddLine(double x1, double y1, double x2, double y2);
    void AddCircle(double centerx, double centery, double radius, double offset);
    void AddArc(double centerx, double centery, double radius, double start_angle, double sweep_angle);
    void AddPartialDisk(double centerx, double centery, double inner_radius, 
                        double outer_radius, double start_angle, double sweep_angle);
    void AddLines(const ARR<Vec2> &pts);
    void AddRectangle(double x, double y, double width, double ht);
    void AddRectangle2(double x1, double x2, double y1, double y2);
    void AddFilledRectangle(double x, double y, double width, double ht);
    void AddPoint(double x, double y, double size);
    void AddPoints(const ARR<Vec2> &pts, double size);
    void AddPolygon(const ARR<Vec2> &pts);
    void AddFilledPolygon(const ARR<Vec2> &pts);
    void AddFilledBand(const ARR<Vec2> &pts, double ratio);
    void AddTransparentBand(const ARR<Vec2> &pts,
                            double opacity_bottom, double opacity_top,
                            RGBt color_bottom, RGBt color_top);
    void AddTransparentBand2(const ARR<Vec2> &pts, double opacity_bottom,
                             double opacity_top, double frac);
    void AddTransparentSquare(const ARR<Vec2> &pts,
                              double opacity_bottom, double opacity_top,
                              RGBt color_bottom, RGBt color_top);
    void AddTransparentSquare2(const ARR<Vec2> &pts, double opacity_bottom,
                               double opacity_top, double frac);
    void AddText(std::string str, float baseX, float baseY, float angle, float scale);
};


#endif
