#include <cmath>
#include <wx/image.h>
#include "DimstackVisualBinMeta.h"
#include "../../MainApplication.h"

DimstackVisualBinMeta::DimstackVisualBinMeta(DimstackDrawer* drawer, 
                          DocumentManager* docManager, bool brushed)
{
    this->drawer = drawer;
    this->brushed = brushed;
    this->colorManager = drawer->GetColorManager();

    axesLayer = new Layer(AXES);
    axesMeta = axesLayer->CreateVisualElementMeta(drawer, docManager);
    axesIter = docManager->GetElementsIterator(AXES, axesMeta);
    visAxesIter  = new VisualElementsIterator(axesMeta, axesIter); 
    
    axes = (DimstackVisualAxes *)(visAxesIter->GetNext());

    canvas = drawer->GetCanvas();

    colorData = NULL;
}

DimstackVisualBinMeta::~DimstackVisualBinMeta()
{
    delete axesLayer;
    delete visAxesIter;
    delete axesMeta;
    delete axesIter; 
    delete [] colorData;
}

VisualElement* DimstackVisualBinMeta::InstantVisualElement()
{
    DimstackVisualBin* visBin = new DimstackVisualBin();
    
//    visBin->ResetBinFlags(axes);

    return visBin;
}

void DimstackVisualBinMeta::Convert(VisualElement* visualEle, DocElement* docEle)
{
    DimstackVisualBin* visualBin = (DimstackVisualBin *)visualEle;
    DocBin* docBin = (DocBin *)docEle;

    GLuint texture;
    int width;
    int height;

    int DimstackID = docBin->GetBinID();
    visualBin->SetBinID(DimstackID);
    if (docBin->GetCount() == 0)
    {
        visualBin->SetEmpty(true);
        return;
    }

    visualBin->SetEmpty(false);
    
    if (colorData == NULL)
    {
        PrepareBitMapColorData(docBin);
    }
    if (!canvas->GetTexture(DIMSTACK_HASH, DimstackID, texture, width, height))
    {
        GetBitMap(docBin, w, h, colorData);
        canvas->CreateTexture(DIMSTACK_HASH, DimstackID, w, h, colorData, texture);
        width = w;
        height = h;
    }

    visualBin->SetTexture(texture);
    visualBin->SetWidth(width);        //width: the width of the original texture object
    visualBin->SetHeight(height);      //height: the height of the origianl texture object

    float xpos, ypos, zpos, xsize, ysize;
//dimstack arrange    
//  bool sort = true;
    bool sort = false;
    if (!sort)
    {
        axes->GetBinXYPosSize(DimstackID, docBin->GetIndexes(), xpos, ypos, xsize, ysize);
        visualBin->SetXPosition(xpos);
        visualBin->SetYPosition(ypos);
        zpos = axes->GetBinZPos(DimstackID);
        visualBin->SetZPosition(zpos);
        visualBin->SetXSize(xsize - 0.002);
        visualBin->SetYSize(ysize - 0.002);
    }
    else
    {
        visualBin->SetXPosition(docBin->GetXPosition());
        visualBin->SetYPosition(docBin->GetYPosition());
        zpos = axes->GetBinZPos(DimstackID);
        visualBin->SetZPosition(zpos);
        visualBin->SetXSize(docBin->GetXSize() - 0.01);
        visualBin->SetYSize(docBin->GetYSize() - 0.01);
    }



}


void DimstackVisualBinMeta::GetBitMap(DocBin* docBin, int &w, int &h, float*& colorData)
{
    int wh = w*h;
    float factor = docBin->GetCount() / aveCount * 0.1;
    if (factor > 1)
        factor = 1;
    float x1 = 0.3f;
    float x2 = 1.0f;
    factor = x1 + factor * (x2 - x1);
    for (int i = 0; i < wh * 4; i += 4)
    {
        colorData[i] = 1.f ;
        colorData[i+1] = 1. - factor;
        colorData[i+2] = 1.f;
        colorData[i+3] = 1.f;
    }

    int* indexes = docBin->GetIndexes();
    int x, y;
    for (i = 0; i < dimNum; i++)
    {
        RGBt color = colorManager->GetColor(CATEGORY, indexes[i]);

        y = (i / wNum) * dimBinSize + dimBinInterval;
        x = (i % wNum) * dimBinSize + dimBinInterval;

        int index;
        for (int j = y; j < y + dimBinSolid; j++)
        {
            index = j * wNum * dimBinSize * 4;
            for (int k = x; k < x + dimBinSolid; k++)
            {
                colorData[index + k * 4] = 1 - color.R;
                colorData[index + k * 4+1] = 1 - color.G;
                colorData[index + k * 4+2] = 1 - color.B;
                colorData[index + k * 4+3] = 1.f;
            }
        }
    }
}


void DimstackVisualBinMeta::PrepareBitMapColorData(DocBin *docBin)
{
    dimBinSize = 10;
    dimBinInterval = 3;
    dimBinSolid = dimBinSize - 2 * dimBinInterval;
    dimNum = docBin->GetDimensionSpace()->GetSize();
    hNum = ceil(sqrt((double)dimNum));
    wNum = ceil(dimNum / (float)hNum);
    w = wNum * 10;
    h = hNum * 10;

//    w = 128;
//    h = 128;
    colorData = new float[w*h*4];

    aveCount = (float) (docBin->GetTotalCount()) / docBin->GetBinNumber();
}