//  there can be many kinds of drawers, such as var drawers, parallel 
//  coordinates drawers, glyph drawers et al. A var drawer draws a var 
//  disply on a canvas, while a glyph drawer draws a glyph display on 
//  a canvas. All of them are derived from the common Drawer class. 

#include "Drawer.h"
#include "../DocumentManager.h"
#include "../ViewManager.h"


Drawer::Drawer()
{
    viewManager = NULL;
    docManager = NULL;
    colorManager = NULL;
}

Drawer::~Drawer()
{
    LinkListIterator* iter;

    iter = new LinkListIterator(&layers);
    Layer* layer;
    while (! iter->IsDone())
    {
        layer = (Layer *) iter->GetNext();
        delete layer;
    }

    delete iter;
}

void Drawer::SetViewManager(ViewManager* viewManager) 
{
    this->viewManager = viewManager;
    docManager = viewManager->GetDocumentManager();
    colorManager = viewManager->GetColorManager();
}

void Drawer::AddCanvas(grCanvas *canvas) 
{
    canvasList.AddNodeAtEnd(canvas);
    this->canvas = canvas;
}

void Drawer::RemoveCanvas(grCanvas *canvas) 
{
    canvasList.DeleteElement(canvas);
}

LinkListIterator* Drawer::IterateLayers() 
{
    return new LinkListIterator(&layers);
}

void Drawer::AddLayer(int layerType) 
{ 
    layers.AddNodeAtEnd(new Layer(layerType));
}

void Drawer::RemoveLayer(int layerType) 
{ 
    LinkListIterator* iter = new LinkListIterator(&layers);
    
    Layer *layer;
	while (!iter->IsDone())
	{
		layer = (Layer*) iter->GetNext();
        if (layer->GetLayerType() == layerType)
        {
            layers.DeleteElement(layer);
            break;
        }
    }

    delete iter;
}

void Drawer::Refresh()
{
    LinkListIterator* iter;

    iter = new LinkListIterator(&canvasList);
    grCanvas* canv;
    while (! iter->IsDone())
    {
        canv = (grCanvas *) iter->GetNext();
        canv->Refresh();
    }

    delete iter;
}

void Drawer::ButtonUp(grCanvas* canvas, int button, Vec2 p)
{
    this->canvas = canvas;
}  

void Drawer::MouseMotion(grCanvas* canvas, Vec2 p) 
{
    this->canvas = canvas;
}  

void Drawer::KeyPress(grCanvas* canvas, int key) 
{
    canvas->KeyPress(key);
}  

void Drawer::KeyRelease(grCanvas* canvas, int key) 
{
    canvas->KeyRelease(key);
}  
