//  there can be many kinds of drawers, such as var drawers, parallel 
//  coordinates drawers, glyph drawers et al. A var drawer draws a var 
//  disply on a canvas, while a glyph drawer draws a glyph display on 
//  a canvas. All of them are derived from the common Drawer class. 

#ifndef DRAWER_H
#define DRAWER_H

#include "../Interface/grCanvas.h"
#include "Layer.h"
#include "../Color/ColorManager.h"

class ViewManager;
class DocumentManager;

class Drawer
{
protected:
    ViewManager* viewManager;
    
    DocumentManager* docManager;
    ColorManager* colorManager;
    
    CLinkList canvasList;  //one drawer is capable to draw multiple canvases of the same type
    grCanvas* canvas;   //current canvas on draw
    CLinkList layers;   //the drawer can draw the canvas in multiple layer manner. However, it is not required to use layers

public:
    Drawer();
    virtual ~Drawer(); 
	
    void SetViewManager(ViewManager* viewManager); 
    ViewManager* GetViewManager() {return viewManager;}
    ColorManager* GetColorManager() {return colorManager;}
    DocumentManager* GetDocumentManager() {return docManager;}

    //add a canvas to the drawing list of the drawer
    void AddCanvas(grCanvas *canvas);
    //remove a canvas from the drawing list of the drawer
    void RemoveCanvas(grCanvas *canvas);

   	
    //layers can be iterated, added, or removed from a linkedlist. 
    LinkListIterator* IterateLayers();
    void AddLayer(int layerType);
    void RemoveLayer(int layerType);   
    grCanvas* GetCanvas() {return canvas;}
    void Refresh();


    //rendering function
    virtual void RefreshCanvas(grCanvas* canvas) {this->canvas = canvas;}
	//function handles keypress event
    virtual void KeyPress(grCanvas* canvas, int key);
	//function handles key release event
    virtual void KeyRelease(grCanvas* canvas, int key);
    //function handles mouse button down event. int button (0, left, 1, midde, 2, right) indicates which button is up. 
    //Vec2 p contains of the x, y coordinates of the event in a normalized (0-1) canvas coordinate system
    virtual void ButtonDown(grCanvas* canvas, int button, Vec2 p) {this->canvas = canvas;}                                      
    //function handles mouse button up event. int button (0, left, 1, midde, 2, right) indicates which button is up. 
    //Vec2 p contains of the x, y coordinates of the event in a normalized (0-1) canvas coordinate system
    virtual void ButtonUp(grCanvas* canvas, int button, Vec2 p);                                       
    //function handles mouse double click event. int button (0, left, 1, midde, 2, right) indicates which button is up. 
    //Vec2 p contains of the x, y coordinates of the event in a normalized (0-1) canvas coordinate system
    virtual void DoubleClick(grCanvas* canvas, int button, Vec2 p) {this->canvas = canvas;}                              
    //function handles mouse motion event. 
    //Vec2 p contains of the current x, y coordinates of the mouse in a normalized (0-1) canvas coordinate system
    virtual void MouseMotion(grCanvas* canvas, Vec2 p); 
    
    virtual void AllDimensionSpaceDeleted() {return;}
    virtual void CurrentDimensionSpaceChanged() {return;}
};

#endif

