#include <FLOAT.h>
#include <sstream>

#include "../../MainApplication.h"
#include "GlyphDrawer.h"
#include "GlyphVisualRow.h"
#include "GlyphVisualRowMeta.h"
#include "GlyphVisualAxesMeta.h"
#include "../../Interface/ViewImageWindow.h"


GlyphDrawer::GlyphDrawer() : TableDrawer()
{
	settings = new GlyphDrawerSettings();
    AddLayer(FLAT_UNBRUSHED_DATA);
    AddLayer(AXES);
}


GlyphDrawer::~GlyphDrawer()
{
    delete settings;
}

void GlyphDrawer::PrepareCanvas()
{
    canvas->SetCurrent();
//    canvas->SetLowerLeftOrigin();
    canvas->SetTopLeftOrigin();
    canvas->SetViewport();

//    canvas->SetBackground(colorManager->GetColor(BG));
    canvas->SetBackground(RGBt(0, 0, 0, 1));
    canvas->Clear();

    glEnable(GL_DEPTH_TEST);
    glClear(GL_DEPTH_BUFFER_BIT);
    glEnable(GL_BLEND);
    glBlendFunc(GL_SRC_ALPHA,GL_ONE_MINUS_SRC_ALPHA);


}

//before drawer draws a canvas
void GlyphDrawer::PrepareDrawLayer(Layer* layer)
{
    TableDrawer::PrepareDrawLayer(layer);
    canvas->SetCurrent();
    if (layer->IsTranslucent())
    {
        // Turn on Translucency
        glEnable(GL_BLEND);
        glBlendFunc(GL_SRC_ALPHA,GL_ONE_MINUS_SRC_ALPHA);
    }
}

//after drawer draws a canvas
void GlyphDrawer::AfterDrawLayer(Layer* layer)
{
    if (layer->IsTranslucent())
    {
        // Turn off Translucency
  //      glDisable(GL_BLEND);
    }
}

void GlyphDrawer::DrawData(VisualRow *row)
{
    GlyphVisualRow* visRow= (GlyphVisualRow *)row;
    int glyphID = visRow->GetItemID();
    if (settings->IsShowSelectedOnly())
    {
        if (settings->IsTextureSelected(glyphID) == false)
            return;
    }
    if (visRow->GetTexture() != -1)
        DrawTexturedData(visRow);
    else
        DrawPlainData(visRow);
}

void GlyphDrawer::DrawPlainData(GlyphVisualRow *visRow)
{
	float xPos = visRow->GetXPosition();
	float yPos = visRow->GetYPosition();
	float zPos = visRow->GetZPosition();
	float size = visRow->GetXSize();
    RGBt color = visRow->GetColor();

	glColor4f(color.R, color.G, color.B, color.A);

    glPointSize((float) size);
    glBegin(GL_POINTS);
    glVertex3f(xPos, yPos, zPos);
    glEnd();
}



void GlyphDrawer::DrawTexturedData(GlyphVisualRow *visRow)
{
    int glyphID = visRow->GetItemID();

    float xPos = visRow->GetXPosition();
	float yPos = visRow->GetYPosition();
	float zPos = visRow->GetZPosition();
	float xSize = visRow->GetXSize();
	float ySize = visRow->GetYSize();

	glColor4f(1,1,1,1);
	glEnable( GL_TEXTURE_2D );
	
    glBindTexture( GL_TEXTURE_2D, visRow->GetTexture());

 	glBegin(GL_QUADS);
	glTexCoord2d(0.0,0.0); glVertex3d(xPos-xSize,yPos-ySize, zPos);
	glTexCoord2d(1.0,0.0); glVertex3d(xPos+xSize,yPos-ySize, zPos);
	glTexCoord2d(1.0,1.0); glVertex3d(xPos+xSize,yPos+ySize, zPos);
	glTexCoord2d(0.0,1.0); glVertex3d(xPos-xSize,yPos+ySize, zPos);
	glEnd();

	glDisable( GL_TEXTURE_2D );

    if (settings->IsSelectedTextureHighlighted())
    {
        if (glyphID == settings->GetFocusTextureID())
        {
            glColor4f(1.f, 0.f, 0.f, 1.f);
	        glLineWidth(1.5);
            glBegin(GL_LINE_STRIP);
            float pad = 0.00001f;
            glVertex3f(xPos-xSize,yPos-ySize, zPos+pad);
            glVertex3f(xPos+xSize,yPos-ySize, zPos+pad);
            glVertex3f(xPos+xSize,yPos+ySize, zPos+pad);
            glVertex3f(xPos-xSize,yPos+ySize, zPos+pad);
            glVertex3f(xPos-xSize,yPos-ySize, zPos+pad);
            glEnd();
        }
        else if (glyphID == settings->GetBaseTextureID())
        {
            glColor4f(1.f, 0.f, 1.f, 1.f);
	        glLineWidth(1.5);
            glBegin(GL_LINE_STRIP);
            float pad = 0.00001f;
            glVertex3f(xPos-xSize,yPos-ySize, zPos+pad);
            glVertex3f(xPos+xSize,yPos-ySize, zPos+pad);
            glVertex3f(xPos+xSize,yPos+ySize, zPos+pad);
            glVertex3f(xPos-xSize,yPos+ySize, zPos+pad);
            glVertex3f(xPos-xSize,yPos-ySize, zPos+pad);
            glEnd();
        }
        else if (settings->IsTextureSelected(glyphID))
        {
            glColor4f(0.f, 1.f, 0.f, 1.f);
	        glLineWidth(1.5);
            glBegin(GL_LINE_STRIP);
            float pad = 0.00001f;
            glVertex3f(xPos-xSize,yPos-ySize, zPos+pad);
            glVertex3f(xPos+xSize,yPos-ySize, zPos+pad);
            glVertex3f(xPos+xSize,yPos+ySize, zPos+pad);
            glVertex3f(xPos-xSize,yPos+ySize, zPos+pad);
            glVertex3f(xPos-xSize,yPos-ySize, zPos+pad);
            glEnd();
        }
    }
}


VisualElementMeta* GlyphDrawer::CreateVisualRowMeta(DocumentManager* docManager, bool brushed, bool hier)
{
    GlyphVisualRowMeta *meta = new GlyphVisualRowMeta(this, docManager, brushed, hier);
    return meta;
}

VisualElementMeta* GlyphDrawer::CreateVisualAxesMeta()
{
    GlyphVisualAxesMeta *meta = new GlyphVisualAxesMeta(this);
    return meta;
}


int GlyphDrawer::FindItemID(float x, float y)
{
    return settings->GetTextureID(x, y);
}

void GlyphDrawer::DoubleClick(grCanvas* canvas, int button, Vec2 p)
{
    Drawer::DoubleClick(canvas, button, p);
    int itemID = FindItemID(p.X, p.Y);
    if (itemID >= 0)
    {
        if (button == 0)
        {
            std::string imagefile = docManager->GetItemDescription(itemID);
            ViewImageWindow *win = new ViewImageWindow(wxString(imagefile.c_str()));
            win->Show(TRUE);
        }
    }
}

void GlyphDrawer::ButtonDown(grCanvas* canvas, int button, Vec2 p)
{
    Drawer::ButtonDown(canvas, button, p);
    inputMode = GLYPH_NONE;
    int itemID = FindItemID(p.X, p.Y);
    if (itemID >= 0)
    {
        if (button == 0)
        {
            if (canvas->IsControlKey())
            {
                inputMode = GLYPH_MOVE;
                canvas->SetHandCursor();
                currentItem = itemID;
                settings->BringTextureToFront(itemID);
            }
            else if (canvas->IsShiftKey())
            {
                if (!settings->IsTextureDistorted(itemID))
                    settings->AddDistortedTexture(itemID);
                else
                    settings->RemoveDistortedTexture(itemID);
                settings->BringTextureToFront(itemID);
                Refresh();
            }
            else 
            {
                settings->BringTextureToFront(itemID);
                Refresh();
            }
        }
    }
}


void GlyphDrawer::ButtonUp(grCanvas* canvas, int button, Vec2 p)
{
    Drawer::ButtonUp(canvas, button, p);
    if (inputMode == GLYPH_MOVE)
    {
        canvas->SetDefaultCursor();
        inputMode = GLYPH_NONE;
    }
    else
    {
        if (button == 2)
        {
            int itemID = FindItemID(p.X, p.Y);
            if (itemID >= 0)
            {
                std::ostringstream oss;
                oss << itemID << ".jpg";
                
                if (viewManager->GetMainFrame()->GetMDSControlFrame() != NULL)
                    viewManager->GetMainFrame()->GetMDSControlFrame()->SetBaseItem(oss.str());

            }
        }
    }
}

void GlyphDrawer::MouseMotion(grCanvas* canvas, Vec2 p)
{
    Drawer::MouseMotion(canvas, p);
    if (inputMode == GLYPH_MOVE)
    {
        settings->SetTextureXYPosition(currentItem, p.X, p.Y);
        canvas->Refresh();      
    }
    else
    {
/*        int itemID = FindItemID(p.X, p.Y);
        if (itemID >= 0)
        {
            std::ostringstream oss;
            oss << itemID << ".jpg";
            canvas->WriteMessage(oss.str().c_str(), 0);
        }*/
    }
}


void GlyphDrawer::SelectedItemsChanged()
{
    GlyphDrawerSettings *settings = GetSettings();
    settings->SelectedItemsChanged();
    LinkListIterator* iter;

    iter = new LinkListIterator(&canvasList);
    grCanvas* canv;
    while (! iter->IsDone())
    {
        canv = (grCanvas *) iter->GetNext();
        canv->DeleteTextures(VAR_HASH);
    }

    delete iter;
}
