//  A layer can be an axes layer, a selected data items layer, a unselected 
//  data items layer, a label layer, or other layers. Users can show/hide axes,
//  data items, labels, or other visual elements by turning on/off the corresponding 
//  layers.  

#include <stdio.h>
#include "Layer.h"
#include "../MainApplication.h"
#include "TableDrawer.h"

Layer::Layer(int type)
{
    this->layerType = type;
    this->displayMode = DRAW;

    if (type == HIER_UNBRUSHED_DATA || type == HIER_BRUSHED_DATA || 
        type == COLUMN || type == DIMENSIONNAME || type == DIMENSIONLABEL)
        translucent = true;
    else
        translucent = false;
}

Layer::~Layer()
{
}

int Layer::GetLayerType()
{
    return layerType;
}

    
Layer::Display_Mode Layer::GetDisplayMode()
{
    return displayMode;
}


VisualElementMeta* Layer::CreateVisualElementMeta(TableDrawer* drawer, DocumentManager* docManager)
{
    if (layerType  == FLAT_UNBRUSHED_DATA)
        return drawer->CreateVisualRowMeta(docManager, false, false);
    else if (layerType  == BIN)
        return drawer->CreateVisualBinMeta(docManager, false);
    else if (layerType  == FLAT_BRUSHED_DATA)
        return drawer->CreateVisualRowMeta(docManager, true, false);
    else if (layerType  == HIER_UNBRUSHED_DATA)
        return drawer->CreateVisualRowMeta(docManager, false, true);
    else if (layerType  == HIER_BRUSHED_DATA)
        return drawer->CreateVisualRowMeta(docManager, true, true);
    else if (layerType  == AXES)
        return drawer->CreateVisualAxesMeta();
    else if (layerType == DIMENSIONNAME)
        return drawer->CreateVisualDimensionNameMeta(docManager);
    else if (layerType == DIMENSIONLABEL)
        return drawer->CreateVisualDimensionLabelMeta(docManager);
    else if (layerType == RULER)
        return drawer->CreateVisualRulerMeta(docManager); 
    else if (layerType == COLUMN)
        return drawer->CreateVisualColumnMeta(docManager, false, false); 
    else if (layerType == TIMEHISTOGRAM)
        return drawer->CreateVisualTimeHistogramMeta(docManager, false, false); 
    else if (layerType == MASK)
        return drawer->CreateVisualMaskMeta(docManager, false, false); 
    return NULL;
}

/*
VisualElementMeta* Layer::CreateVisualElementMeta(TableDrawer* drawer, DocumentManager* docManager)
{
    VisualElementMeta* vMeta = NULL;
    VisualElementsIterator *dependantVisEleIter = NULL;
    VisualElementMeta *meta;
    Iterator *eleIter; 
    if (dependantLayer != NULL)
    {
        meta = dependantLayer->CreateVisualElementMeta(drawer, docManager);
        eleIter = docManager->GetElementsIterator(dependantLayer->GetLayerType(), meta);
        dependantVisEleIter  = new VisualElementsIterator(meta, eleIter); 
    }

    if (layerType  == DATA)
        vMeta = drawer->CreateVisualRowMeta(dependantVisEleIter);
    else if (layerType  == AXES)
        vMeta = drawer->CreateVisualAxesMeta();
    if (layerType  == BRUSHES)
        return drawer->CreateVisualBrushMeta(this, dependantVisEleIter);
    if (layerType  == LABELS)
        return drawer->CreateVisualLabelsMeta(this, dependantVisEleIter);

    if (dependantVisEleIter != NULL)
    {
  		delete dependantVisEleIter;
        delete eleIter;
        delete meta;
    }
    return vMeta;
}
*/
    /*
VisualElementsIterator* Layer::IterateVisualElements(TableDrawer *drawer)
{
    VisualElementMeta *meta = CreateVisualElementMeta(drawer);

    MainApplication *app = &wxGetApp(); 
    
    DocumentManager* docManager = app->GetDocumentManager();
    
    Iterator *eleIter = docManager->GetElementsIterator(layerType, meta);
    VisualElementsIterator *visEleIter = new VisualElementsIterator(meta, eleIter); 
    return visEleIter;
}*/

/*
void Layer::Visualize(TableDrawer* drawer)
{
    MainApplication *app = &wxGetApp(); 
    
    DocumentManager* docManager = app->GetDocumentManager();
    if (docManager == NULL)
        return; 

    if (displayMode == DRAW)
	{
		//set common setting for drawing in drawer
		drawer->PrepareDrawLayer(this);

        VisualElementMeta *meta = CreateVisualElementMeta(this);
        Iterator *eleIter = docManager->GetElementsIterator(layer->GetLayerType(), meta);
        VisualElementsIterator *visEleIter = new VisualElementsIterator(meta, eleIter); 
		VisualElement* ele;
		//draw all visual elements in a layer
		while (!visEleIter->IsDone())
		{
			//draw a single visual elements
			ele = (VisualElement *)(visEleIter->GetNext());
            ele->Draw(this);
		}
		delete visEleIter;
        delete eleIter;
        delete meta;
		//cleanup drawer
		AfterDrawLayer(layer);
    }
}
*/