#include "ParcoordDrawer.h"

ParcoordDrawer::ParcoordDrawer() : TableDrawer()
{
    axNum = 1;
    AddLayer(AXES);
    AddLayer(FLAT_UNBRUSHED_DATA);
    AddLayer(DIMENSIONLABEL);
    AddLayer(RULER);
    AddLayer(DIMENSIONNAME);
}


ParcoordDrawer::~ParcoordDrawer()
{
}

int ParcoordDrawer::GetAxNum()
{
    return axNum;
}

bool ParcoordDrawer::SetAxNum(int axNum)
{
    if (axNum != 1 && axNum != 3 && axNum != 4)
        return false;
    else
        this->axNum = axNum;

    return true;
}

void ParcoordDrawer::PrepareCanvas()
{
    canvas->SetViewPortMode(PARCOORD);
    canvas->SetCurrent();
    canvas->SetLowerLeftOrigin();
    canvas->SetViewport();

//    canvas->SetBackground(colorManager->GetColor(BG));
    canvas->SetBackground(RGBt(0, 0, 0, 1));
    canvas->Clear();

    glDisable(GL_DEPTH_TEST);

}

//before drawer draws a canvas
void ParcoordDrawer::PrepareDrawLayer(Layer* layer)
{
    TableDrawer::PrepareDrawLayer(layer);
    canvas->SetCurrent();
    if (layer->IsTranslucent())
    {
        // Turn on Translucency
        glEnable(GL_BLEND);
        glBlendFunc(GL_SRC_ALPHA,GL_ONE_MINUS_SRC_ALPHA);
    }
}

//after drawer draws a canvas
void ParcoordDrawer::AfterDrawLayer(Layer* layer)
{
    if (layer->IsTranslucent())
    {
        // Turn off Translucency
        glDisable(GL_BLEND);
    }
}

/*
void ParcoordDrawer::DrawBrush(VisualBrush *brush)
{
    RGBt *color = brush->GetColor();
    glColor4f(color.R, color.G, color.B, color.A);

    float *xs = brush->GetXPositions();

    float *minValues = brush->GetMinValues();
    float *maxValues = brush->GetMaxValues();

   	glBegin(GL_QUAD_STRIP);
    for (int i = 0; i < segNum; i++)
    {
        glVertex2f(xs[i], maxValues[i]);
        glVertex2f(xs[i], minValues[i]);
    }
	glEnd();

    if (brush->IsRampOn() != true)
        return;

    //draw ramp

    RGBt *colorOut = brush->GetRampOutEdgeColor();
    RGBt *colorIn = brush->GetRampInEdgeColor();

    float *rampMinValues = brush->GetRampMinValues();
    float *rampMaxValues = brush->GetRampMaxValues();

   	glBegin(GL_QUAD_STRIP);
    for (i = 0; i < segNum; i++)
    {
        glColor4f(colorOut.R, colorOut.G, colorOut.B, colorOut.A);
        glVertex2f(xs[i], rampMaxValues[i]);
        glColor4f(colorIn.R, colorIn.G, colorIn.B, colorIn.A);
        glVertex2f(xs[i], maxValues[i]);
    }
	glEnd();

   	glBegin(GL_QUAD_STRIP);
    for (i = 0; i < segNum; i++)
    {
        glColor4f(colorOut.R, colorOut.G, colorOut.B, colorOut.A);
        glVertex2f(xs[i], rampMinValues[i]);
        glColor4f(colorIn.R, colorIn.G, colorIn.B, colorIn.A);
        glVertex2f(xs[i], minValues[i]);
    }
	glEnd();
}
*/

void ParcoordDrawer::DrawData(VisualRow *data)
{
    ParcoordVisualRow* visData = (ParcoordVisualRow *)data;
    
    if (visData->IsShowBand())
    {
        DrawBand(visData);
        DrawMean(visData);
    }
    else
        DrawMean(visData);
}



void ParcoordDrawer::DrawMean(ParcoordVisualRow* data)
{
    RGBt color = data->GetVColor(0);
    glColor4f(color.R, color.G, color.B, color.A);
    glLineWidth(1.0);

    int segNum = data->GetSegNum();
    glBegin(GL_LINE_STRIP);
    for (int i = 0; i < segNum; i++)
    {
        glVertex2f(data->GetSegPosition(i), data->GetVPosition(0, i));
    }
	glEnd();
}

void ParcoordDrawer::DrawBand(ParcoordVisualRow* data)
{
    RGBt color1 = data->GetVColor(1);
    RGBt color2 = data->GetVColor(2);

    int segNum = data->GetSegNum();

    float r1 = 0.5f;
    float r2 = 0.02f;

   	glBegin(GL_QUAD_STRIP);
    for (int i = 0; i < segNum; i++)
    {
        glColor4f(color1.R, color1.G, color1.B, r1);
        glVertex2f(data->GetSegPosition(i), data->GetVPosition(0, i));
        glColor4f(color1.R, color1.G, color1.B, r2);
        glVertex2f(data->GetSegPosition(i), data->GetVPosition(1, i));
    }
	glEnd();

   	glBegin(GL_QUAD_STRIP);
    for (i = 0; i < segNum; i++)
    {
        glColor4f(color2.R, color2.G, color2.B, r1);
        glVertex2f(data->GetSegPosition(i), data->GetVPosition(0, i));
        glColor4f(color2.R, color2.G, color2.B, r2);
        glVertex2f(data->GetSegPosition(i), data->GetVPosition(2, i));
    }
	glEnd();
}

void ParcoordDrawer::DrawAxes(VisualAxes *visAxes)
{
    ParcoordVisualAxes* axes = (ParcoordVisualAxes *)visAxes;
    
    int segNum = axes->GetSegNum();
    for (int i = 0; i < segNum; i++)
    {
        RGBt color = axes->GetSegColor(i);
        glColor4f(color.R, color.G, color.B, color.A);

        glLineWidth(axes->GetSegWidth(i));
        
        glBegin(GL_LINES);
        glVertex2f(axes->GetSegPosition(i), 0);
        glVertex2f(axes->GetSegPosition(i), 1);
    	glEnd();
    }
}

VisualElementMeta* ParcoordDrawer::CreateVisualRowMeta(DocumentManager* docManager, bool brushed, bool hier)
{
    ParcoordVisualRowMeta *meta = new ParcoordVisualRowMeta(this, docManager, brushed, hier);
    return meta;
}

VisualElementMeta* ParcoordDrawer::CreateVisualAxesMeta()
{
    ParcoordVisualAxesMeta *meta = new ParcoordVisualAxesMeta(this);
    return meta;
}

VisualElementMeta* ParcoordDrawer::CreateVisualDimensionNameMeta(DocumentManager* docManager)
{
    float angle, scale, shiftX, shiftY;
    angle = 0.;
    scale = 1.;
    
    Vec2 p = canvas->MapScreeVectorToDataVector(Vec2(2, 2));
    shiftX = p.X;
    shiftY = p.Y;
    
    float fontHeight = canvas->GetFontHeight();

    float rim = docManager->GetCurrentDimensionSpace()->GetDimension(0)->GetRimFactor();

    if (rim < fontHeight)
        shiftY += fontHeight - rim;

    ParcoordVisualLabelMeta *meta = new ParcoordVisualLabelMeta(this, docManager, 
        angle, scale, shiftX, shiftY, shiftX, shiftY);
    return meta;
}

VisualElementMeta* ParcoordDrawer::CreateVisualDimensionLabelMeta(DocumentManager* docManager)
{
    float angle, scale, shiftX, shiftY, minShiftX, minShiftY;
    angle = 0.;
    scale = 1.;
    
    Vec2 p = canvas->MapScreeVectorToDataVector(Vec2(2, 2));
    shiftX = p.X;
    shiftY = p.Y;

    minShiftX = shiftX;
    minShiftY = -shiftY - canvas->GetFontHeight();


    ParcoordVisualLabelMeta *meta = new ParcoordVisualLabelMeta(this, docManager, 
        angle, scale, shiftX, shiftY, minShiftX, minShiftY);
    return meta;
}

VisualElementMeta* ParcoordDrawer::CreateVisualRulerMeta(DocumentManager* docManager)
{
    RGBt color = colorManager->GetColorSetting(GRID, 2)->GetColor();
    float length;

    Vec2 p = canvas->MapScreeVectorToDataVector(Vec2(5, 0));
    length = p.X;
    
    ParcoordVisualRulerMeta *meta = new ParcoordVisualRulerMeta(this, docManager, 
        color, length);
    return meta;
}


void ParcoordDrawer::DrawRuler(VisualRuler* ruler)
{
    RGBt color = ruler->GetColor();

    glColor4f(color.R, color.G, color.B, color.A);

    glLineWidth(3.0);
    
    glBegin(GL_LINES);
    glVertex2f(ruler->GetX1(), ruler->GetY1());
    glVertex2f(ruler->GetX2(), ruler->GetY2());
    glEnd();
}

bool ParcoordDrawer::IsNeedDimensionClusterMinMaxInfo()
{
    if (GetAxNum() > 1)
        return true;
    else
        return false;
}

void ParcoordDrawer::DrawLabel(VisualLabel *label)
{
    glColor4f(1.0, 1.0, 1.0, 1);

    canvas->DrawString(label->GetText(), label->GetX(), label->GetY(), 
        label->GetAngle(), label->GetScale());
}

