#include <stdio.h>
#include "ParcoordVisualAxes.h"

ParcoordVisualAxes::ParcoordVisualAxes() : VisualAxes()
{
    axNum = 0;  //number of axes used to display a non-leaf dimension;
    positions = NULL;
    colors = NULL;  
    widths = NULL;
    
    segNum = 0;
    segDimIDs = NULL;
    segTypes = NULL;
    segPositions = NULL;
    segColors = NULL;
    segWidths = NULL;

    dimIDs = NULL;
}

ParcoordVisualAxes::~ParcoordVisualAxes()
{
    int i;
    if (positions != NULL)
    {
        for (i = 0; i < axNum; i++)
        {
            delete [] positions[i];
            delete [] colors[i];
            delete [] widths[i];
        }
        delete positions;
        delete colors;
        delete widths;
    }

    if (segNum != 0)
    {
        delete [] segDimIDs;
        delete [] segTypes;
        delete [] segPositions;
        delete [] segColors;
        delete [] segWidths;
        delete [] dimIDs;
    }
}

void ParcoordVisualAxes::Init()
{
    positions = new float * [axNum];
    colors = new RGBt * [axNum];
    widths = new float * [axNum];
    dimIDs = new int [dimNum];

    for (int i = 0; i < axNum; i++)
    {
        positions[i] = new float [dimNum];
        colors[i] = new RGBt [dimNum];
        widths[i] = new float [dimNum];

        for (int j = 0; j < dimNum; j++)
        {
            positions[i][j] = 0.01f;
            colors[i][j] = RGBt(0xffffff);
            widths[i][j] = 1.0f;
        }
    }
}

bool ParcoordVisualAxes::SetDimensionNum(int num)
{
    if (!VisualAxes::SetDimensionNum(num))
        return false;

    if (dimNum != 0 && axNum != 0 && positions == NULL)
    {
        Init();
    }

    return true;
}

bool ParcoordVisualAxes::SetAxNum(int num)
{
    if (num != 1 && num != 3 && num != 4)
        return false;

    axNum = num;

    if (dimNum != 0 && axNum != 0 && positions == NULL)
    {
        Init();
    }
    
    return true;
}

void ParcoordVisualAxes::SetAxInterval(int axID, float interval)
{
    if (axID >= axNum)
        return;

    for (int i = 0; i < dimNum; i++)
    {
        positions[axID][i] = interval;
    }
}

bool ParcoordVisualAxes::CreateSegments()
{
    if (dimNum == 0 || axNum == 0)
        return false;

    segNum = 0;
    for (int i = 0; i < dimNum; i++)
    {
        if (IsLeafDimension(i))
            segNum ++;
        else
            segNum += axNum;
    }
    
    segDimIDs = new int [segNum];
    segTypes = new int [segNum];

    if (axNum == 1)
    {
        for (i = 0; i < dimNum; i++)
        {
            segDimIDs[i] = i;
            segTypes[i] = 0;
        }
    }
    else if (axNum == 3)
    {
        int counter = 0;
        for (i = 0; i < dimNum; i++)
        {
            if (IsLeafDimension(i))
            {
                segDimIDs[counter] = i;
                segTypes[counter] = 0;
                counter ++;
            }
            else
            {
                //min axis
                segDimIDs[counter] = i;
                segTypes[counter] = 1;
                counter ++;
                //mean axis
                segDimIDs[counter] = i;
                segTypes[counter] = 0;
                counter ++;
                //max axis
                segDimIDs[counter] = i;
                segTypes[counter] = 2;
                counter ++;
            }
        }
    }
    else if (axNum == 4)
    {
        int counter = 0;
        for (i = 0; i < dimNum; i++)
        {
            if (IsLeafDimension(i))
            {
                segDimIDs[counter] = i;
                segTypes[counter] = 0;
                counter ++;
            }
            else
            {
                //mean axis
                segDimIDs[counter] = i;
                segTypes[counter] = 0;
                counter ++;
                //min axis
                segDimIDs[counter] = i;
                segTypes[counter] = 1;
                counter ++;
                //max axis
                segDimIDs[counter] = i;
                segTypes[counter] = 2;
                counter ++;
                //mean axis again
                segDimIDs[counter] = i;
                segTypes[counter] = 0;
                counter ++;
            }
        }
    }

    segPositions = new float[segNum];

    SetAxInterval(1, 0.01f);
    SetAxInterval(2, 0.01f);
    SetAxInterval(3, 0.01f);
    ResetSegPositions();

    segColors = new RGBt[segNum];

    ResetSegColors();

    segWidths = new float[segNum];

    ResetSegWidths();
    return true;
}

void ParcoordVisualAxes::ResetSegPositions()
{
    int i, counter;
    if (axNum == 1)
    {
        for (i = 0; i < dimNum; i++)
        {
            segPositions[i] = positions[0][i];
        }
    }
    else if (axNum == 3)
    {
        counter = 0; 
        for (i = 0; i < dimNum; i++)
        {
            if (IsLeafDimension(i))
            {
                segPositions[counter] = positions[0][i];
                counter ++;
            }
            else
            {
                segPositions[counter] = positions[0][i] - positions[1][i];
                counter ++;
                segPositions[counter] = positions[0][i];
                counter ++;
                segPositions[counter] = positions[0][i] + positions[2][i];
                counter ++;
            }
        }
    }
    else if (axNum == 4)
    {
        counter = 0; 
        for (i = 0; i < dimNum; i++)
        {
            if (IsLeafDimension(i))
            {
                segPositions[counter] = positions[0][i];
                counter ++;
            }
            else
            {
                segPositions[counter] = positions[0][i];
                counter ++;
                segPositions[counter] = positions[0][i] + positions[1][i];
                counter ++;
                segPositions[counter] = positions[0][i] + positions[2][i];
                counter ++;
                segPositions[counter] = positions[0][i] + positions[3][i];
                counter ++;
            }
        }
    }
}

void ParcoordVisualAxes::ResetSegColors()
{
    int i, counter;
    if (axNum == 1)
    {
        for (i = 0; i < dimNum; i++)
        {
            segColors[i] = colors[0][i];
        }
    }
    else if (axNum == 3)
    {
        counter = 0; 
        for (i = 0; i < dimNum; i++)
        {
            if (IsLeafDimension(i))
            {
                segColors[counter] = colors[0][i];
                counter ++;
            }
            else
            {
                segColors[counter] = colors[1][i];
                counter ++;
                segColors[counter] = colors[0][i];
                counter ++;
                segColors[counter] = colors[2][i];
                counter ++;
            }
        }
    }
    else if (axNum == 4)
    {
        counter = 0; 
        for (i = 0; i < dimNum; i++)
        {
            if (IsLeafDimension(i))
            {
                segColors[counter] = colors[0][i];
                counter ++;
            }
            else
            {
                segColors[counter] = colors[0][i];
                counter ++;
                segColors[counter] = colors[1][i];
                counter ++;
                segColors[counter] = colors[2][i];
                counter ++;
                segColors[counter] = colors[0][i];
                counter ++;
            }
        }
    }
}


void ParcoordVisualAxes::ResetSegWidths()
{
    int i, counter;
    if (axNum == 1)
    {
        for (i = 0; i < dimNum; i++)
        {
            segWidths[i] = widths[0][i];
        }
    }
    else if (axNum == 3)
    {
        counter = 0; 
        for (i = 0; i < dimNum; i++)
        {
            if (IsLeafDimension(i))
            {
                segWidths[counter] = widths[0][i];
                counter ++;
            }
            else
            {
                segWidths[counter] = widths[1][i];
                counter ++;
                segWidths[counter] = widths[0][i];
                counter ++;
                segWidths[counter] = widths[2][i];
                counter ++;
            }
        }
    }
    else if (axNum == 4)
    {
        counter = 0; 
        for (i = 0; i < dimNum; i++)
        {
            if (IsLeafDimension(i))
            {
                segWidths[counter] = widths[0][i];
                counter ++;
            }
            else
            {
                segWidths[counter] = widths[0][i];
                counter ++;
                segWidths[counter] = widths[1][i];
                counter ++;
                segWidths[counter] = widths[2][i];
                counter ++;
                segWidths[counter] = widths[0][i];
                counter ++;
            }
        }
    }
}

void ParcoordVisualAxes::SetDimensionPosition(int dimID, float pos)
{
    positions[0][dimID] = pos;
}

float ParcoordVisualAxes::GetDimensionPosition(int dimID)
{
    return positions[0][dimID];
}

float ParcoordVisualAxes::GetDimensionPositionByDimensionID(int dimID)
{
    int order = 0;
    while (dimIDs[order] != dimID && order < dimNum)
    {
        order ++;
    }
    if (order < dimNum)
        return positions[0][order];
    else
        return -1;
}

void ParcoordVisualAxes::SetDimensionWidth(int dimID, float width)
{
    widths[0][dimID] = width;
}

void ParcoordVisualAxes::SetDimensionColor(int dimID, RGBt color)
{
    colors[0][dimID] = color;
}

void ParcoordVisualAxes::SetColor(int dimID, int axID, RGBt color)
{
    colors[axID][dimID] = color;
}

float ParcoordVisualAxes::GetSegPosition(int segID)
{
    return segPositions[segID];
}

float ParcoordVisualAxes::GetSegWidth(int segID)
{
    return segWidths[segID];
}

RGBt ParcoordVisualAxes::GetSegColor(int segID)
{
    return segColors[segID];
}

int ParcoordVisualAxes::GetSegNum()
{
    return segNum;
}

int ParcoordVisualAxes::GetAxNum()
{
    return axNum;
}

void ParcoordVisualAxes::GetSegInformation(int segID, int &dimID, int &type)
{
    dimID = segDimIDs[segID];
    type = segTypes[segID];
}

void ParcoordVisualAxes::SetDimensionID(int order, int dimID)
{
    dimIDs[order] = dimID;
}