#include "ParcoordVisualAxesMeta.h"
#include "ParcoordVisualAxes.h"
#include "../../DataStructure/DocAxes.h"
#include "../../MainApplication.h"

ParcoordVisualAxesMeta::ParcoordVisualAxesMeta(ParcoordDrawer* drawer)
{
    this->drawer = drawer;
}

ParcoordVisualAxesMeta::~ParcoordVisualAxesMeta()
{
}


VisualElement* ParcoordVisualAxesMeta::InstantVisualElement()
{
    ParcoordVisualAxes* visAxes = new ParcoordVisualAxes();
    
    return visAxes;
}

void ParcoordVisualAxesMeta::Convert(VisualElement* visualEle, DocElement* docEle)
{
    ParcoordVisualAxes* visualAxes = (ParcoordVisualAxes *)visualEle;
    DocAxes* docAxes = (DocAxes *)docEle;

    int dimNum = docAxes->GetDimenionNum();

    visualAxes->SetDimensionNum(dimNum);
    visualAxes->SetAxNum(drawer->GetAxNum());

    //set leaf dims
    for (int i = 0; i < dimNum; i++)
    {
        visualAxes->SetLeafDimension(i, docAxes->IsLeafDimension(i));
    }

    for (i = 0; i < dimNum; i++)
    {
        visualAxes->SetDimensionID(i, docAxes->GetDimensionID(i));
    }

    //set positions
    float total = 0;
    for (i = 0; i < dimNum-1; i++)
    {
        float temp = docAxes->GetDimensionDistance(i);
        total += temp;
    }

    float gap = 1./(dimNum+1);
    float factor = (1.-gap)/total;

    float acc = gap*0.5;

    for (i = 0; i < dimNum; i++)
    {
        float temp = docAxes->GetDimensionDistance(i);
        temp = temp * factor;
        visualAxes->SetDimensionPosition(i, acc);
        acc += temp;
    }

    //set colors
    ColorSetting *cs0;
    cs0 = drawer->GetColorManager()->GetColorSetting(AXES, 0);
    for (i = 0; i < dimNum; i++)
    {
        if (cs0->IsSingleColor())
            visualAxes->SetDimensionColor(i, cs0->GetColor());
        else
        {
            float value = docAxes->GetNorValueForColor(i);
            RGBt color = cs0->GetColor(value);
            visualAxes->SetDimensionColor(i, color);
        }
    }

    if (visualAxes->GetAxNum() > 1)
    {
        ColorSetting *cs1, *cs2;
        cs1 = drawer->GetColorManager()->GetColorSetting(AXES, 1);
        cs2 = drawer->GetColorManager()->GetColorSetting(AXES, 2);
        for (i = 0; i < dimNum; i++)
        {
            visualAxes->SetColor(i, 1, cs1->GetColor());
            visualAxes->SetColor(i, 2, cs2->GetColor());
        }
    }

    //set widths
    for (i = 0; i < dimNum; i++)
    {
        visualAxes->SetDimensionWidth(i, docAxes->GetWidth(i));
    }

    visualAxes->CreateSegments();
}