#include "VarVisualLabelMeta.h"
#include "../VisualLabel.h"
#include "../../DataStructure/DocLabel.h"
#include "../../MainApplication.h"

VarVisualLabelMeta::VarVisualLabelMeta(VarDrawer* drawer, DocumentManager* docManager, float angle, float scale, float shiftX, float shiftY, float minShiftX, float minShiftY)
{
    this->drawer = drawer;

    axesLayer = new Layer(AXES);
    axesMeta = axesLayer->CreateVisualElementMeta(drawer, docManager);
    axesIter = docManager->GetElementsIterator(AXES, axesMeta);
    visAxesIter  = new VisualElementsIterator(axesMeta, axesIter); 
    
    axes = (VarVisualAxes *)(visAxesIter->GetNext());

    VarDrawerSettings* setting = drawer->GetSettings();
    this->angle = setting->GetAngle();
    this->scale = scale;
    this->shiftX = shiftX;
    this->shiftY = shiftY;
    this->minShiftX = minShiftX;
    this->minShiftY = minShiftY;
}

VarVisualLabelMeta::~VarVisualLabelMeta()
{
    delete axesLayer;
    delete visAxesIter;
    delete axesMeta;
    delete axesIter; 
}

VisualElement* VarVisualLabelMeta::InstantVisualElement()
{
    VisualLabel* visLabel= new VisualLabel();
    
    visLabel->SetAngle(angle);
    visLabel->SetScale(scale);

    return visLabel;
}

void VarVisualLabelMeta::Convert(VisualElement* visualEle, DocElement* docEle)
{
    VisualLabel* visualLabel = (VisualLabel *)visualEle;
    DocLabel* docLabel= (DocLabel *)docEle;

    visualLabel->SetText(docLabel->GetText());
    
    int id = docLabel->GetDimensionID();

    bool visible = axes->IsLabelDimensionVisible(id);
    visualLabel->SetVisible(visible);
    if (visible)
    {
        float xPos, yPos, zPos, xSize, ySize;

        axes->GetDimensionXYPos(id, xPos, yPos);
        zPos = axes->GetDimensionZPos(id);
        axes->GetDimensionXYSize(id, xSize, ySize);
  
        visualLabel->SetX(xPos);
        visualLabel->SetY(yPos - ySize);
        visualLabel->SetZ(zPos);
    }
}