#include <cmath>
#include <wx/image.h>
#include "VideoClipVisualRowMeta.h"
#include "VideoClipVisualRow.h"
#include "../../DataStructure/DocRow.h"
#include "../../MainApplication.h"

VideoClipVisualRowMeta::VideoClipVisualRowMeta(VideoClipDrawer* drawer, 
                          DocumentManager* docManager, bool brushed, bool hier)
{
    this->drawer = drawer;

    axesLayer = new Layer(AXES);
    axesMeta = axesLayer->CreateVisualElementMeta(drawer, docManager);
    axesIter = docManager->GetElementsIterator(AXES, axesMeta);
    visAxesIter  = new VisualElementsIterator(axesMeta, axesIter); 
    
    axes = (VideoClipVisualAxes *)(visAxesIter->GetNext());

    canvas = drawer->GetCanvas();
    count = 0;
    wxInitAllImageHandlers();
    colorData = NULL;

    focusItem = docManager->GetFocusItemID();
}

VideoClipVisualRowMeta::~VideoClipVisualRowMeta()
{
    delete axesLayer;
    delete visAxesIter;
    delete axesMeta;
    delete axesIter; 
    delete [] colorData;
}

VisualElement* VideoClipVisualRowMeta::InstantVisualElement()
{
    VideoClipVisualRow* visRow = new VideoClipVisualRow();
    
    return visRow;
}

void VideoClipVisualRowMeta::Convert(VisualElement* visualEle, DocElement* docEle)
{
    VideoClipVisualRow* visualRow = (VideoClipVisualRow *)visualEle;
    DocRow* docRow = (DocRow *)docEle;

    GLuint texture;
    int width;
    int height;

    int VideoClipID = docRow->GetItemID();
    w = 128;
    h = 128;
/*    w = 64;
    h = 64;
*/    if (colorData == NULL)
        colorData = new float[w*h*4];
    if (!canvas->GetVideoClipTexture(VideoClipID, texture, width, height))
    {
        GetBitMap(VideoClipID, w, h, NULL, colorData);
        canvas->CreateVideoClipTexture(VideoClipID, w, h, colorData, texture);
        width = w;
        height = h;
    }
  //
//    canvas->GetVideoClipTexture(0, texture, width, height);

    visualRow->SetTexture(texture);
    visualRow->SetWidth(width);        //width: the width of the original texture object
    visualRow->SetHeight(height);      //height: the height of the origianl texture object

    float xpos, ypos, zpos, xsize, ysize;
    axes->GetItemXYPos(VideoClipID, xpos, ypos);
    zpos = axes->GetItemZPos(VideoClipID);
    axes->GetItemXYSize(VideoClipID, xsize, ysize);
    visualRow->SetXPosition(xpos);
    visualRow->SetYPosition(ypos);
 //
 //   visualRow->SetZPosition(-1./VideoClipID);
    visualRow->SetZPosition(zpos);
   // visualRow->SetZPosition(-1.5);
    visualRow->SetXSize(xsize);
    visualRow->SetYSize(ysize);
    visualRow->SetItemID(VideoClipID);

    count ++;
}


/*void VideoClipVisualRowMeta::GetBitMap(int VideoClipID, int &w, int &h, float* data, float*& colorData)
{
//    wxString imgFile = wxString::Format("C:\\yangjing\\Fan\\Querydb\\%d.jpg", VideoClipID);
    wxString imgFile = wxString::Format("C:\\yangjing\\Fan\\Querydb\\%d.jpg", 0);
    wxImage image(imgFile);
    int w1 = image.GetWidth();
    int h1 = image.GetHeight();
    unsigned char* imgData = image.GetData();
    int A, B;
    if (w * h1 < h * w1)
    {
        A = w1;
        B = w;
    }
    else
    {
        A = h1;
        B = h;
    }

    int pos, pos1;
    int i, j, i1, j1;
    for (i = 0; i < h; i++)
    {
        i1 = i * A / B;
        if (i1 >= h1)
        {
            for (j = 0; j < w; j++)
            {
                pos = i * w + j;
		        colorData[pos*4+0] = 0.f;
		        colorData[pos*4+1] = 0.f;
		        colorData[pos*4+2] = 0.f;
		        colorData[pos*4+3] = 0.01f;
            }
        }
        else
        {
            for (j = 0; j < w; j++)
            {
                pos = i * w + j;
                j1 = j * A / B;
                if (j1 >= w1)
                {
		            colorData[pos*4+0] = 0.f;
		            colorData[pos*4+1] = 0.f;
		            colorData[pos*4+2] = 0.f;
		            colorData[pos*4+3] = 0.01f;
                }
                else
                {
                    pos1 = i1 * w1 + j1;
		            colorData[pos*4+0] = 1-imgData[pos1*3] / 255.f;
		            colorData[pos*4+1] = 1-imgData[pos1*3+1] / 255.f;
		            colorData[pos*4+2] = 1-imgData[pos1*3+2] / 255.f;
		            colorData[pos*4+3] = 1.f;
                }
            }
        }
    }
}*/

void VideoClipVisualRowMeta::GetBitMap(int VideoClipID, int &w, int &h, float* data, float*& colorData)
{
    wxString imgFile = wxString::Format("C:\\yangjing\\Fan\\Querydb\\%d.jpg", VideoClipID);
//    wxString imgFile = wxString::Format("C:\\yangjing\\Fan\\Querydb\\%d.jpg", 0);
    wxImage image(imgFile);
    int w1 = image.GetWidth();
    int h1 = image.GetHeight();
    unsigned char* imgData = image.GetData();
    int A, B;
    if (w * h1 < h * w1)
    {
        A = w1;
        B = w;
        h = w * h1 / w1;
    }
    else
    {
        A = h1;
        B = h;
        w = h * w1 / h1;
    }

    int pos, pos1;
    int i, j, i1, j1;
    for (i = 0; i < h; i++)
    {
        i1 = i * A / B;
        if (i1 >= h1)
        {
            for (j = 0; j < w; j++)
            {
                pos = i * w + j;
		        colorData[pos*4+0] = 0.f;
		        colorData[pos*4+1] = 0.f;
		        colorData[pos*4+2] = 0.f;
		        colorData[pos*4+3] = 0.01f;
            }
        }
        else
        {
            for (j = 0; j < w; j++)
            {
                pos = i * w + j;
                j1 = j * A / B;
                if (j1 >= w1)
                {
		            colorData[pos*4+0] = 0.f;
		            colorData[pos*4+1] = 0.f;
		            colorData[pos*4+2] = 0.f;
		            colorData[pos*4+3] = 0.01f;
                }
                else
                {
                    pos1 = i1 * w1 + j1;
		            colorData[pos*4+0] = 1-imgData[pos1*3] / 255.f;
		            colorData[pos*4+1] = 1-imgData[pos1*3+1] / 255.f;
		            colorData[pos*4+2] = 1-imgData[pos1*3+2] / 255.f;
		            colorData[pos*4+3] = 1.f;
                }
            }
        }
    }

    if (VideoClipID != focusItem)
    {
        for (i = 0; i < h; i++)
        {
            pos = i * w;
		    colorData[pos*4+0] = 1.f;
		    colorData[pos*4+1] = 1.f;
		    colorData[pos*4+2] = 1.f;
		    colorData[pos*4+3] = 1.f;
            pos = i * w + w - 1;
		    colorData[pos*4+0] = 1.f;
		    colorData[pos*4+1] = 1.f;
		    colorData[pos*4+2] = 1.f;
		    colorData[pos*4+3] = 1.f;
        }
        for (j = 0; j < w; j++)
        {
            pos = j;
		    colorData[pos*4+0] = 1.f;
		    colorData[pos*4+1] = 1.f;
		    colorData[pos*4+2] = 1.f;
		    colorData[pos*4+3] = 1.f;
            pos = (h - 1) * w + j;
		    colorData[pos*4+0] = 1.f;
		    colorData[pos*4+1] = 1.f;
		    colorData[pos*4+2] = 1.f;
		    colorData[pos*4+3] = 1.f;
        }
    }
    else
    {
        for (i = 0; i < h; i++)
        {
            pos = i * w;
		    colorData[pos*4+0] = 0.f;
		    colorData[pos*4+1] = 1.f;
		    colorData[pos*4+2] = 1.f;
		    colorData[pos*4+3] = 1.f;
            pos = i * w + 1;
		    colorData[pos*4+0] = 0.f;
		    colorData[pos*4+1] = 1.f;
		    colorData[pos*4+2] = 1.f;
		    colorData[pos*4+3] = 1.f;
            pos = i * w + w - 2;
		    colorData[pos*4+0] = 0.f;
		    colorData[pos*4+1] = 1.f;
		    colorData[pos*4+2] = 1.f;
		    colorData[pos*4+3] = 1.f;
            pos = i * w + w - 1;
		    colorData[pos*4+0] = 0.f;
		    colorData[pos*4+1] = 1.f;
		    colorData[pos*4+2] = 1.f;
		    colorData[pos*4+3] = 1.f;
        }
        for (j = 0; j < w; j++)
        {
            pos = j;
		    colorData[pos*4+0] = 0.f;
		    colorData[pos*4+1] = 1.f;
		    colorData[pos*4+2] = 1.f;
		    colorData[pos*4+3] = 1.f;
            pos = w + j;
		    colorData[pos*4+0] = 0.f;
		    colorData[pos*4+1] = 1.f;
		    colorData[pos*4+2] = 1.f;
		    colorData[pos*4+3] = 1.f;
            pos = (h - 2) * w + j;
		    colorData[pos*4+0] = 0.f;
		    colorData[pos*4+1] = 1.f;
		    colorData[pos*4+2] = 1.f;
		    colorData[pos*4+3] = 1.f;
            pos = (h - 1) * w + j;
		    colorData[pos*4+0] = 0.f;
		    colorData[pos*4+1] = 1.f;
		    colorData[pos*4+2] = 1.f;
		    colorData[pos*4+3] = 1.f;
        }
    }
}

