#include <string>
#include <fstream>
#include <sstream>
#include <ctime>

#include "DocumentManager.h"
#include "ViewManager.h"
#include "Interface/MainFrame.h"
#include "Interface/DBConnectDialog.h"
#include "DBConnection/DBManager.h"

#include "DataStructure/DimensionSpace.h"
#include "DataStructure/FlatDataSet.h"
#include "DataStructure/HierDataSet.h"
#include "DataStructure/AxesIterator.h"
#include "DataStructure/DocColumn.h"
#include "DataStructure/DimensionNameIterator.h"
#include "DataStructure/ItemNameIterator.h"
#include "DataStructure/BinIterator.h"
#include "DataStructure/TimeHistogramIterator.h"
#include "DataStructure/DimensionLabelIterator.h"
#include "DataStructure/InMemory/InMemoryFlatDataSet.h"
#include "DataStructure/InMemory/InMemoryHierDataSet.h"
#include "DataStructure/DB/DBFlatDataSetLoader.h"
#include "DataStructure/DB/DBFlatDataSet.h"
#include "DataStructure/DB/SimilarityCalculator.h"
#include "DataStructure/DB/NumericSimilarityCalculator.h"
#include "DataStructure/DB/BinarySimilarityCalculator.h"

#include "Display/VisualElementMeta.h"
#include "Display/VisualRowMeta.h"
#include "Display/VisualBinMeta.h"
#include "Display/VisualColumnMeta.h"
#include "Display/Var/VarVisualMaskMeta.h"
#include "Display/VisualTimeHistogramMeta.h"
#include "Display/VisualAxesMeta.h"

#include "Analysis/MDS/Multidimensional_scaling.h"
#include "Analysis/A_MDS/mds.h"
#include "Analysis/Sort/sorting.h"
#include "Analysis/Treemap/TreeMap.h"
#include "Analysis/Treemap/FastClustering.h"


DocumentManager::DocumentManager()
{
    allDimSpace = NULL;
    selectedDimSpace = NULL;
    currDimSpace = NULL;
    flatDataSet = NULL;
    hierDataSet = NULL;

    dataSetName = "";

    dbManager = NULL;
    dbMode = false;

    binaryMode = false;
    
    imageDisplayMode = 0;
    preImageDisplayMode = -1;

    varDisplayMode = 0;
    preVarDisplayMode = -1;

    useSelectedDimSpace = false;

    dimSelectMode = 0;
    dimDistinctSelectOption = 0;

    query = new Query();

    activeDimID = -1;
    baseDimID = -1;
    centerDimID = -1;

    focusItemID = -1;
//    centerItemID = -1;
    centerItemT = 10;
    randomItemX = NULL;
    baseItemID = -1;

    centerT = 10;
    randomX = NULL;

//    distancesToCenterItem = NULL;
    distancesToBaseItem = NULL;
    distancesToBaseDim= NULL;

    cov_matrix = NULL;
    mds_done = false;
    mds_inited = false;

    labelDimHash = NULL;
}

DocumentManager::~DocumentManager()
{
    delete query;

    if (allDimSpace != NULL)
    {
        int len = allDimSpace->GetSize();
        for (int i = 0; i < len; i++)
        {
            delete (Dimension *)(allDimSpace->GetDimension(i));
        }

        delete allDimSpace;

        if (cov_matrix != NULL)
        {
            for (i = 0; i < len; i++)
            {
                delete [] cov_matrix[i];
            }
            delete [] cov_matrix;
        }
        
//        if (distancesToCenterItem != NULL)
//            delete [] distancesToCenterItem;
        if (distancesToBaseItem != NULL)
            delete [] distancesToBaseItem;
        if (distancesToBaseDim != NULL)
            delete [] distancesToBaseDim;
    }

    mds_done = false;
    mds_inited = false;

    if (selectedDimSpace != NULL)
        delete selectedDimSpace;

    if (flatDataSet != NULL)
        delete flatDataSet;
    if (hierDataSet != NULL)
        hierDataSet = NULL;

    EmptyDimXYPosHash();
    EmptyItemXYPosHash();

    if (dbMode == true)
        dbManager->Disconnect();
    if (dbManager != NULL)
        delete dbManager;

    if (randomX != NULL)
        delete [] randomX;

    if (randomItemX != NULL)
        delete [] randomItemX;

}

void DocumentManager::CloseDataSet()
{
    query->Reset();

    if (allDimSpace != NULL)
    {
        int len = allDimSpace->GetSize();
        for (int i = 0; i < len; i++)
        {
            delete (Dimension *)(allDimSpace->GetDimension(i));
        }

        delete allDimSpace;
        allDimSpace = NULL;
        viewManager->AllDimensionSpaceDeleted();
        
        if (cov_matrix != NULL)
        {
            for (i = 0; i < len; i++)
            {
                delete [] cov_matrix[i];
            }
            delete [] cov_matrix;
            cov_matrix = NULL;
        }

//        if (distancesToCenterItem != NULL)
//            delete [] distancesToCenterItem;
//        distancesToCenterItem = NULL;

        if (distancesToBaseItem != NULL)
            delete [] distancesToBaseItem;
        distancesToBaseItem = NULL;

        if (distancesToBaseDim != NULL)
            delete [] distancesToBaseDim;
        distancesToBaseDim = NULL;

        mds_done = false;
        mds_inited = false;
    }

    if (selectedDimSpace != NULL)
    {
        delete selectedDimSpace;
        selectedDimSpace = NULL;
    }
    
    if (flatDataSet != NULL)
    {
        delete flatDataSet;
        flatDataSet = NULL;
    }
    if (hierDataSet != NULL)
    {
        delete hierDataSet;
        hierDataSet = NULL;
    }

    EmptyDimXYPosHash();
    EmptyItemXYPosHash();

    dataSetName = "";

    activeDimID = -1;
    baseDimID = -1;
    centerDimID = -1;

    focusItemID = -1;
//    centerItemID = -1;
    centerItemT = 10;
    randomItemX = NULL;
    baseItemID = -1;

    if (randomX != NULL)
    {
        delete [] randomX;
        randomX = NULL;
    }

    if (randomItemX != NULL)
    {
        delete [] randomItemX;
        randomItemX = NULL;
    }

    selectedItemHash.clear();
    selectedDimHash.clear();
    manualLabelDimHash.clear();
    labelDimHash = NULL;

    varDisplayMode = 0;
    preVarDisplayMode = 0;
    imageDisplayMode = 0;
    preImageDisplayMode = 0;

}

void DocumentManager::EmptyDimXYPosHash()
{
    if (dimXYPosHash.size() != 0)
    {
        IPHash::iterator it;
        it = dimXYPosHash.begin();
        while (it != dimXYPosHash.end())
        {
            delete (Vec2 *)(it->second);
            it ++;
        }
    }
    dimXYPosHash.clear();
}

void DocumentManager::EmptyItemXYPosHash()
{
    if (itemXYPosHash.size() != 0)
    {
        IPHash::iterator it;
        it = itemXYPosHash.begin();
        while (it != itemXYPosHash.end())
        {
            delete (Vec2 *)(it->second);
            it ++;
        }
    }
    itemXYPosHash.clear();
}

void DocumentManager::SetGlobalPath(std::string path)
{
    if (path.at(path.length()-1) == '\\')
        path = path.substr(0, path.length() - 1);

    globalPath = path;
}

bool DocumentManager::OpenDataSet(std::string fileName)
{
    std::string name = fileName.substr(0, fileName.rfind('.'));

    CloseDataSet();

    int index = fileName.rfind('.');
    if (index == std::string::npos)
        return false;

    std::string fileSuffix = fileName.substr(index+1);
    if (fileSuffix == "okc" && IsDBMode())
    {
        //need a db file. If there is no existing db file, 
        //load the okc data into the database and creat the db file
        std::string dbFileName = fileName.substr(0, fileName.length()-3) + "db";
        //judge if the db file exist
        FILE *fptr = fopen(dbFileName.c_str(),"rb");
        if (fptr) 
        {
            fclose(fptr);
        }
        else
        {
            DBFlatDataSetLoader* loader = new DBFlatDataSetLoader();
            if (!loader->Load(dbManager, fileName, binaryMode))
            {
                delete loader;
                return false;
            }
        }
        fileName = dbFileName;
        fileSuffix = "db";
    }
    else if (fileSuffix == "db" && !IsDBMode())
    {
        //need an okc file. If there is no existing okc file, return false; 
        std::string okcFileName = fileName.substr(0, fileName.length()-2) + "okc";
        //judge if the db file exist
        FILE *fptr = fopen(okcFileName.c_str(),"rb");
        if (fptr) 
        {
            fclose(fptr);
            fileName = okcFileName;
            fileSuffix = "okc";
        }
        else
        {
            return false;
        }
    }
        
    if (fileSuffix == "okc" || "db")
    {
        if (fileSuffix == "okc")
            flatDataSet = new InMemoryFlatDataSet();
        else
            flatDataSet = new DBFlatDataSet(dbManager);

        if (!ReadFlatDataSet(fileName))
        {
            CloseDataSet();
            return false;
        }
    }
    else if (fileSuffix == "cg")
    {
        if (!ReadCgFile(fileName))
        {
            CloseDataSet();
            return false;
        }
    }
    else
    {
        return false;
    }

    dataSetName = name;
    SetUseSelectedDimensionSpace(false);
    Dimension *dim = GetTimeDimension();
    if (dim != NULL)
        query->SetTimeDimensionID(dim->GetDimensionID());
    else
        query->SetTimeDimensionID(-1);

    query->SetRowNum(itemNum);
    
    return true;
}

bool DocumentManager::ReadFlatDataSet(std::string fileName)
{
    //judge if the file can be read
    FILE *fptr = fopen(fileName.c_str(),"rb");
    if (!fptr) 
        return false;
    else
        fclose(fptr);

    std::ifstream infile (fileName.c_str());
    
    if (!ReadMetaData(&infile))
    {
        infile.close();
        return false;
    }

    if (!flatDataSet->ReadData(&infile, allDimSpace, itemNum, hasDescription))
    {
        infile.close();
        return false;
    }

    LoadGroupInformation(fileName);

//    allDimSpace->AddRims(1/200.);
    infile.close();
    return true;
}


bool DocumentManager::ReadCgFile(std::string fileName)
{
    if (hierDataSet != NULL)
        return true;
        
    if (allDimSpace == NULL)
    {
        std::string okcfname = fileName;
	    okcfname = okcfname.substr(0, okcfname.length()-2) + "okc";

        //judge if the file can be read
        FILE *fptr = fopen(okcfname.c_str(),"rb");
        if (!fptr) 
            return false;
        else
            fclose(fptr);

        std::ifstream infile (okcfname.c_str());
        if (!ReadMetaData(&infile))
        {
            infile.close();
            return false;
        }
        else
        {
//            allDimSpace->AddRims(1/200.);
        }
    }

    //judge if the file can be read
    FILE *fptr = fopen(fileName.c_str(),"rb");
    if (!fptr) 
        return false;
    else
        fclose(fptr);

    if (!ReadHierData(fileName))
    {
        if (hierDataSet != NULL)
            delete hierDataSet;
        return false;
    }
   return true;
}


bool DocumentManager::ReadMetaData(std::ifstream* infile) 
{
    char buf[2500];
    char *separator = ", \t\n";  // separators required by strtok() function
    (*infile).getline(buf, 2500); // read all the dimension names
    char *p1 = strtok(buf, separator);  // get the first token name
    char *p2 = strtok(NULL, separator);  // get the first token name
    char *p3 = strtok(NULL, separator);  // get the first token name
    dimNum = atoi(p1);
    itemNum = atoi(p2);
    if (p3 != NULL)
    {
        hasDescription = true;
        std::string temp = p3;
        SetGlobalPath(temp);
    }
    else 
    {
        hasDescription = false;
    }

//	(*infile) >> dimNum;
//	(*infile) >> itemNum;

    activeItemNum = itemNum;

    //construct all dimension space of the okc file
	allDimSpace = new DimensionSpace();
	if (!allDimSpace->LoadDimensionsFromFile(infile, dimNum))
    {    
        delete allDimSpace;
        allDimSpace = NULL;
        currDimSpace = NULL;
        selectedDimSpace = NULL;
        return false;
    }

    return true;

}

bool DocumentManager::ReadHierData(std::string fileName) 
{
    //construct all dimension space of the okc file
	hierDataSet = new InMemoryHierDataSet();
    if (!((InMemoryHierDataSet *)hierDataSet)->ReadData(fileName, allDimSpace))
    {    
        delete hierDataSet;
        hierDataSet = NULL;
        return false;
    }
    return true;
}
    
void DocumentManager::LoadGroupInformation(std::string fileName)
{
    std::string dataTableName = fileName.substr(fileName.rfind('/')+1, fileName.rfind('.') - fileName.rfind('/') - 1);
    dataTableName = "wx" + dataTableName.substr(dataTableName.rfind('\\')+1, dataTableName.rfind('.') - dataTableName.rfind('\\') - 1);

    for (int i = 0; i < allDimSpace->GetSize(); i ++)
    {
        Dimension *dim = allDimSpace->GetDimension(i);
        int id = dim->GetDimensionID();
        if (dim->GetType() == NORMINAL_DIMENSION)
        {
            std::ostringstream oss;
	        oss << "select t" << id << ", count(*) as a from " << dataTableName << 
                " group by t" << id << " order by a";

            ResultSet* re = dbManager->ExecuteQuery(oss.str());

            CLinkList *link1, *link2;
            link1 = new CLinkList();
            link2 = new CLinkList();
            while (!re->isDone())
            {
                re->next();
                std::string* str = new std::string("");
                *str = re->getString(0);
                int *size = new int;
//                *size = re->getInt(1);
                *size = re->getDouble(1);
                link1->AddNodeAtEnd(str);
                link2->AddNodeAtEnd(size);
            }
            delete re;

            int groupNum = link1->GetLength();
            std::string* strs = new std::string [groupNum];
            int* sizes = new int [groupNum];
            LinkListIterator *it = new LinkListIterator(link1);
            int j = 0;
            while (!it->IsDone())
            {
                std::string* str = (std::string *)(it->GetNext());
                strs[j] = *str;
                delete str;
                j++;
            }
            delete it;
            delete link1;

            it = new LinkListIterator(link2);
            j = 0;
            while (!it->IsDone())
            {
                int* size = (int *)(it->GetNext());
                sizes[j] = *size;
                delete size;
                j++;
            }
            delete it;
            delete link2;

            dim->SetGroups(strs, sizes, groupNum);
        }
    }
}


Iterator* DocumentManager::GetElementsIterator(int layerType, VisualElementMeta* meta)
{
	if (meta == NULL)
	{
		Iterator* iter = new Iterator();
		return iter;
	}

    if (layerType == FLAT_UNBRUSHED_DATA || layerType == FLAT_BRUSHED_DATA)
    {
        VisualRowMeta *rowMeta = (VisualRowMeta *)meta;
        return GetFlatDataIterator(currDimSpace, rowMeta->IsShowDimCluster(), rowMeta->IsBrushed(), ROW_NORVALUEONLY, true, true); //to be done
    }
    else if (layerType == BIN)
    {
        VisualBinMeta *binMeta = (VisualBinMeta *)meta;
        return GetBinIterator(false, binMeta->IsBrushed()); //to be done
    }
    else if (layerType == TIMEHISTOGRAM)
    {
        VisualTimeHistogramMeta *timeHistogramMeta = (VisualTimeHistogramMeta *)meta;
        return GetTimeHistogramIterator(timeHistogramMeta->GetLoadSpace()); 
    }
    else if (layerType == HIER_UNBRUSHED_DATA || layerType == HIER_BRUSHED_DATA)
    {
        VisualRowMeta *rowMeta = (VisualRowMeta *)meta;
        return GetHierDataIterator(rowMeta->IsShowDimCluster(), rowMeta->IsShowBand(),
                                   rowMeta->IsBrushed()); 
    }
    else if (layerType == COLUMN)
    {
        VisualColumnMeta *columnMeta = (VisualColumnMeta *)meta;
        return GetColumnIterator(columnMeta->GetLoadSpace()); 
    }
    else if (layerType == MASK)
    {
        VarVisualMaskMeta *maskMeta = (VarVisualMaskMeta *)meta;
        return GetColumnIterator(maskMeta->GetLoadSpace()); 
    }
    else if (layerType == AXES)
    {
        VisualAxesMeta *axesMeta = (VisualAxesMeta *)meta;
        return GetAxesIterator(axesMeta->IsDimXYNeededFromDataSet(), axesMeta->IsItemXYNeededFromDataSet(), axesMeta->IsBinXYNeededFromDataSet());
    }
    else if (layerType == DIMENSIONNAME)
    {
        return GetDimensionNameIterator();
    }
    else if (layerType == DIMENSIONLABEL || layerType == RULER)
    {
        return GetDimensionLabelIterator();
    }
    else 
	{
		Iterator* iter = new Iterator();
		return iter;
	}
}


Iterator* DocumentManager::GetFlatDataIterator(DimensionSpace* loadSpace, bool dimCluster, bool brushed, int outputType, bool loadDescriptions, bool storeDescriptions)
{
    if (flatDataSet == NULL)
    {
//        std::string fileName = dataSetName + ".okc";
//       if (!ReadOkcFile(fileName))
            return NULL;
    }
    return flatDataSet->LoadRows(loadSpace, query, dimCluster, false, outputType, loadDescriptions, storeDescriptions); //to be done
}

Iterator* DocumentManager::GetBinIterator(bool dimCluster, bool brushed)
{
    Iterator* rowIter = GetFlatDataIterator(currDimSpace, dimCluster, brushed, ROW_CARDONLY, true, false);
    if (rowIter == NULL)
        return NULL;
    else
    {
        Iterator* binIter = new BinIterator(rowIter, currDimSpace);
        delete rowIter;
        return binIter;
    }
}

std::string DocumentManager::GetItemDescription(int itemID)
{
    std::string name = flatDataSet->GetItemDescription(itemID);
    return globalPath + "\\" + name;
}

Dimension* DocumentManager::GetTimeDimension()
{
    LinkListIterator* it = allDimSpace->IterateDimensions();
    while (!it->IsDone())
    {
        Dimension *dim = (Dimension *)(it->GetNext());
        if (dim->GetType() == DATE_DIMENSION)
        {
            delete it;
            return dim;
        }
    }
    delete it;
    return NULL;
}

Iterator* DocumentManager::GetTimeHistogramIterator(DimensionSpace* loadSpace)
{
    int timeDimID = query->GetTimeDimensionID();
    if (timeDimID == -1)
        return NULL;

    Dimension *timeDim = GetDimensionByID(timeDimID);
    loadSpace->AddDimensionAtEnd(timeDim);
    Iterator* rowIter = GetFlatDataIterator(loadSpace, 0, false, ROW_CARDONLY, true, false);
    if (rowIter == NULL)
    {
        return NULL;
    }
    else
    {
        Iterator* timeHistogramIter = new TimeHistogramIterator(rowIter, loadSpace, timeDim);
        delete rowIter;
        return timeHistogramIter;
    }
}

Iterator* DocumentManager::GetItemNameIterator()
{
    if (allDimSpace == NULL)
		return new Iterator();

    DimensionSpace *tempSpace = new DimensionSpace();
    tempSpace->AddDimension(allDimSpace->GetDimension(0));
    Iterator* rowIter = GetFlatDataIterator(tempSpace, 0, false, ROW_NORVALUEONLY, true, false);
    if (rowIter == NULL)
    {
        delete tempSpace;
        return new Iterator();
    }
    else
    {
        Iterator* itemNameIter = new ItemNameIterator(rowIter);
        delete tempSpace;
        return itemNameIter;
    }
}

Iterator* DocumentManager::GetColumnIterator(DimensionSpace* loadSpace)
{
/*    if (flatDataSet == NULL)
    {
//        std::string fileName = dataSetName + ".okc";
 //       if (!ReadOkcFile(fileName))
            return NULL;
    }*/
    return flatDataSet->LoadColumns(loadSpace, query); //to be done
}


Iterator* DocumentManager::GetHierDataIterator(bool dimCluster, bool showBand, bool brushed)
{
    if (hierDataSet == NULL)
    {
        std::string fileName = dataSetName + ".cg";
        if (!ReadCgFile(fileName))
            return NULL;
    }
    return hierDataSet->LoadData(currDimSpace, dimCluster, showBand, brushed);
}

//dimXYNeeded indicates if the information of the x and y position of the dimensions in a display 
//need to be obtained from the dataset, such as the case of VaR display
//itemXYNeeded indicates if the information of the x and y position of the data items in a display 
//need to be obtained from the dataset, such as the case of glyphs

Iterator* DocumentManager::GetAxesIterator(bool dimXYNeeded, bool itemXYNeeded, bool binXYNeeded)
{
    if (currDimSpace != NULL)
    {
        if (dimXYNeeded == true && dimXYPosHash.size() != currDimSpace->GetSize())
        {
            ResetVarPositions(varDisplayMode);
        }
        if (itemXYNeeded == true && itemXYPosHash.size() == 0 )
        {
            ResetImagePositions(imageDisplayMode);
        }

        return new AxesIterator(currDimSpace, dimXYNeeded, itemXYNeeded, &dimXYPosHash, &itemXYPosHash, &selectedDimHash, &selectedItemHash, &selectedBinHash, labelDimHash);
    }
	else
		return new Iterator();
}


//return: the orderby dimension autoselected by the simi calculator
int DocumentManager::GetDimensionCovMatrix()
{
    int i, j, orderby, numDims;
    numDims = allDimSpace->GetSize();

    if (cov_matrix != NULL)
    {
         for (i = 0; i < numDims; i++)
        {
            delete [] cov_matrix[i];
        }
        delete [] cov_matrix;
    }

    cov_matrix = new double* [numDims];
    for (i = 0; i < numDims; i++)
    {
        cov_matrix[i] = new double [numDims];
    }


    std::string fileName = dataSetName + ".cov";

    std::ifstream infile(fileName.c_str());

    infile >> numDims >> orderby;

    for (i = 0; i < numDims; i++)
    {
        for (j = i; j < numDims; j++)
        {
            infile >> cov_matrix[i][j];
            cov_matrix[j][i] = cov_matrix[i][j];
        }
    }
    infile.close();
    return orderby;
}


bool DocumentManager::CreateMDSPosForDimensions()
{
    int i;

    int numDims = allDimSpace->GetSize();
    std::string posfname = dataSetName + ".pos";
    
    FILE *posfptr = fopen(posfname.c_str(),"rb");
    if (posfptr) 
    {
        fclose(posfptr);
        std::ifstream inposfile (posfname.c_str());

        double x, y;
        Vec2 *p;
	    for (i = 0; i < numDims; i++)
        {
            inposfile >> x;
            inposfile >> y;

            p = new Vec2(x, y);
            dimXYPosHash[i] = p;
        }
        
        int orderbyDimID;
        inposfile >> orderbyDimID;
        query->SetOrderByDimensionID(orderbyDimID);
        query->SetScatterDimensionID(orderbyDimID);
        inposfile.close();
    }
    else
    {
        if (dbMode == false)
            return false;

        if (cov_matrix == NULL)
        {
            int orderbyDimID; 
            orderbyDimID = GetDimensionCovMatrix();
            query->SetOrderByDimensionID(orderbyDimID);
            query->SetScatterDimensionID(orderbyDimID);
        }

        double **dim_pos;
        dim_pos = new double* [2];
        for (i = 0; i < 2; i++)
        {
            dim_pos[i] = new double [numDims];
        }

        //There are two MDS algorithms in this system, a slower one and a faster one. The commented line calls the slower one.
        //Currently the faster one is used. 
        //multidimensional_scaling(numDims, 2, cov_matrix, dim_pos);
        mds(numDims, 1, cov_matrix,  2, dim_pos, 40, 40, 0);
        std::ofstream outposfile(posfname.c_str());

	    for (i = 0; i < numDims; i++)
        {
            outposfile << dim_pos[0][i] << " ";
            outposfile << dim_pos[1][i] << std::endl;

            Vec2 *p;
            p = new Vec2(dim_pos[0][i], dim_pos[1][i]);
            dimXYPosHash[i] = p;
        }
        outposfile << query->GetOrderByDimensionID();
        outposfile.close();

        for (i = 0; i < 2; i++)
        {
            delete [] dim_pos[i];
        }
        delete [] dim_pos;
    }

    return true;
}

bool DocumentManager::CreateCurveTreemapPosForDimensions()
{
    EmptyDimXYPosHash();

    int i;

    int numDims = allDimSpace->GetSize();
    std::string posfname = dataSetName + ".tre";
    FILE *posfptr = fopen(posfname.c_str(),"rb");
    if (!posfptr)
    {
        std::string covfname = dataSetName + ".cov";
        FastClustering(covfname.c_str(),1, 1);
        std::string cgfname = dataSetName + ".cg";
        TreeMap(cgfname.c_str(), posfname.c_str(), 0.2);
    }
    else
    {
        fclose(posfptr);
    }
    std::ifstream inposfile (posfname.c_str());

    int num, width;
    inposfile >> num >> width;
    float bsize = 1.0/width;
    float hbsize = bsize/2.0;

    double x, y;
    Vec2 *p;
    int dimID;
    int xid, yid;
	for (i = 0; i < num; i++)
    {
        inposfile >> dimID;
        inposfile >> xid;
        inposfile >> yid;

        x = xid * bsize + hbsize;
        y = yid * bsize + hbsize;

        p = new Vec2(x, y);
        dimXYPosHash[dimID] = p;
    }
    inposfile.close();
    return true;
}


bool DocumentManager::OnlineCreateXYDimensionPos()
{
    int xDimID, yDimID;
    xDimID = query->GetXDimensionID();
    yDimID = query->GetYDimensionID();

    if (xDimID == -1 && yDimID == -1)
        return false;

    int i;

    DimensionSpace* loadDimSpace = allDimSpace;
    int numDims = loadDimSpace->GetSize();

    if (cov_matrix == NULL)
    {
        GetDimensionCovMatrix();
    }
    
    float x, y;
    for (i = 0; i < numDims; i++)
    {
        x = cov_matrix[xDimID][i];
        y = cov_matrix[yDimID][i];
        Vec2 *p = new Vec2(x, y);
        dimXYPosHash[i] = p;
    }
    return true;
}


bool DocumentManager::OnlineCreateAroundCenterPos()
{
    if (dbMode == false)
        return false;

    if (centerDimID == -1)
        return false;

    int i;

    DimensionSpace* loadDimSpace = allDimSpace;
    int numDims = loadDimSpace->GetSize();

    if (randomX == NULL)
    {
        randomX = new float[numDims];
//	    srand( (unsigned)time( NULL ));
        for (i = 0; i < numDims; i++)
            randomX[i] = (double)rand()/RAND_MAX;
    }

    if (cov_matrix == NULL)
    {
        GetDimensionCovMatrix();
    }
    
    float x, y, dis;
    for (i = 0; i < numDims; i++)
    {
        if (i != centerDimID)
        {
            //here , cernterDimID is incorrect
            dis = cov_matrix[centerDimID][i];
            x = randomX[i];
//            y = (1./100 - dis/100.) * centerT;
            y = 0.2 * (1./100 - dis/100.) * centerT * centerT;
            Vec2 *p = new Vec2(x, y);
            dimXYPosHash[i] = p;
        }
        else
        {
            x = 0.5;
            y = 1;
            Vec2 *p = new Vec2(x, y);
            dimXYPosHash[i] = p;
        }
    }

    return true;
}

/*
void DocumentManager::GetDistancesFromCenterItem()
{
    distancesToCenterItem = new float[activeItemNum];

    std::string covfname = dataSetName + ".iov";

    std::ifstream covfile(covfname.c_str());

	for (int i = 0; i < baseItemID; i++)
//	for (int i = 0; i < centerItemID; i++)
    {
        covfile.ignore(1000000, '\n');
    }
	
    for (int j = 0; j < activeItemNum; j++)
		covfile >> distancesToCenterItem[j];
    
    covfile.close();
}*/

void DocumentManager::GetDistancesFromBaseItem()
{
    distancesToBaseItem = new float[activeItemNum];

    std::string covfname = dataSetName + ".iov";

    std::ifstream covfile(covfname.c_str());

	for (int i = 0; i < baseItemID; i++)
    {
        covfile.ignore(1000000, '\n');
    }
	
    for (int j = 0; j < activeItemNum; j++)
		covfile >> distancesToBaseItem[j];
    
    covfile.close();
}

void DocumentManager::GetDistancesFromBaseDimension()
{
    int numDims = allDimSpace->GetSize();
    distancesToBaseDim = new float [numDims];

    if (cov_matrix == NULL)
    {
        GetDimensionCovMatrix();
    }

    for (int j = 0; j < numDims; j++)
		distancesToBaseDim[j] = cov_matrix[baseDimID][j];
}

bool DocumentManager::OnlineCreateAroundCenterItemPos()
{
    int centerItemID = GetBaseItemID();
    EmptyItemXYPosHash();
    if (centerItemID == -1)
        return false;

    int i;

    if (randomItemX == NULL)
    {
        randomItemX = new float[activeItemNum];
//	    srand( (unsigned)time( NULL ));
        for (i = 0; i < activeItemNum; i++)
            randomItemX[i] = (double)rand()/RAND_MAX;
    }

    if (distancesToBaseItem == NULL)
//    if (distancesToCenterItem == NULL)
    {
//        GetDistancesFromCenterItem();
        GetDistancesFromBaseItem();
    }
    
    float x, y, dis;
    for (i = 0; i < activeItemNum; i++)
    {
        if (i != centerItemID)
        {
//            dis = distancesToCenterItem[i];
            dis = distancesToBaseItem[i];
            x = randomItemX[i];
//            y = (1./100 - dis/100.) * centerT;
 //           if (fabs(dis-1) < 1e-7)
   //             y = 0;
    //        else
                y = (1./100 - dis/100.) * (1./100 - dis/100.) * centerItemT * centerItemT;
            Vec2 *p = new Vec2(x, y);
            itemXYPosHash[i] = p;
        }
        else
        {
            x = 0.5;
            y = 1;
            Vec2 *p = new Vec2(x, y);
            itemXYPosHash[i] = p;
        }
    }

    return true;
}

bool DocumentManager::SetOrderbyDimension(std::string dimName)
{
    Dimension* dim  = allDimSpace->GetDimension(dimName);

    if (dim != NULL)
    {
        query->SetOrderByDimensionID(dim->GetDimensionID());
        return true;
    }
    else
    {
        query->SetOrderByDimensionID(-1);
        return false;
    }
}

void DocumentManager::SetDimensionSelectMode(int i)
{
    dimSelectMode = i;
}

void DocumentManager::SetDistinctDimensionSelectOption(int i)
{
    dimDistinctSelectOption = i;
}


bool DocumentManager::SelectDimensionsAroundBase(float thres)
{
    if (dimSelectMode == 2)
        return false;

    selectedDimHash.clear();
    if (baseDimID == -1)
        return false;
    
    if (dimSelectMode == 0)
        SelectSimilarDimensionsAroundBase(baseDimID, thres);
    else
    {
        if (dimDistinctSelectOption == 0)
            SelectDistinctDimensionsAroundBaseForGroups(baseDimID, thres);
        else
            SelectDistinctDimensionsAroundBaseForLabeling(baseDimID, thres);
    }
    GetSelectedDimensionSpaceFromSelectedDimensionHash();

    if (useSelectedDimSpace)
        SetCurrentDimensionSpace(selectedDimSpace);

    return true;

}

void DocumentManager::SetCurrentDimensionSpace(DimensionSpace* dimSpace)
{
    currDimSpace = dimSpace;
    viewManager->CurrentDimensionSpaceChanged();
}

// for smallest set
bool DocumentManager::SelectDistinctDimensionsAroundBaseForLabeling(int seed_dim, float threshold)
{
    if (cov_matrix == NULL)
    {
        GetDimensionCovMatrix();
    }

    int i, j;
    int dims = allDimSpace->GetSize();

    int* selected_dims = new int[dims];
    for (i = 0; i < dims; i++)
        selected_dims[i] = 0;

    selected_dims[seed_dim] = 1;

    bool is_candidate = true;

    while (is_candidate)
    {
        //find candidate for next selected dimes;
        for (i = 0; i < dims; i++)
        {
            if (selected_dims[i] == 0)
            {
                selected_dims[i] = 2;
                for (j = 0; j < dims; j++)
                {
                    if (selected_dims[j] == 1)
                    {
                        if (cov_matrix[i][j] < threshold)
                            selected_dims[i] = 0;
                    }
                }
            }
        }

        int next_dim = -1;
        double max_var = 0;
        for (i = 0; i < dims; i++)
        {
            if (selected_dims[i] == 2)
            {
                double var = 0;
                for (j = 0; j < dims; j++)
                {
                    var += cov_matrix[i][j];
                }

                if (var > max_var)
                {
                    next_dim = i;
                    max_var = var;
                }
                selected_dims[i] = 0;
            }
        }
        if (next_dim == -1)
            is_candidate = false;
        else
        {
            selected_dims[next_dim] = 1;
        }
    }

    int count = 0;
    for (i = 0; i < dims; i++)
    {
        if (i != seed_dim && selected_dims[i] == 1)
        {
            selectedDimHash[i] = count +1;
            count ++;
        }
    }
    selectedDimHash[seed_dim] = 0;

    delete [] selected_dims;
    return true;
}

//for best representative
bool DocumentManager::SelectDistinctDimensionsAroundBaseForGroups(int seed_dim, float threshold)
{
    if (cov_matrix == NULL)
    {
        GetDimensionCovMatrix();
    }

    int i, j;
    int dims = allDimSpace->GetSize();

    int* selected_dims = new int[dims];
    for (i = 0; i < dims; i++)
        selected_dims[i] = 0;

    selected_dims[seed_dim] = 1;

    bool is_candidate = true;

    while (is_candidate)
    {
        //find candidate for next selected dimes;
        for (i = 0; i < dims; i++)
        {
            if (selected_dims[i] == 0)
            {
                selected_dims[i] = 2;
                for (j = 0; j < dims; j++)
                {
                    if (selected_dims[j] == 1)
                    {
                        if (cov_matrix[i][j] < threshold)
                            selected_dims[i] = 0;
                    }
                }
            }
        }

        int next_dim = -1;
        int count = 0;
        int max_count = -1;
        for (i = 0; i < dims; i++)
        {
            if (selected_dims[i] == 2)
            {
                count = 0;
                for (j = 0; j < dims; j++)
                {
                    if (selected_dims[i] != 1)
                    {
                        if (cov_matrix[i][j] < threshold)
                            count ++;
                    }
                }

                if (count > max_count)
                {
                    next_dim = i;
                    max_count = count;
                }
                selected_dims[i] = 0;
            }
        }
        if (next_dim == -1)
            is_candidate = false;
        else
        {
            selected_dims[next_dim] = 1;
        }
    }

    int count = 0;
    for (i = 0; i < dims; i++)
    {
        if (i != seed_dim && selected_dims[i] == 1)
        {
            selectedDimHash[i] = count +1;
            count ++;
        }
    }
    selectedDimHash[seed_dim] = 0;

    delete [] selected_dims;
    return true;
}


/*for best labeling
//don't delete this piece of code. An interface can be added to allow user to use this selecting for 
//labeling with less clutter
bool DocumentManager::SelectDistinctDimensionsAroundBase(int seed_dim, float threshold)
{
    if (cov_matrix == NULL)
    {
        GetDimensionCovMatrix();
    }

    int i, j;
    int dims = allDimSpace->GetSize();

    int* selected_dims = new int[dims];
    for (i = 0; i < dims; i++)
        selected_dims[i] = 0;

    selected_dims[seed_dim] = 1;

    bool is_candidate = true;

    while (is_candidate)
    {
        //find candidate for next selected dimes;
        for (i = 0; i < dims; i++)
        {
            if (selected_dims[i] == 0)
            {
                selected_dims[i] = 2;
                for (j = 0; j < dims; j++)
                {
                    if (selected_dims[j] == 1)
                    {
                        if (cov_matrix[i][j] < threshold)
                            selected_dims[i] = 0;
                    }
                }
            }
        }

        int next_dim = -1;
        double max_var = 0;
        IPHash::iterator it;
        Vec2* p;
        Vec2* p2;
        double temp;
        for (i = 0; i < dims; i++)
        {
            if (selected_dims[i] == 2)
            {
                double var = 1e6;
                
                it = dimXYPosHash.find(i); 
                if ( it != dimXYPosHash.end())
                {
                    p2 = (Vec2 *)(it->second);
                    for (j = 0; j < dims; j++)
                    {
    //                    var += cov_matrix[i][j];
                        if (selected_dims[j] == 1)
                        {
                            it = dimXYPosHash.find(j); 
                            if ( it != dimXYPosHash.end())
                            {
                                p = (Vec2 *)(it->second);
                                temp = fabs((p2->X - p->X)) + 8 * fabs((p2->Y - p->Y));
                                if (temp < var)
                                    var = temp;
                            }
                        }
                    }
                }

                if (var > max_var)
                {
                    next_dim = i;
                    max_var = var;
                }
                selected_dims[i] = 0;
            }
        }
        if (next_dim == -1)
            is_candidate = false;
        else
        {
            selected_dims[next_dim] = 1;
        }
    }

    int count = 0;
    for (i = 0; i < dims; i++)
    {
        if (i != seed_dim && selected_dims[i] == 1)
        {
            selectedDimHash[i] = count +1;
            count ++;
        }
    }
    selectedDimHash[seed_dim] = 0;

    delete [] selected_dims;
    return true;
}
*/

//for best grouping  
bool DocumentManager::SelectSimilarDimensionsAroundBase(int baseDimID, float thres)
{
    int i;

    if (distancesToBaseDim == NULL)
    {
        GetDistancesFromBaseDimension();
    }
    
    float dis;
    int count = 0;
    IFHash temp;
    
    int dimNum= allDimSpace->GetSize();

    for (i = 0; i < dimNum; i++)
    {
        if (i != baseDimID)
        {
            dis = distancesToBaseDim[i];
            if (dis <= thres)
            {
                temp[i] = dis;
                count ++;
            }
        }
    }
    
    SortStruct *sortArray = new SortStruct[count];
	
    IFHash::iterator it;
    i = 0;
	for (it = temp.begin(); it != temp.end(); it++)			
	{
		sortArray[i].number = it->first;
		sortArray[i].value = it->second;
		i++;
	}

	qsort( sortArray, count, sizeof( SortStruct ), IncreaseSort );
    for (i = 0; i < count; i++)
    {
        selectedDimHash[sortArray[i].number] = i+1; 
    }

    selectedDimHash[baseDimID] = 0;

    delete [] sortArray;
    return true;
}

void DocumentManager::SetUseSelectedDimensionSpace(bool use)
{
    if (use)
    {
        useSelectedDimSpace = true;
        SetCurrentDimensionSpace(selectedDimSpace);
    }
    else
    {
        useSelectedDimSpace = false;
        SetCurrentDimensionSpace(allDimSpace);
    }

}


void DocumentManager::GetSelectedDimensionSpaceFromSelectedDimensionHash()
{
    delete selectedDimSpace;

    selectedDimSpace = new DimensionSpace();
    
    IIHash::iterator it;
    for (it = selectedDimHash.begin(); it != selectedDimHash.end(); it++)
    {
        selectedDimSpace->AddDimension(allDimSpace->GetDimensionByID(it->first));
    }
}

bool DocumentManager::SelectItemsAroundBase(float thres)
{
    // to be done
    selectedItemHash.clear();
    if (baseItemID == -1)
        return false;

    int i;

    if (distancesToBaseItem == NULL)
    {
        GetDistancesFromBaseItem();
    }
    
    float dis;
    int count = 0;
    IFHash temp;
    for (i = 0; i < activeItemNum; i++)
    {
        if (i != baseItemID)
        {
            dis = distancesToBaseItem[i];
            if (dis <= thres)
            {
                temp[i] = dis;
                count ++;
            }
        }
    }
    
    SortStruct *sortArray = new SortStruct[count];
	
    IFHash::iterator it;
    i = 0;
	for (it = temp.begin(); it != temp.end(); it++)			
	{
		sortArray[i].number = it->first;
		sortArray[i].value = it->second;
		i++;
	}

	qsort( sortArray, count, sizeof( SortStruct ), IncreaseSort );
    for (i = 0; i < count; i++)
    {
        selectedItemHash[sortArray[i].number] = i+1; 
    }

    selectedItemHash[baseItemID] = 0;

    delete [] sortArray;
    return true;
}

bool DocumentManager::SelectItemByDimensionValue(int dimID, float value1, float value2)
{
    selectedItemHash.clear();
    int i;
    DimensionSpace* dimSpace = new DimensionSpace();

    dimSpace->AddDimension(allDimSpace->GetDimension(dimID));

    int dimNum = dimSpace->GetSize();

    bool temp = query->IsSampling();
    query->SetSampling(false);
    Iterator* iter = flatDataSet->LoadColumns(dimSpace, query);
    query->SetSampling(temp);
    float* data;
    DocColumn* docColumn = (DocColumn *)(iter->GetNext());
    data = docColumn->GetData();
    int data_size = docColumn->GetSize();

    int* ids = docColumn->GetItemIDs();
    int id;
    int count = 0;
    for (i = 0; i < data_size; i++)
    {
        if ((data[i] >= value1 && data[i] <= value2))
        {
            id = ids[i];
            selectedItemHash[id] = count;
            count ++;
        }
    }
    delete iter;

    baseItemID = -1;
    EmptyItemXYPosHash();
    viewManager->SelectedItemsChanged();

    delete dimSpace;

    return true;
}


bool DocumentManager::SelectSelectedItemByDimensionValue(int dimID, float value1, float value2)
{
    int i;
    DimensionSpace* dimSpace = new DimensionSpace();

    dimSpace->AddDimension(allDimSpace->GetDimension(dimID));

    int dimNum = dimSpace->GetSize();
    bool* notLoad = new bool[dimNum];
    for (i = 0; i < dimNum; i++)
        notLoad[i] = false;

    bool temp = query->IsSampling();
    query->SetSampling(false);
    Iterator* iter = flatDataSet->LoadColumns(dimSpace, query);
    query->SetSampling(temp);
    float* data;
    DocColumn* docColumn = (DocColumn *)(iter->GetNext());
    data = docColumn->GetData();
    int data_size = docColumn->GetSize();

    int* ids = docColumn->GetItemIDs();
    int id;
    for (i = 0; i < data_size; i++)
    {
        if (!(data[i] >= value1 && data[i] <= value2))
        {
            id = ids[i];
            selectedItemHash.erase(id);
        }
    }
    delete iter;

    if (baseItemID >= 0)
    {
        selectedItemHash[baseItemID] = 0;
        IIHash::iterator it;
        IIHash::iterator it1;
        it1 = selectedItemHash.find(baseItemID);
        int count = 1;
        for (it = selectedItemHash.begin(); it != selectedItemHash.end(); it++)
        {
            if (it != it1)
            {
                it->second = count;
                count ++;
            }
        }
    }
    else
    {
        IIHash::iterator it;
        int count = 0;
        for (it = selectedItemHash.begin(); it != selectedItemHash.end(); it++)
        {
            it->second = count;
            count ++;
        }
    }

    EmptyItemXYPosHash();
    viewManager->SelectedItemsChanged();

    delete [] notLoad;
    delete dimSpace;

    return true;
}

bool DocumentManager::SetActiveDimension(std::string dimName)
{
    Dimension* dim  = allDimSpace->GetDimension(dimName);

    if (dim != NULL)
    {
        activeDimID = dim->GetDimensionID();
        return true;
    }
    else
    {
        activeDimID = -1;
        return false;
    }
}

int DocumentManager::GetItemIDByName(std::string itemName)
{
    if (itemName.compare("") == 0)
        return -1;

    std::string str = itemName.substr(0, itemName.find('.'));
    return atoi(str.c_str());
}

bool DocumentManager::SetFocusItem(std::string itemName, int &newID)
{
    int id = GetItemIDByName(itemName);
    if (id > -1)
    {
        focusItemID = id;
        newID = id;
        return true;
    }
    else
    {
        focusItemID = -1;
        newID = -1;
        return false;
    }
}

bool DocumentManager::SetGroupDimension(std::string dimName)
{
    Dimension* dim  = allDimSpace->GetDimension(dimName);

    if (dim != NULL && dim->GetType() == NORMINAL_DIMENSION)
    {
        query->SetGroupByDimensionID(dim->GetDimensionID());
        return true;
    }
    else
    {
        query->SetGroupByDimensionID(-1);
        return false;
    }
}

Iterator* DocumentManager::GetDimensionNameIterator()
{
    if (currDimSpace != NULL)
        return new DimensionNameIterator(currDimSpace);
	else
		return new Iterator();
}

Iterator* DocumentManager::GetDimensionLabelIterator()
{
    if (currDimSpace != NULL)
        return new DimensionLabelIterator(currDimSpace);
	return new Iterator();
}

DimensionSpace* DocumentManager::GetCurrentDimensionSpace()
{
    return allDimSpace;
}

bool DocumentManager::IsEmpty()
{
    if (dataSetName == "" || currDimSpace == NULL)
        return true;
    else
        return false;
}

bool DocumentManager::SetDBMode(bool db)
{
    if (dbMode == db)
        return true;

    if (db == true)
    {
        dbManager = new DBManager();

	    if (!dbManager->IsConnected())
        {
            DBConnectDialog *dlg = new DBConnectDialog(NULL, -1, dbManager);
            dlg->ShowModal();
        }

	    if (!dbManager->IsConnected())
        {
            delete dbManager;
            dbManager = NULL;
            dbMode = false;
            return false;
        }
        else
            dbMode = true;
    }
    else
    {
        CloseDataSet();
        if (dbManager != NULL)
        {
            delete dbManager;
            dbManager = NULL;
            dbMode = false;
        }
    }

    return true;
}

DBManager* DocumentManager::GetDBManager()
{
    return dbManager;
}

bool DocumentManager::IsDBMode()
{
    return dbMode;
}


bool DocumentManager::SetXDimension(std::string dimName)
{
    bool refresh;

    if (query->GetYDimensionID() != -1)
        refresh = true;
    else 
        refresh = false;
    
    Dimension* dim  = allDimSpace->GetDimension(dimName);

    if (dim != NULL)
    {
        query->SetXDimensionID(dim->GetDimensionID());
    }
    else
    {
        query->SetXDimensionID(-1);
    }
    
    return refresh;
}


bool DocumentManager::SetYDimension(std::string dimName)
{
    bool refresh;

    if (query->GetXDimensionID() != -1)
        refresh = true;
    else 
        refresh = false;

    Dimension* dim  = allDimSpace->GetDimension(dimName);

    if (dim != NULL)
    {
        query->SetYDimensionID(dim->GetDimensionID());
    }
    else
    {
        query->SetYDimensionID(-1);
    }

    return refresh;
}

bool DocumentManager::SetCenterDimension(std::string dimName)
{
    Dimension* dim  = allDimSpace->GetDimension(dimName);

    if (dim != NULL)
    {
        centerDimID = dim->GetDimensionID();
        return true;
    }
    else
    {
        centerDimID = -1;
        return false;
    }
}

/*bool DocumentManager::SetCenterItem(std::string itemName)
{
    int id = GetItemIDByName(itemName);
    if (id > -1)
    {
        centerItemID = id;
        if (distancesToCenterItem != NULL)
            delete [] distancesToCenterItem;
        distancesToCenterItem = NULL;
        return true;
    }
    else
    {
        centerItemID = -1;
        if (distancesToCenterItem != NULL)
            delete [] distancesToCenterItem;
        distancesToCenterItem = NULL;
        return false;
    }
}*/

bool DocumentManager::SetVarGlyphBaseDimension(std::string dimName, int mode)
{
    Dimension* dim  = allDimSpace->GetDimension(dimName);

    if (dim != NULL)
    {
        if (mode == 4)
        {
            query->SetTimeDimensionID(dim->GetDimensionID());
        }
        else if (mode == 3)
        {
            query->SetScatterDimensionID(dim->GetDimensionID());
        }
        else
        {
            query->SetOrderByDimensionID(dim->GetDimensionID());
        }
        return true;
    }
    else
    {
        return false;
    }
}

bool DocumentManager::SetBaseItem(std::string itemName, int &newID)
{
    int id = GetItemIDByName(itemName);
    if (id > -1)
    {
        baseItemID = id;
        if (distancesToBaseItem != NULL)
            delete [] distancesToBaseItem;
        distancesToBaseItem = NULL;
        newID = baseItemID;
        return true;
    }
    else
    {
        baseItemID = -1;
        newID = baseItemID;
        if (distancesToBaseItem != NULL)
            delete [] distancesToBaseItem;
        distancesToBaseItem = NULL;
        return false;
    }
}

bool DocumentManager::SetBaseDimension(std::string dimName, int &newID)
{
    int id = GetDimensionIDByName(dimName);

    if (dimSelectMode == 2)
    {
        if (id > -1)
        {
            IIHash::Iterator it;
            it = selectedDimHash.find(id);
            if (it == selectedDimHash.end())
            {
                int size = selectedDimHash.size();
                selectedDimHash[id] = size;
                return true;
            }
            else
            {
                selectedDimHash.erase(id);
                return true;
            }

        }
        else
           return false;
    }
    else
    {
        if (id > -1)
        {
            baseDimID = id;
            if (distancesToBaseDim != NULL)
                delete [] distancesToBaseDim;
            distancesToBaseDim = NULL;
            newID = baseDimID;
            return true;
        }
        else
        {
            baseDimID = -1;
            newID = baseDimID;
            if (distancesToBaseDim != NULL)
                delete [] distancesToBaseDim;
            distancesToBaseDim = NULL;
            return false;
        }
    }
}

/*
// this function will be called only if mds_done == false
bool DocumentManager::CreateAnimateMDSPos()
{
    int i;

    if (mds_done)
    {
        // do nothing if mds is done
        return true;
    }
    else if (!mds_inited)
    {
        // initialized an animated mds process
        
        // try to read positions from a .pos file. If the file exists and 
        // the current dim space is alldimspace, then directly use pos. 
        // if the file exists but the current dim space is not all dim space, 
        // use pos in the file as the init pos in the iteractive mds process.
        int numDims = allDimSpace->GetSize();
        std::string posfname = dataSetName + ".pos";
    
        FILE *posfptr = fopen(posfname.c_str(),"rb");
        if (posfptr) 
        {
            // the pos file exists. Use of it
            fclose(posfptr);
            std::ifstream inposfile (posfname.c_str());

            double x, y;
            Vec2 *p;
	        for (i = 0; i < numDims; i++)
            {
                inposfile >> x;
                inposfile >> y;

                p = new Vec2(x, y);
                dimXYPosHash[i] = p;
            }
        
            inposfile >> orderbyDimID;
            inposfile.close();

            // if current dimension space is global dime space, then it is done. 
            // Else, use globle pos as initial pos of MDS, to be done.
            mds_done = true;
            return true;
        }
        else
        {
            // the pos file doesn't exist
            
            //Load the cov_matrix
            if (dbMode == false)
                return false;

            if (cov_matrix == NULL)
            {
                orderbyDimID = GetDimensionCovMatrix();
            }

            //initialize the positions of mds positions using random numbers
	        srand( (unsigned)time( NULL ));

            Vec2 *p;
	        for (i = 0; i < numDims; i++)
            {
                p = new Vec2((double)rand()/RAND_MAX,(double)rand()/RAND_MAX);
                dimXYPosHash[i] = p;
            }

            mds_inited = true;
            mds_done = false;

            return true;
        }
    }
    else
    {
        //need to change to curr dim space. To be done
        int numDims = allDimSpace->GetSize();
        
        double **dim_pos;
        dim_pos = new double* [2];
        for (i = 0; i < 2; i++)
        {
            dim_pos[i] = new double [numDims];
        }
        //to be done
        Vec2 *p;
        for (i = 0; i < numDims; i++)
        {
            p = (Vec2 *)(dimXYPosHash[i]);
            dim_pos[0][i] = p->X;
            dim_pos[1][i] = p->Y;
            delete p;
        }

//        mds_done = animate_multidimensional_scaling(numDims, 2, cov_matrix, dim_pos);
        multidimensional_scaling(numDims, 2, cov_matrix, dim_pos);
        mds_done = true;

        for (i = 0; i < numDims; i++)
        {
            p = new Vec2(dim_pos[0][i], dim_pos[1][i]);
            dimXYPosHash[i] = p;
        }
        
        if (mds_done)
        {

            // if curr dim space is all dim space and that the pos file does not exit, write the poses into
            // a pos file
            
            std::string posfname = dataSetName + ".pos";
    
            FILE *posfptr = fopen(posfname.c_str(),"rb");
            if (!posfptr) 
            {
                std::ofstream outposfile(posfname.c_str());

	            for (i = 0; i < numDims; i++)
                {
                    outposfile << dim_pos[0][i] << " ";
                    outposfile << dim_pos[1][i] << std::endl;

                }
                outposfile << orderbyDimID;
                outposfile.close();
            }
        }

        for (i = 0; i < 2; i++)
        {
            delete dim_pos[i];
        }
        delete [] dim_pos;

        return true;
    }

}
*/

/*void DocumentManager::CreateMDSPosForItems()
{
	int i, j, k;

    std::string posfname = dataSetName + ".ios";
    
    FILE *posfptr = fopen(posfname.c_str(),"rb");
    if (posfptr) 
    {
        fclose(posfptr);
        std::ifstream inposfile (posfname.c_str());

        double x, y;
        Vec2 *p;

        int numItems;
        inposfile >> numItems;

	    for (i = 0; i < numItems; i++)
        {
            inposfile >> x;
            inposfile >> y;

            p = new Vec2(x, y);
            itemXYPosHash[i] = p;
        }
        
        inposfile.close();
    }
    else
    {
        DimensionSpace* dimSpace = new DimensionSpace();

        int dimNum = allDimSpace->GetSize();

        Dimension* dim;

        for (i = 0; i < dimNum; i++)
        {
            dim = allDimSpace->GetDimension(i);
            if (dim->GetType() != LABELDIMENSION)
                dimSpace->AddDimension(dim);
        }

        dimNum = dimSpace->GetSize();
        bool* notLoad = new bool[dimNum];
        for (i = 0; i < dimNum; i++)
            notLoad[i] = false;

        Iterator* iter = flatDataSet->LoadColumns(dimSpace, notLoad);
    
        float* data;
        DocColumn* docColumn = (DocColumn *)(iter->GetNext());
        data = docColumn->GetData();
        int data_size = docColumn->GetSize();

        double **dis_matrix = new double *[data_size];
        for (i = 0; i < data_size; i++)
        {
            dis_matrix[i] = new double[data_size];
            for (j = 0; j < data_size; j++)
                dis_matrix[i][j] = 0;
        }
    
        for (i = 0; i < dimNum; i++)
        {
            if (i != 0)
            {
                data = docColumn->GetData();
            }

            for (j = 0; j < data_size; j++)
            {
                for (k = j; k < data_size; k++)
                {
                    dis_matrix[j][k] += (data[j] - data[k]) * (data[j] - data[k]);
                }
            }
            docColumn = (DocColumn *)(iter->GetNext());
        }

        for (j = 0; j < data_size; j++)
        {
            for (k = j; k < data_size; k++)
            {
                dis_matrix[j][k] = sqrt(dis_matrix[j][k] / dimNum);
                dis_matrix[k][j] = dis_matrix[j][k];
            }
        }

        //debug
        std::string covfname = dataSetName + ".iov";

        std::ofstream outcovfile(covfname.c_str());

	    for (i = 0; i < data_size; i++)
        {
		    for (int j = 0; j < data_size; j++)
		        outcovfile << dis_matrix[i][j] << " ";
            outcovfile << std::endl;
        }
        outcovfile.close();
        //debug


        double **pos;
        pos = new double* [2];
        for (i = 0; i < 2; i++)
        {
            pos[i] = new double [data_size];
        }

//        multidimensional_scaling(data_size, 2, dis_matrix, pos);
        mds(data_size, 1, dis_matrix, 2, pos, 10, 20);

        Vec2 *p;
        for (i = 0; i < data_size; i++)
        {
            p = new Vec2(pos[0][i], pos[1][i]);
            itemXYPosHash[i] = p;
        }

        std::string posfname = dataSetName + ".ios";

        std::ofstream outposfile(posfname.c_str());

        outposfile << data_size << std::endl;

	    for (i = 0; i < data_size; i++)
        {
            outposfile << pos[0][i] << " ";
            outposfile << pos[1][i] << std::endl;

        }
        outposfile.close();

        for (i = 0; i < data_size; i++)
        {
            delete [] dis_matrix[i];
        }
    
        delete [] dis_matrix;

        delete [] pos;

        delete dimSpace;

    //    AddBuffer();
    }
}
*/


/*
void DocumentManager::CreatePCAPosForItems()
{
	int i, j, k;

    std::string posfname = dataSetName + ".ios";
    
    FILE *posfptr = fopen(posfname.c_str(),"rb");
    if (posfptr) 
    {
        fclose(posfptr);
        std::ifstream inposfile (posfname.c_str());

        double x, y;
        Vec2 *p;

        int numItems;
        inposfile >> numItems;

	    for (i = 0; i < numItems; i++)
        {
            inposfile >> x;
            inposfile >> y;

            p = new Vec2(x, y);
            itemXYPosHash[i] = p;
        }
        
        inposfile.close();
    }
    else
    {
        double **dis_matrix;
	    double **pos;

        DimensionSpace* dimSpace = new DimensionSpace();

        int dimNum = allDimSpace->GetSize();

        Dimension* dim;

        for (i = 0; i < dimNum; i++)
        {
            dim = allDimSpace->GetDimension(i);
            if (dim->GetType() != LABELDIMENSION)
                dimSpace->AddDimension(dim);
        }

        dimNum = dimSpace->GetSize();
        bool* notLoad = new bool[dimNum];
        for (i = 0; i < dimNum; i++)
            notLoad[i] = false;

        Iterator* iter = flatDataSet->LoadColumns(dimSpace, notLoad);
    
        float* data;
        DocColumn* docColumn = (DocColumn *)(iter->GetNext());
        data = docColumn->GetData();
        int data_size = docColumn->GetSize();

        dis_matrix = new double *[data_size];
        for (i = 0; i < data_size; i++)
        {
            dis_matrix[i] = new double[data_size];
            for (j = 0; j < data_size; j++)
                dis_matrix[i][j] = 0;
        }
    
        for (i = 0; i < dimNum; i++)
        {
            if (i != 0)
            {
                data = docColumn->GetData();
            }

            for (j = 0; j < data_size; j++)
            {
                for (k = j; k < data_size; k++)
                {
                    dis_matrix[j][k] += (data[i] - data[k]) * (data[i] - data[k]);
                }
            }
        }

  	    multidimensional_scaling(data_size, 2, dis_matrix, pos);

        Vec2 *p;
        for (i = 0; i < data_size; i++)
        {
            p = new Vec2(pos[0][i], pos[1][i]);
            itemXYPosHash[i] = p;
        }

        std::string posfname = dataSetName + ".ios";

        std::ofstream outposfile(posfname.c_str());

        outposfile << data_size;

	    for (i = 0; i < dimNum; i++)
        {
            outposfile << pos[0][i] << " ";
            outposfile << pos[1][i] << std::endl;

        }
        outposfile << orderbyDimID;
        outposfile.close();

        for (i = 0; i < data_size; i++)
        {
            delete [] dis_matrix[i];
        }
    
        delete [] dis_matrix;

        delete [] pos;

        delete dimSpace;

    //    AddBuffer();
    }
}
*/


void DocumentManager::CreateMDSPosForItems()
{
    EmptyItemXYPosHash();

	int i, j, k;

    std::string posfname = dataSetName + ".ios";
    
    FILE *posfptr = fopen(posfname.c_str(),"rb");
    if (posfptr) 
    {
        fclose(posfptr);
        std::ifstream inposfile (posfname.c_str());

        double x, y;
        Vec2 *p;

        int numItems;
        inposfile >> numItems;

	    for (i = 0; i < numItems; i++)
        {
            inposfile >> x;
            inposfile >> y;

            p = new Vec2(x, y);
            itemXYPosHash[i] = p;
        }
        
        inposfile.close();
    }
    else
    {
        DimensionSpace* dimSpace = new DimensionSpace();

        int dimNum = allDimSpace->GetSize();

        Dimension* dim;

        for (i = 0; i < dimNum; i++)
        {
            dim = allDimSpace->GetDimension(i);
            dimSpace->AddDimension(dim);
        }

        dimNum = dimSpace->GetSize();

        bool temp = query->IsSampling();
        query->SetSampling(false);
        int orderby = query->GetOrderByDimensionID();
        bool increase = query->IsIncreasingOrder();
        query->SetOrderByDimensionID(-1);
        query->SetIncreasingOrder(true);
        Iterator* iter = flatDataSet->LoadColumns(dimSpace, query);
        query->SetSampling(temp);
        query->SetOrderByDimensionID(orderby);
        query->SetIncreasingOrder(increase);
    
        float* data;
        DocColumn* docColumn = (DocColumn *)(iter->GetNext());
        data = docColumn->GetData();
        int data_size = docColumn->GetSize();
        activeItemNum = data_size;

        double **dis_matrix = new double *[data_size];
        for (i = 0; i < data_size; i++)
        {
            dis_matrix[i] = new double[data_size];
            for (j = 0; j < data_size; j++)
                dis_matrix[i][j] = 0;
        }
    
        double *counts = new double [data_size];
        for (i = 0; i < data_size; i++)
        {
            counts[i] = 0;
        }
        for (i = 0; i < dimNum; i++)
        {
            if (i != 0)
            {
                data = docColumn->GetData();
            }

            for (j = 0; j < data_size; j++)
            {
                if (data[j] > 0.5)
                {
                    counts[j] ++;
                    for (k = j; k < data_size; k++)
                    {
                        if (data[k] > 0.5)
                            dis_matrix[j][k] += 1;
                    }
                }
            }
            docColumn = (DocColumn *)(iter->GetNext());
        }
        delete iter;

        for (j = 0; j < data_size; j++)
        {
            for (k = j; k < data_size; k++)
            {
                if (counts[j] > 0.1 && counts[k] > 0.1) 
                {
                    dis_matrix[j][k] = dis_matrix[j][k]/counts[j] + dis_matrix[j][k]/counts[k];
                    dis_matrix[j][k] = 1-dis_matrix[j][k] / 2;
                }
                else if (counts[j] < 0.1 && counts[k] < 0.1)
                    dis_matrix[j][k] = 0;
                else
                    dis_matrix[j][k] = 1;
                dis_matrix[k][j] = dis_matrix[j][k];
            }
        }

        delete [] counts;

        std::string covfname = dataSetName + ".iov";

        std::ofstream outcovfile(covfname.c_str());

	    for (i = 0; i < data_size; i++)
        {
		    for (int j = 0; j < data_size; j++)
		        outcovfile << dis_matrix[i][j] << " ";
            outcovfile << std::endl;
        }
        outcovfile.close();

        double **pos;
        pos = new double* [2];
        for (i = 0; i < 2; i++)
        {
            pos[i] = new double [data_size];
        }

//        multidimensional_scaling(data_size, 2, dis_matrix, pos);
        mds(data_size, 1, dis_matrix, 2, pos, 50, 200, 0);

        Vec2 *p;
        for (i = 0; i < data_size; i++)
        {
            p = new Vec2(pos[0][i], pos[1][i]);
            itemXYPosHash[i] = p;
        }

        std::string posfname = dataSetName + ".ios";

        std::ofstream outposfile(posfname.c_str());

        outposfile << data_size << std::endl;

	    for (i = 0; i < data_size; i++)
        {
            outposfile << pos[0][i] << " ";
            outposfile << pos[1][i] << std::endl;

        }
        outposfile.close();

        for (i = 0; i < data_size; i++)
        {
            delete [] dis_matrix[i];
        }
    
        delete [] dis_matrix;

        for (i = 0; i < 2; i++)
        {
            delete [] pos[i];
        }
        delete [] pos;

        delete dimSpace;
    }
   
}

bool DocumentManager::OnlineCreateOneByOneSelectedItemPos()
{
    EmptyItemXYPosHash();

    int data_size = selectedItemHash.size();

    float blocksize, halfblocksize;
    int blockcnt;

    blockcnt = ceil(sqrt((double)data_size));
    blocksize = 1.0/blockcnt;
    halfblocksize = .5 * blocksize;

    IIHash::iterator it;
    Vec2 *p;
    int i;
    for (it = selectedItemHash.begin(); it != selectedItemHash.end(); it++)
    {
        i = it->second;
        p = new Vec2();
		p->X = blocksize * (i%blockcnt) + halfblocksize;
		p->Y = blocksize * (i/blockcnt) + halfblocksize;
        itemXYPosHash[it->first] = p;
    }
    return true;
}

bool DocumentManager::OnlineCreateMDSSelectedItemPos()
{
    int i, j, k;
    int data_size = selectedItemHash.size();
    
    double **pos;
    pos = new double* [2];
    for (i = 0; i < 2; i++)
    {
        pos[i] = new double [data_size];
    }

    int *ids = new int[data_size];
    IIHash::iterator it;
    IPHash::iterator it2;
    Vec2 *p;
    i = 0;
    //get the inital position of the selected items for animated mds
    for (it = selectedItemHash.begin(); it != selectedItemHash.end(); it++)
    {
        ids[i] = it->first;
        it2 = itemXYPosHash.find(it->first);
        if (it2 != itemXYPosHash.end())
        {
            p = (Vec2 *)(it2->second);
            pos[0][i] = p->X;
            pos[1][i] = p->Y;
        }
        else
        {
            pos[0][i] = (double)rand()/RAND_MAX;
            pos[1][i] = (double)rand()/RAND_MAX;
        }
        i++;
    }
    EmptyItemXYPosHash();

    DimensionSpace* dimSpace = new DimensionSpace();

    int dimNum = allDimSpace->GetSize();

    Dimension* dim;

    for (i = 0; i < dimNum; i++)
    {
        dim = allDimSpace->GetDimension(i);
        dimSpace->AddDimension(dim);
    }

    dimNum = dimSpace->GetSize();

    bool temp = query->IsSampling();
    query->SetSampling(false);
    int orderby = query->GetOrderByDimensionID();
    bool increase = query->IsIncreasingOrder();
    query->SetOrderByDimensionID(-1);
    query->SetIncreasingOrder(true);
    Iterator* iter = flatDataSet->LoadColumns(dimSpace, query);
    query->SetSampling(temp);
    query->SetOrderByDimensionID(orderby);
    query->SetIncreasingOrder(increase);

    float* data;
    DocColumn* docColumn = (DocColumn *)(iter->GetNext());
    data = docColumn->GetData();

    double **dis_matrix = new double *[data_size];
    for (i = 0; i < data_size; i++)
    {
        dis_matrix[i] = new double[data_size];
        for (j = 0; j < data_size; j++)
            dis_matrix[i][j] = 0;
    }
    
    double *counts = new double [data_size];
    for (i = 0; i < data_size; i++)
    {
        counts[i] = 0;
    }
    for (i = 0; i < dimNum; i++)
    {
        if (i != 0)
        {
            data = docColumn->GetData();
        }

        for (j = 0; j < data_size; j++)
        {
            if (data[ids[j]] > 0.5)
            {
                counts[j] ++;
                for (k = j; k < data_size; k++)
                {
                    if (data[ids[k]] > 0.5)
                        dis_matrix[j][k] += 1;
                }
            }
        }
        docColumn = (DocColumn *)(iter->GetNext());
    }
    delete iter;

    for (j = 0; j < data_size; j++)
    {
        for (k = j; k < data_size; k++)
        {
            if (counts[j] > 0.1 && counts[k] > 0.1) 
            {
                dis_matrix[j][k] = dis_matrix[j][k]/counts[j] + dis_matrix[j][k]/counts[k];
                dis_matrix[j][k] = 1-dis_matrix[j][k] / 2;
            }
            else if (counts[j] < 0.1 && counts[k] < 0.1)
                dis_matrix[j][k] = 0;
            else
                dis_matrix[j][k] = 0.5;
            dis_matrix[k][j] = dis_matrix[j][k];
        }
    }


    //get MDS positions

    int size = 20;
    if (data_size / 10 > 20)
        size = data_size / 10;
    mds(data_size, 1, dis_matrix, 2, pos, 30, 100, 0);
//    mds(data_size, 1, dis_matrix, 2, pos, size, size * 5, 0);
//    mds(data_size, 1, dis_matrix, 2, pos, 20, 100, 0);
    for (i = 0; i < data_size; i++)
    {
        delete [] dis_matrix[i];
    }
    delete [] dis_matrix;

    //put the positions into the hash
    for (i = 0; i < data_size; i++)
    {
        p = new Vec2(pos[0][i], pos[1][i]);
        itemXYPosHash[ids[i]] = p;
    }
    for (i = 0; i < 2; i++)
    {
        delete [] pos[i];
    }
    delete [] pos;

    delete dimSpace;
    delete [] ids;

    return true;

}

bool DocumentManager::OnlineCreateMDSSelectedDimensionPos2()
{
    int i;
    int dimNum = selectedDimHash.size();
    int orderbyDimID = query->GetOrderByDimensionID();

    double **pos;
    pos = new double* [2];
    for (i = 0; i < 2; i++)
    {
        pos[i] = new double [dimNum];
    }
 
    int orderID;
    IIHash::iterator it;
    IPHash::iterator it2;
    Vec2 *p;
    int *ids = new int[dimNum];
    i = 0;
    for (it = selectedDimHash.begin(); it != selectedDimHash.end(); it++)
    {
        ids[i] = it->first;
        if (ids[i] == orderbyDimID)
            orderID = ids[i];

        it2 = dimXYPosHash.find(it->first);
        if (it2 != dimXYPosHash.end())
        {
            p = (Vec2 *)(it2->second);
            pos[0][i] = p->X;
            pos[1][i] = p->Y;
        }
        else
        {
            pos[0][i] = (double)rand()/RAND_MAX;
            pos[1][i] = (double)rand()/RAND_MAX;
        }
        i++;
    }
    EmptyDimXYPosHash();

    //build a dimension space using selected dimensions, without labeldimensions. 
    /*DimensionSpace* dimSpace = new DimensionSpace();
    int dimNum = selectedDimSpace->GetSize();
    Dimension* dim;
    for (i = 0; i < dimNum; i++)
    {
        dim = selectedDimSpace->GetDimension(i);
        if (dim->GetType() != LABELDIMENSION)
            dimSpace->AddDimension(dim);
    }
    dimNum = dimSpace->GetSize(); */
    DimensionSpace* dimSpace = selectedDimSpace;
    dimNum = dimSpace->GetSize(); 

    //load rows for the new dimension space
    bool* notLoad = new bool[dimNum];
    for (i = 0; i < dimNum; i++)
        notLoad[i] = false;
    Iterator* iter = flatDataSet->LoadRows(dimSpace, query, false, false, ROW_NORVALUEONLY, false, false);

    //get distance matrix of the dimensions
    SimilarityCalculator* similarityCalculator;
    if (binaryMode)
        similarityCalculator = new BinarySimilarityCalculator();
    else
        similarityCalculator = new NumericSimilarityCalculator();

    int numBins = 20;
	similarityCalculator->init(dimNum, numBins); 
    double* data = new double[dimNum];
    //to be done
    while (!iter->IsDone())
    {
        DocRow* docRow = (DocRow *)(iter->GetNext());
        for (i = 0; i < dimNum; i++)
        {
            data[i] = docRow->GetNorValue(0, 0, i);
        }
        similarityCalculator->Update(data);
    }
    delete [] data;
    delete [] notLoad;
    delete iter;

    //get distance matrix from the calculater
    similarityCalculator->Calculate();
//    similarityCalculator->CalculateAroundCenter(orderID);
    double **dis_matrix = new double *[dimNum];
    for (i = 0; i < dimNum; i++)
    {
        dis_matrix[i] = new double[dimNum];
    }
    similarityCalculator->GetSimilarityMatrix(dis_matrix);
    delete similarityCalculator;

    //get MDS positions of the dimensions
   
    mds(dimNum, 1, dis_matrix,  2, pos, 40, 40, 0);
    //mds(dimNum, 1, dis_matrix, 2, pos, 5, 20, 0);
    
    for (i = 0; i < dimNum; i++)
    {
        delete [] dis_matrix[i];
    }
    delete [] dis_matrix;


    //put the positions into the position hash
    for (i = 0; i < dimNum; i++)
    {
        p = new Vec2(pos[0][i], pos[1][i]);
        dimXYPosHash[ids[i]] = p;
    }
    for (i = 0; i < 2; i++)
    {
        delete [] pos[i];
    }
    delete [] pos;
    delete [] ids;

    return true;
}


bool DocumentManager::OnlineCreateMDSSelectedDimensionPos()
{
    EmptyDimXYPosHash();
    if (cov_matrix == NULL)
    {
        GetDimensionCovMatrix();
    }

    int i, j;
    int dims = selectedDimHash.size();
    int *dimIDs = new int [dims];
    int *ids = new int[dims];
    IIHash::iterator it;
    i = 0;
    for (it = selectedDimHash.begin(); it != selectedItemHash.end(); it++)
    {
        ids[i] = it->first;
        i++;
    }

    double ** scov_matrix;
    scov_matrix = new double* [dims];
    for (i = 0; i < dims; i++)
        scov_matrix[i] = new double [dims];

    for (i = 0; i < dims; i++)
        for (j = 0; j < dims; j++)
            scov_matrix[i][j] = cov_matrix[ids[i]][ids[j]];

    double **pos;
    pos = new double* [2];
    for (i = 0; i < 2; i++)
    {
        pos[i] = new double [dimNum];
    }

    mds(dims, 1, scov_matrix,  2, pos, 20, 20, 0);
    
    for (i = 0; i < dims; i++)
    {
        delete [] scov_matrix[i];
    }
    delete [] scov_matrix;

    //put the positions into the position hash
    Vec2* p;
    for (i = 0; i < dims; i++)
    {
        p = new Vec2(pos[0][i], pos[1][i]);
        dimXYPosHash[ids[i]] = p;
    }
    for (i = 0; i < 2; i++)
    {
        delete [] pos[i];
    }
    delete [] pos;
    delete [] ids;

    return true;
}


void DocumentManager::ResetImagePositions(int i)
{
    if (IsEmpty())
        return;

    if ( i < 0 || i > 5)
        return;

    imageDisplayMode = i;

    if (i == 0 || i == 1)
    {
        if (preImageDisplayMode != 0 && preImageDisplayMode != 1)
            EmptyItemXYPosHash();

        if (itemXYPosHash.size() == 0)
             CreateMDSPosForItems();
    }
    else if (i == 2)
    {
        OnlineCreateOneByOneSelectedItemPos();
    }
    else if (i == 3)
    {
        OnlineCreateMDSSelectedItemPos();
    }
    else 
    {
        OnlineCreateAroundCenterItemPos();
    }

    preImageDisplayMode = imageDisplayMode;

}

void DocumentManager::ResetVarPositions(int i)
{
    if ( i < 0 || i > 3)
        return;

    varDisplayMode = i;

    if (i == 0 || i == 1)
    {
        if (dimXYPosHash.size() == 0)
        {
             CreateMDSPosForDimensions();
        }
        else
        {
            if (preVarDisplayMode != 0 && preVarDisplayMode != 1 && preVarDisplayMode != 2)
            {
                if (centerDimID != -1)
                    OnlineCreateAroundCenterPos();
                else 
                    CreateMDSPosForDimensions();
            }
            else if (preVarDisplayMode == 2 && i == 0)
                CreateMDSPosForDimensions();
        }
    }
    else if (i == 2)
    {
//        OnlineCreateOneByOneSelectedDimensionPos();
        CreateCurveTreemapPosForDimensions();
    }
    else
    {
        OnlineCreateMDSSelectedDimensionPos();
    }

    if (i == 1 || i == 3)
        SetUseSelectedDimensionSpace(true);
    else if (i == 0 || i == 2)
        SetUseSelectedDimensionSpace(false);

    preVarDisplayMode = varDisplayMode;

}

void DocumentManager::ResetVarLabelingMode(int i)
{
    if ( i < 0 || i > 3)
        return;

//    labelDisplayMode = i;

    if (i == 0 || i == 1)
    {
        labelDimHash = NULL;
    }
    else if (i == 2)
    {
        labelDimHash = &selectedDimHash;
    }
    else if (i == 3)
    {
        labelDimHash = &manualLabelDimHash;
    }
//    preVarDisplayMode = varDisplayMode;
}

void DocumentManager::SetSamplingMode(int i)
{
    if (i == 0)
    {
        query->SetSampling(false);
    }
    else
    {
        query->SetSampling(true);
    }
}

void DocumentManager::SetBinaryMode(int i)
{
    if (i == 0)
        binaryMode = false;
    else
        binaryMode = true;
}


int DocumentManager::GetDimensionIDByName(std::string dimName)
{
    if (dimName.compare("") == 0)
        return -1;

    return allDimSpace->GetDimension(dimName)->GetDimensionID();
}

Dimension* DocumentManager::GetDimensionByID(int dimID)
{
    return allDimSpace->GetDimensionByID(dimID);

}

void DocumentManager::SetVarGlyphMode(int i)
{
    if (i != 4)
    {
        if (i == 3)
            query->SetQueryMode(SCATTER_QUERY);
        else
            query->SetQueryMode(SINGLE_QUERY);
    }
}

