//GLFrame is a frame with a gl canvas in it. Basic canvas 
//operations, such as zooming, panning, image saving can be 
//performed in this frame

//if you need to create a "drawable" window, you can derived it from this class  

#include <string>
#include <iostream>
#include <sstream>
#include <wx/wx.h>
#include <wx/glcanvas.h>
#include <wx/app.h>
#include "GLFrame.h"
#include "grCanvas.h"

BEGIN_EVENT_TABLE(GLFrame, wxFrame)
    EVT_MENU(MENU_SAVEIMAGE, GLFrame::OnSaveImage)
    
    EVT_SCROLL(GLFrame::OnScrBarThumbTrack)
    EVT_SCROLL(GLFrame::OnScrBarThumbTrack)
    
    EVT_TEXT_ENTER(TEXT_XZOOM, GLFrame::OnXZoomTextEnter)
    EVT_TEXT_ENTER(TEXT_YZOOM, GLFrame::OnYZoomTextEnter)
    
    EVT_BUTTON(BUTTON_XZOOMPP, GLFrame::OnXZoomPlusPlus)
    EVT_BUTTON(BUTTON_XZOOMP, GLFrame::OnXZoomPlus)
    EVT_BUTTON(BUTTON_XZOOMR, GLFrame::OnXZoomReset)
    EVT_BUTTON(BUTTON_XZOOMMM, GLFrame::OnXZoomMinusMinus)
    EVT_BUTTON(BUTTON_XZOOMM, GLFrame::OnXZoomMinus)

    EVT_BUTTON(BUTTON_YZOOMPP, GLFrame::OnYZoomPlusPlus)
    EVT_BUTTON(BUTTON_YZOOMP, GLFrame::OnYZoomPlus)
    EVT_BUTTON(BUTTON_YZOOMR, GLFrame::OnYZoomReset)
    EVT_BUTTON(BUTTON_YZOOMMM, GLFrame::OnYZoomMinusMinus)
    EVT_BUTTON(BUTTON_YZOOMM, GLFrame::OnYZoomMinus)
//	EVT_LEFT_DOWN(GLFrame:
END_EVENT_TABLE()

GLFrame::GLFrame(wxFrame *parent, int id, const wxChar *title, int xpos, int ypos, int width, int height)
        :Frame(parent, id, title, xpos, ypos, width, height)
{
	//set menu bar
	menuBar = new wxMenuBar;
	SetMenuBar(menuBar);
	//set status bar
    CreateStatusBar(3);
	SetStatusText("Ready", 0);

	//the sizer contains all stuffs
	topSizer = new wxBoxSizer( wxVERTICAL );
	SetSizer( topSizer );      

	//for child classes: quick buttons can be added into quickButtonSizer

	quickButtonSizer = new wxBoxSizer( wxHORIZONTAL );

    wxStaticText* xZoomLabel = new wxStaticText(this, -1, "XZoom");

    xZoomPlusPlusButton = new wxButton(this, BUTTON_XZOOMPP, " ++ ", wxPoint(-1,-1), wxSize(20,-1));
    xZoomPlusButton = new wxButton(this, BUTTON_XZOOMP, " + ", wxPoint(-1,-1), wxSize(15,-1));
    xZoomResetButton = new wxButton(this, BUTTON_XZOOMR, "Reset", wxPoint(-1,-1), wxSize(15,-1));
    xZoomMinusButton = new wxButton(this, BUTTON_XZOOMM, " - ", wxPoint(-1,-1), wxSize(15,-1));
    xZoomMinusMinusButton = new wxButton(this, BUTTON_XZOOMMM, " - - ", wxPoint(-1,-1), wxSize(20,-1));
    xZoomText = new wxTextCtrl(this, TEXT_XZOOM, "1.0", wxPoint(-1,-1), wxSize(40,-1));

    quickButtonSizer->Add(xZoomLabel, 1, wxALIGN_CENTRE_VERTICAL | wxLEFT | wxRIGHT , 2);
    quickButtonSizer->Add(xZoomPlusPlusButton, 1, wxEXPAND | wxALL, 1);
    quickButtonSizer->Add(xZoomPlusButton, 1, wxEXPAND | wxALL, 1);
    quickButtonSizer->Add(xZoomResetButton, 1, wxEXPAND | wxALL, 1);
    quickButtonSizer->Add(xZoomMinusButton, 1, wxEXPAND | wxALL, 1);
    quickButtonSizer->Add(xZoomMinusMinusButton, 1, wxEXPAND | wxALL, 1);
    quickButtonSizer->Add(xZoomText, 1, wxALIGN_CENTRE_VERTICAL, 1);

    wxStaticText* yZoomLabel = new wxStaticText(this, -1, "YZoom");

    yZoomPlusPlusButton = new wxButton(this, BUTTON_YZOOMPP, " ++ ", wxPoint(-1,-1), wxSize(20,-1));
    yZoomPlusButton = new wxButton(this, BUTTON_YZOOMP, " + ", wxPoint(-1,-1), wxSize(15,-1));
    yZoomResetButton = new wxButton(this, BUTTON_YZOOMR, "Reset", wxPoint(-1,-1), wxSize(15,-1));
    yZoomMinusMinusButton = new wxButton(this, BUTTON_YZOOMMM, " - - ", wxPoint(-1,-1), wxSize(20,-1));
    yZoomMinusButton = new wxButton(this, BUTTON_YZOOMM, " - ", wxPoint(-1,-1), wxSize(15,-1));
    yZoomText = new wxTextCtrl(this, TEXT_YZOOM, "1.0", wxPoint(-1,-1), wxSize(40,-1));

    quickButtonSizer->Add(yZoomLabel, 1, wxALIGN_CENTRE_VERTICAL | wxLEFT | wxRIGHT , 2);
    quickButtonSizer->Add(yZoomPlusPlusButton, 1, wxEXPAND | wxALL, 1);
    quickButtonSizer->Add(yZoomPlusButton, 1, wxEXPAND | wxALL, 1);
    quickButtonSizer->Add(yZoomResetButton, 1, wxEXPAND | wxALL, 1);
    quickButtonSizer->Add(yZoomMinusButton, 1, wxEXPAND | wxALL, 1);
    quickButtonSizer->Add(yZoomMinusMinusButton, 1, wxEXPAND | wxALL, 1);
    quickButtonSizer->Add(yZoomText, 1, wxALIGN_CENTRE_VERTICAL, 1);
    
    
//	wxPanel *quickButtonPanel = new wxPanel(this, -1, wxPoint(-1,-1), wxSize(-1,-1));
//    quickButtonSizer->Add(quickButtonPanel, 1, wxEXPAND | wxALL, 0);
   

    topSizer->Add(quickButtonSizer, 0, wxEXPAND | wxALL, 1);



	//for child classes: a sizer containing right buttons can be added into majorSizer
	majorSizer = new wxBoxSizer( wxHORIZONTAL );

    topSizer->Add(majorSizer, 1, wxEXPAND | wxALL, 1); 

	wxFlexGridSizer* scrWinSizer = new wxFlexGridSizer(2, 2, 0, 0);
	majorSizer->Add(scrWinSizer, 1, wxEXPAND | wxALL, 0);

	scrWinSizer->AddGrowableCol(0);
	scrWinSizer->AddGrowableRow(0);

	vScrBar = new wxScrollBar(this, VSCRBAR, wxPoint(-1,-1), wxSize(-1,-1), 
		                     wxSB_VERTICAL , wxDefaultValidator, "vScrBar");
	hScrBar = new wxScrollBar(this, VSCRBAR, wxPoint(-1,-1), wxSize(-1,-1), 
	                         wxSB_HORIZONTAL, wxDefaultValidator, "hScrBar");
		
	canvas = new grCanvas(this, -1, wxPoint(-1,-1), wxSize(-1,-1));
    canvas->SetFrame(this);

	wxPanel *cornerPanel = new wxPanel(this, -1, wxPoint(-1,-1), wxSize(-1,-1));

	scrWinSizer->Add(canvas, 1, wxEXPAND | wxALL, 0); 
	scrWinSizer->Add(vScrBar, 1, wxEXPAND | wxALL, 0); 
	scrWinSizer->Add(hScrBar, 1, wxEXPAND | wxALL, 0); 
	scrWinSizer->Add(cornerPanel, 1, wxEXPAND | wxALL, 0); 

	wxMenu *saveMenu = new wxMenu;
	saveMenu->Append(MENU_SAVEIMAGE,  "&Save Image");
	menuBar->Append(saveMenu, "&Save");

    SetVScrollBar();
    SetHScrollBar();
}

GLFrame::~GLFrame()
{
}

void GLFrame::SetVScrollBar()
{
    float zoom = canvas->GetYZoom();
    if (fabs(zoom - 1) < 1e-5)
        canvas->SetYTranslate(0);

    float translate = fabs(canvas->GetYTranslate());


    int range = 1000;
    int thumb = range / zoom;

//    int pos = range * (translate + 1 - 1/zoom);
    int pos = range * translate;
    vScrBar->SetScrollbar(pos, thumb, range, thumb);
}

void GLFrame::SetHScrollBar()
{
    float zoom = canvas->GetXZoom();
    if (fabs(zoom - 1) < 1e-5)
        canvas->SetXTranslate(0);

    float translate = fabs(canvas->GetXTranslate());

    int range = 1000;
    int thumb = range / zoom;
    hScrBar->SetScrollbar(translate * range, thumb, range, thumb);
}

void GLFrame::OnScrBarThumbTrack(wxScrollEvent& event)
{
    int pos = hScrBar->GetThumbPosition();
    int range = hScrBar->GetRange();
    canvas->SetXTranslate(-1. * (float)pos/range);

    pos = vScrBar->GetThumbPosition();
    range = vScrBar->GetRange();
    float zoom = canvas->GetYZoom();
//    canvas->SetYTranslate((float)pos/range - 1 + 1./zoom);
    canvas->SetYTranslate(-1. * (float)pos/range);
    
    canvas->Refresh();
}

void GLFrame::OnSaveImage(wxCommandEvent& event)
{
    wxFileDialog* dlg = new wxFileDialog(this);
    dlg->SetStyle(wxSAVE|wxOVERWRITE_PROMPT);
    dlg->SetWildcard("bmp files (*.bmp)|*.bmp");

    if (dlg->ShowModal() == wxID_CANCEL)
    {
        delete dlg;
        return;
    }

    std::string fileName = dlg->GetPath().c_str();

	canvas->WriteImage(fileName);
}

void GLFrame::OnXZoomTextEnter(wxCommandEvent& event)
{
    canvas->SetXZoom(atof(xZoomText->GetValue().c_str()));
    SetHScrollBar();
    canvas->Refresh();
}

void GLFrame::OnYZoomTextEnter(wxCommandEvent& event)
{
    canvas->SetYZoom(atof(yZoomText->GetValue().c_str()));
    SetVScrollBar();
    canvas->Refresh();
}

void GLFrame::OnXZoomPlusPlus(wxCommandEvent& event)
{
    float x = canvas->GetXZoom() * 2;
    canvas->SetXZoom(x);
 //   std::string str(x);
 //   xZoomText->SetValue(str);
    std::ostringstream oss;
    oss << x;
    xZoomText->SetValue(oss.str().c_str());
    SetHScrollBar();
    canvas->Refresh();
}

void GLFrame::OnXZoomPlus(wxCommandEvent& event)
{
    float x = canvas->GetXZoom() * 1.1;
    canvas->SetXZoom(x);
    std::ostringstream oss;
    oss << x;
    xZoomText->SetValue(oss.str().c_str());
    SetHScrollBar();
    canvas->Refresh();
}

void GLFrame::OnXZoomReset(wxCommandEvent& event)
{
    float x = 1.;
    canvas->SetXZoom(x);
    std::ostringstream oss;
    oss << x;
    xZoomText->SetValue(oss.str().c_str());
    SetHScrollBar();
    canvas->Refresh();
}

void GLFrame::OnXZoomMinusMinus(wxCommandEvent& event)
{
    float x = canvas->GetXZoom() / 2.;
    canvas->SetXZoom(x);
    std::ostringstream oss;
    oss << x;
    xZoomText->SetValue(oss.str().c_str());
    SetHScrollBar();
    canvas->Refresh();
}

void GLFrame::OnXZoomMinus(wxCommandEvent& event)
{
    float x = canvas->GetXZoom() / 1.1;
    canvas->SetXZoom(x);
    std::ostringstream oss;
    oss << x;
    xZoomText->SetValue(oss.str().c_str());
    SetHScrollBar();
    canvas->Refresh();
}

void GLFrame::OnYZoomPlusPlus(wxCommandEvent& event)
{
    float x = canvas->GetYZoom() * 2;
    canvas->SetYZoom(x);
    std::ostringstream oss;
    oss << x;
    yZoomText->SetValue(oss.str().c_str());
    SetVScrollBar();
    canvas->Refresh();
}

void GLFrame::OnYZoomPlus(wxCommandEvent& event)
{
    float x = canvas->GetYZoom() * 1.1;
    canvas->SetYZoom(x);
    std::ostringstream oss;
    oss << x;
    yZoomText->SetValue(oss.str().c_str());
    SetVScrollBar();
    canvas->Refresh();
}

void GLFrame::OnYZoomReset(wxCommandEvent& event)
{
    float x = 1.;
    canvas->SetYZoom(x);
    std::ostringstream oss;
    oss << x;
    yZoomText->SetValue(oss.str().c_str());
    SetVScrollBar();
    canvas->Refresh();
}

void GLFrame::OnYZoomMinusMinus(wxCommandEvent& event)
{
    float x = canvas->GetYZoom() / 2.;
    canvas->SetYZoom(x);
    std::ostringstream oss;
    oss << x;
    yZoomText->SetValue(oss.str().c_str());
    SetVScrollBar();
    canvas->Refresh();
}

void GLFrame::OnYZoomMinus(wxCommandEvent& event)
{
    float x = canvas->GetYZoom() / 1.1;
    canvas->SetYZoom(x);
    std::ostringstream oss;
    oss << x;
    yZoomText->SetValue(oss.str().c_str());
    SetVScrollBar();
    canvas->Refresh();
}

