// The interface for setting control modes and parameters of the Glyph display


#include <wx/wx.h>
#include<wx/glcanvas.h>
#include <wx/app.h>
#include <wx/colordlg.h>


#include <string>
#include <iostream>
#include <fstream>
#include <sstream>

#include "MDSControlFrame.h"
#include "MainApplication.h"
#include "../DataStructure/DocLabel.h"
#include "UserNotifier.h"


BEGIN_EVENT_TABLE(MDSControlFrame, wxFrame)
    EVT_SCROLL(MDSControlFrame::OnScroll)
    EVT_COMBOBOX(COMBO_BASEItem, MDSControlFrame::SetBaseItem) 
//    EVT_COMBOBOX(COMBO_GRPItem, MDSControlFrame::SetGroupItem) 
//    EVT_COMBOBOX(COMBO_GRPVALUEItem, MDSControlFrame::SetSelectedGroupValue) 
    EVT_BUTTON(BUTTON_RANDOMZ, MDSControlFrame::RandomizeZ)
    EVT_BUTTON(BUTTON_RESAMPLEIMAGE, MDSControlFrame::Resample)
    EVT_RADIOBOX(RADIO_SELECTIIONDISPLAY, MDSControlFrame::SetSelectionDisplayMode)
    EVT_CLOSE(MDSControlFrame::WindowClose)
END_EVENT_TABLE()

MDSControlFrame::MDSControlFrame(wxFrame *parent, int id, const wxChar *title, int xpos, int ypos, int width, int height)
        :Frame(parent, id, title, xpos, ypos, width, height)
{
    SetBackgroundColour(wxColour(*wxLIGHT_GREY));
    SetForegroundColour(wxColour(*wxBLUE));
//    SetModal(false);
	//set menu bar
	menuBar = new wxMenuBar;
	SetMenuBar(menuBar);
	//set status bar
    CreateStatusBar(3);
	SetStatusText("Ready", 0);

	//the sizer contains all stuffs
	topSizer = new wxBoxSizer( wxVERTICAL );
	SetSizer( topSizer );      

	wxPanel *quickButtonPanel = new wxPanel(this, -1, wxPoint(-1,-1), wxSize(-1,-1));
	//for child classes: quick buttons can be added into quickButtonSizer
	quickButtonSizer = new wxBoxSizer( wxHORIZONTAL );
	quickButtonPanel->SetSizer(quickButtonSizer);
	
	//for child classes: a sizer containing right buttons can be added into majorSizer
	majorSizer = new wxBoxSizer( wxHORIZONTAL );

    wxStaticBox *box1, *box2;
    box1 = new wxStaticBox(this, -1, "Glyph Size", wxPoint(-1,-1), wxSize(-1,-1),
        0, "staticBox");
    box2 = new wxStaticBox(this, -1, "Distortion Factor", wxPoint(-1,-1), wxSize(-1,-1),
        0, "staticBox");

    sizeSlider = new wxSlider(this, -1, 0.001, 0, 500, wxPoint(-1,-1), wxSize(-1,-1),
        wxSL_HORIZONTAL, wxDefaultValidator, "slider");

	distortionSlider = new wxSlider(this, -1, 0.001, 0, 1000, wxPoint(-1,-1), wxSize(-1,-1),
        wxSL_HORIZONTAL, wxDefaultValidator, "slider");

    wxStaticBoxSizer *boxSizer1 = new wxStaticBoxSizer (box1, wxVERTICAL);
    boxSizer1->Add(sizeSlider, 1, wxEXPAND | wxALL, 0);

    wxStaticBoxSizer *boxSizer2 = new wxStaticBoxSizer (box2, wxVERTICAL);
    boxSizer2->Add(distortionSlider, 1, wxEXPAND | wxALL, 0);
    
    majorSizer->Add(boxSizer1 , 1, wxEXPAND | wxALL, 0);
	majorSizer->Add(boxSizer2 , 1, wxEXPAND | wxALL, 0);

    topSizer->Add(majorSizer, 0, wxEXPAND | wxALL, 1); 
  
    /////////////////////////////////////////////

    maskColorButton = new wxButton(this, BUTTON_RANDOMZ, "Randomize Layers", wxPoint(-1,-1), wxSize(5,-1));
    resampleButton = new wxButton(this, BUTTON_RESAMPLEIMAGE, "Resample", wxPoint(-1,-1), wxSize(5,-1));

    wxBoxSizer *sampleSizer1= new wxBoxSizer(wxHORIZONTAL);
    sampleSizer1->Add(maskColorButton, 1, wxEXPAND | wxALL, 0);
    sampleSizer1->AddSpacer(10);
    sampleSizer1->Add(resampleButton, 1, wxEXPAND | wxALL, 0);

    topSizer->Add(sampleSizer1, 0, wxEXPAND | wxALL, 1); 

    wxStaticBox *box7;
    box7 = new wxStaticBox(this, -1, "Search by Sample Image", wxPoint(-1,-1), wxSize(-1,-1),
        0, "staticBox");

    wxStaticBoxSizer *boxSizer7 = new wxStaticBoxSizer (box7, wxVERTICAL);

    baseItemComboBox = new wxComboBox(this, COMBO_BASEItem, "", wxDefaultPosition, wxDefaultSize, 
        0, NULL, wxCB_SORT, wxDefaultValidator, "comboBox");

    simiSelectSlider = new wxSlider(this, -1, 0.001, 0, 1000, wxPoint(-1,-1), wxSize(-1,-1),
        wxSL_HORIZONTAL, wxDefaultValidator, "slider");

    wxString choices[6];
    choices[0] = "Display All (MDS)";
    choices[1] = "Selected Only (Original)";
    choices[2] = "Selected Only (One by One)";
    choices[3] = "Selected Only (RE-MDS)";
    choices[4] = "Display All (Rainfall)";
    choices[5] = "Selected Only (Raifall)";
    selectionDisplayRadioBox = new wxRadioBox(this, RADIO_SELECTIIONDISPLAY, "Display Mode",
        wxDefaultPosition, wxDefaultSize, 6, choices, 2, wxRA_SPECIFY_COLS, 
        wxDefaultValidator, "radioBox");

    boxSizer7->Add(baseItemComboBox, 0, wxEXPAND | wxALL, 0);
    boxSizer7->Add(simiSelectSlider, 0, wxEXPAND | wxBOTTOM | wxTOP, 4);
    boxSizer7->Add(selectionDisplayRadioBox, 0, wxEXPAND | wxALL, 0);

	topSizer->Add(boxSizer7, 0, wxEXPAND | wxALL, 1); 

    wxStaticBox *boxa;
    boxa = new wxStaticBox(this, -1, "Rainfall Time Controller", wxPoint(-1,-1), wxSize(-1,-1),
        0, "staticBox");

    wxStaticBoxSizer *boxSizera = new wxStaticBoxSizer (boxa, wxVERTICAL);

    speedSlider = new wxSlider(this, -1, 0.001, 0, 2000, wxPoint(-1,-1), wxSize(-1,-1),
        wxSL_HORIZONTAL, wxDefaultValidator, "slider");

    boxSizera->Add(speedSlider, 1, wxEXPAND | wxBOTTOM | wxTOP, 3);

	topSizer->Add(boxSizera, 0, wxEXPAND | wxALL, 1); 

    Fit();

    pre_simivalue = -10;
    pre_displayMode = 0;
}

void MDSControlFrame::Reset()
{
     baseItemComboBox->Clear();

     baseItemComboBox->Append("");
     
     selectionDisplayRadioBox->SetSelection(0);

     Iterator* it = docManager->GetItemNameIterator();
     std::string name;
     DocLabel* label;
     while (!it->IsDone())
     {
         label = (DocLabel*)(it->GetNext());
         name = label->GetText();
         baseItemComboBox->Append(name.c_str());
     }
     delete it;
}


MDSControlFrame::~MDSControlFrame()
{
}


void MDSControlFrame::OnScroll(wxScrollEvent& event)
{
    wxObject* object = event.GetEventObject();
    if (object == (wxObject *)sizeSlider)
    {
        int value = sizeSlider->GetValue();
        viewManager->GlyphSizeChanged(value/1000.f);
    }
    else if (object == (wxObject *)distortionSlider)
    {
        int value = distortionSlider->GetValue();
        viewManager->GlyphDistortionChanged(value/1000.f*5);
    }
    else if (object == (wxObject *)simiSelectSlider)
    {
        int value = simiSelectSlider->GetValue() - 1;
        if (value != pre_simivalue)
        {
            pre_simivalue = value;
            docManager->SelectItemsAroundBase(value/1000.f);
            int i = selectionDisplayRadioBox->GetSelection();

            if (i == 2)
            {
                docManager->OnlineCreateOneByOneSelectedItemPos();
            }
            else if (i == 3)
            {
                docManager->OnlineCreateMDSSelectedItemPos();
            }
            viewManager->SelectedItemsChanged();
        }
    }
    else if (object == (wxObject *)speedSlider)
    {
        int i = selectionDisplayRadioBox->GetSelection();

        if (i == 4 || i == 5)
        {    
            int value = speedSlider->GetValue();
            docManager->SetCenterItemT(value/1000.f * 100);
            docManager->OnlineCreateAroundCenterItemPos();
            viewManager->GlyphPositionReset();
        }
    }
}

void MDSControlFrame::SetBaseItem(wxCommandEvent& event)
{
    std::string str = baseItemComboBox->GetStringSelection();
    int value = simiSelectSlider->GetValue() - 1;

    int id;
    docManager->SetBaseItem(str, id);
    viewManager->BaseItemReset(id);
    docManager->SelectItemsAroundBase(value/1000.f);
    viewManager->SelectedItemsChanged();
}

void MDSControlFrame::SetBaseItem(std::string str)
{
    baseItemComboBox->SetValue(str.c_str());
    int value = simiSelectSlider->GetValue() - 1;

    int id;
    docManager->SetBaseItem(str, id);
    viewManager->BaseItemReset(id);
    docManager->SelectItemsAroundBase(value/1000.f);
    viewManager->SelectedItemsChanged();
}

void MDSControlFrame::RandomizeZ(wxCommandEvent& event)
{
     viewManager->GlyphZPosRandomReset();
}

void MDSControlFrame::SetSelectionDisplayMode(wxCommandEvent& event)
{
    if (docManager->IsEmpty())
    {
        selectionDisplayRadioBox->SetSelection(0);
        return;
    }

    int i = selectionDisplayRadioBox->GetSelection();

    if (i == 4 || i == 5) 
    {
        if (docManager->GetBaseItemID() == -1)
        {
            selectionDisplayRadioBox->SetSelection(docManager->GetPrevImageDisplayMode());
            UserNotifier::ShowMessageDialog("", "Please select a sample image");
            return;
        }
    }

    docManager->ResetImagePositions(i);
    viewManager->GlyphDisplayModeReset(i);
    pre_displayMode = i;
}

void MDSControlFrame::WindowClose(wxCloseEvent& event)
{
    MainFrame *frame = (MainFrame *)this->GetParent();
    frame->SetImageFrame(NULL);
    Destroy();
}

void MDSControlFrame::Resample(wxCommandEvent& event)
{
    docManager->GetQuery()->Resample();
    viewManager->RefreshGlyphView();

    baseItemComboBox->Clear();

    baseItemComboBox->Append("");

    Iterator* it = docManager->GetItemNameIterator();
    std::string name;
    DocLabel* label;
    while (!it->IsDone())
    {
        label = (DocLabel*)(it->GetNext());
        name = label->GetText();
        baseItemComboBox->Append(name.c_str());
    }
    delete it;

}
