// The interface for setting control modes and parameters of the var display


#include <wx/wx.h>
#include<wx/glcanvas.h>
#include <wx/app.h>
#include <wx/colordlg.h>


#include <string>
#include <iostream>
#include <fstream>
#include <sstream>

#include "VarControlFrame.h"
#include "MainFrame.h"
#include "../DataStructure/DocLabel.h"
#include "../Basic/Iterator.h"
#include "../DocumentManager.h"
#include "../ViewManager.h"

BEGIN_EVENT_TABLE(VarControlFrame, wxFrame)
    EVT_SCROLL(VarControlFrame::OnScroll)
    EVT_COMBOBOX(COMBO_ACTDIM, VarControlFrame::SetActiveDimension) 
    EVT_COMBOBOX(COMBO_BASEDIM, VarControlFrame::SetBaseDimension) 
    EVT_COMBOBOX(COMBO_SCATTERDIM, VarControlFrame::SetVarGlyphBaseDimension) 
    EVT_COMBOBOX(COMBO_GRPDIM, VarControlFrame::SetGroupDimension) 
    EVT_COMBOBOX(COMBO_GRPVALUE, VarControlFrame::SetSelectedGroupValue) 
    EVT_COMBOBOX(COMBO_XDIM, VarControlFrame::SetXDimension) 
    EVT_COMBOBOX(COMBO_YDIM, VarControlFrame::SetYDimension) 
    EVT_COMBOBOX(COMBO_CENTERDIM, VarControlFrame::SetCenterDimension) 
    EVT_RADIOBOX(RADIO_DIMDISPLAYMODE, VarControlFrame::SetDimensionDisplayMode)
    EVT_RADIOBOX(RADIO_DIMSELECTMODE, VarControlFrame::SetDimensionSelectMode)
    EVT_RADIOBOX(RADIO_DIMSELECTOPTION, VarControlFrame::SetDimensionDistinctSelectOption)
    EVT_RADIOBOX(RADIO_LABELDISPLAYMODE, VarControlFrame::SetLabelDisplayMode)
    EVT_RADIOBOX(RADIO_SAMPLINGMODE, VarControlFrame::SetSamplingMode)
    EVT_RADIOBOX(RADIO_BINARYMODE, VarControlFrame::SetBinaryMode)
    EVT_RADIOBOX(RADIO_VARGLYPHMODE, VarControlFrame::SetVarGlyphMode)
    EVT_BUTTON(BUTTON_MASKCOLOR, VarControlFrame::SetMaskColor)
    EVT_BUTTON(BUTTON_RANDOMVARZ, VarControlFrame::RandomizeZ)
    EVT_BUTTON(BUTTON_RESAMPLE, VarControlFrame::Resample)
    EVT_CHECKBOX(CHECKBOX_ORDER, VarControlFrame::SetPixelIncreaseOrder)
    EVT_TEXT_ENTER(TEXTCTRL_SAMPLESIZE, VarControlFrame::SetSampleSize)
    EVT_CLOSE(VarControlFrame::WindowClose)
END_EVENT_TABLE()

VarControlFrame::VarControlFrame(wxFrame *parent, int id, const wxChar *title, int xpos, int ypos, int width, int height)
        :Frame(parent, id, title, xpos, ypos, width, height)
{
    SetBackgroundColour(wxColour(*wxLIGHT_GREY));
    SetForegroundColour(wxColour(*wxBLUE));
//    SetModal(false);
	//set menu bar
	menuBar = new wxMenuBar;
	SetMenuBar(menuBar);
	//set status bar
    CreateStatusBar(3);
	SetStatusText("Ready", 0);

	//the sizer contains all stuffs
	topSizer = new wxBoxSizer( wxVERTICAL );
	SetSizer( topSizer );      


    wxString schoices[2];
    schoices[0] = "No sampling";
    schoices[1] = "Simple random sampling";
    samplingModeRadioBox = new wxRadioBox(this, RADIO_SAMPLINGMODE, "Sampling Mode",
        wxDefaultPosition, wxDefaultSize, 2, schoices, 2, wxRA_SPECIFY_COLS, 
        wxDefaultValidator, "radioBox");

    sampleSizeTextCtrl = new wxTextCtrl(this, TEXTCTRL_SAMPLESIZE, "500");
    resampleButton = new wxButton(this, BUTTON_RESAMPLE, "Resample", wxPoint(-1,-1), wxSize(-1,-1));

    wxStaticText *sampleLabel = new wxStaticText(this, -1, "Approx. Sample Size");
    wxBoxSizer *sampleSizer1= new wxBoxSizer(wxHORIZONTAL);
    sampleSizer1->Add(sampleLabel , 0, wxEXPAND | wxALL, 1);
    sampleSizer1->Add(sampleSizeTextCtrl, 1, wxEXPAND | wxALL, 0);
    sampleSizer1->AddSpacer(10);
    sampleSizer1->Add(resampleButton, 1, wxEXPAND | wxALL, 0);

	topSizer->Add(samplingModeRadioBox, 0, wxEXPAND | wxALL, 1); 
	topSizer->Add(sampleSizer1, 0, wxEXPAND | wxALL, 1); 

    wxString lchoices[4];
    lchoices[0] = "Display All";
    lchoices[1] = "Hide All";
    lchoices[2] = "Selected Only";
 //   lchoices[3] = "Manually Picked Only";
    labelDisplayRadioBox = new wxRadioBox(this, RADIO_LABELDISPLAYMODE, "Labeling Mode",
        wxDefaultPosition, wxDefaultSize, 3, lchoices, 3, wxRA_SPECIFY_COLS, 
        wxDefaultValidator, "radioBox");

	topSizer->Add(labelDisplayRadioBox, 0, wxEXPAND | wxALL, 1); 

    wxString choices[4];
    choices[0] = "Display All (MDS or Rainfall)";
    choices[1] = "Selected Only (MDS or Rainfall)";
    choices[2] = "Display All (Treemap)";
    choices[3] = "Selected Only (RE-MDS)";
    selectionDisplayRadioBox = new wxRadioBox(this, RADIO_DIMDISPLAYMODE, "Dimension Layout",
        wxDefaultPosition, wxDefaultSize, 4, choices, 2, wxRA_SPECIFY_COLS, 
        wxDefaultValidator, "radioBox");

    topSizer->Add(selectionDisplayRadioBox, 0, wxEXPAND | wxALL, 1); 

    /////////////////////////////////////////////
    wxStaticBox *boxa;
    boxa = new wxStaticBox(this, -1, "Rainfall Positioning (select Empty for MDS Positioning)", wxPoint(-1,-1), wxSize(-1,-1),
        0, "staticBox");

    wxStaticBoxSizer *boxSizera = new wxStaticBoxSizer (boxa, wxVERTICAL);

    centerDimComboBox = new wxComboBox(this, COMBO_CENTERDIM, "", wxDefaultPosition, wxDefaultSize, 
        0, NULL, wxCB_SORT, wxDefaultValidator, "comboBox");
    boxSizera->Add(centerDimComboBox, 1, wxEXPAND | wxALL, 0);

    speedSlider = new wxSlider(this, -1, 0.001, 0, 1000, wxPoint(-1,-1), wxSize(-1,-1),
        wxSL_HORIZONTAL, wxDefaultValidator, "slider");

    boxSizera->Add(speedSlider, 1, wxEXPAND | wxALL, 0);

	topSizer->Add(boxSizera, 0, wxEXPAND | wxALL, 1); 
    /////////////////////////////////////////////
    /////////////////////////////////////////////
    wxStaticBox *box11;
    box11 = new wxStaticBox(this, -1, "Select Dimensions", wxPoint(-1,-1), wxSize(-1,-1),
        0, "staticBox");

    wxStaticBoxSizer *boxSizer11 = new wxStaticBoxSizer (box11, wxVERTICAL);

    baseDimComboBox = new wxComboBox(this, COMBO_BASEDIM, "", wxDefaultPosition, wxDefaultSize, 
        0, NULL, wxCB_SORT, wxDefaultValidator, "comboBox");
    selectionLabel = new wxStaticText(this, -1, "Sample Dimension");
    wxBoxSizer *selectSizer1= new wxBoxSizer(wxHORIZONTAL);
    selectSizer1->Add(selectionLabel, 0, wxEXPAND | wxALL, 1);
    selectSizer1->Add(baseDimComboBox, 1, wxEXPAND | wxALL, 0);
     
    simiSelectSlider = new wxSlider(this, -1, 0.001, 0, 1000, wxPoint(-1,-1), wxSize(-1,-1),
        wxSL_HORIZONTAL, wxDefaultValidator, "slider");
    wxStaticText *label8 = new wxStaticText(this, -1, "Threshold");
    wxBoxSizer *selectSizer2= new wxBoxSizer(wxHORIZONTAL);
    selectSizer2->Add(label8, 0, wxEXPAND | wxALL, 1);
    selectSizer2->Add(simiSelectSlider, 1, wxEXPAND | wxALL, 0);
  
/*    selectAllButton = new wxButton(this, BUTTON_RANDOMVARZ, "Select All", wxPoint(-1,-1), wxSize(-1,-1));
    unselectAllButton = new wxButton(this, BUTTON_RANDOMVARZ, "Unselect All", wxPoint(-1,-1), wxSize(-1,-1));
    wxBoxSizer *selectSizer3= new wxBoxSizer(wxHORIZONTAL);
    selectSizer3->Add(selectAllButton, 0, wxHORIZONTAL, 0);
    selectSizer3->AddSpacer(20);
    selectSizer3->Add(unselectAllButton, 0, wxHORIZONTAL, 0); */

    wxString choices2[3];
    choices2[0] = "Similar to sample";
    choices2[1] = "Distinct to sample    ";
    choices2[2] = "Manual";
    selectionModeRadioBox = new wxRadioBox(this, RADIO_DIMSELECTMODE, "Selection Mode",
        wxDefaultPosition, wxDefaultSize, 3, choices2, 3, wxRA_SPECIFY_COLS, 
        wxDefaultValidator, "radioBox");

 /*   wxString choices3[2];
    choices3[0] = "Optimize for groups";
    choices3[1] = "Optimize for labeling";
    selectionOptionRadioBox = new wxRadioBox(this, RADIO_DIMSELECTOPTION, "Distinct Select Options",
        wxDefaultPosition, wxDefaultSize, 2, choices3, 2, wxRA_SPECIFY_COLS, 
        wxDefaultValidator, "radioBox");
*/
//    boxSizer11->Add(selectSizer3, 0, wxEXPAND | wxALL, 2);
    boxSizer11->Add(selectionModeRadioBox, 0, wxEXPAND | wxALL, 2);
    boxSizer11->Add(selectSizer1, 0, wxEXPAND | wxALL, 2);
    boxSizer11->Add(selectSizer2, 0, wxEXPAND | wxALL, 2);
//    boxSizer11->Add(selectionOptionRadioBox, 0, wxEXPAND | wxALL, 0);


	topSizer->Add(boxSizer11, 0, wxEXPAND | wxALL, 1); 

    
    wxStaticBox *glyphSizeBox;
    glyphSizeBox = new wxStaticBox(this, -1, "Glyph size and order", wxPoint(-1,-1), wxSize(-1,-1),
        0, "staticBox");
    wxStaticBoxSizer *glyphSizeBoxSizer = new wxStaticBoxSizer (glyphSizeBox, wxVERTICAL);

    sizeSlider = new wxSlider(this, -1, 0.001, 0, 1000, wxPoint(-1,-1), wxSize(-1,-1),
        wxSL_HORIZONTAL, wxDefaultValidator, "slider");

    wxStaticText *label1 = new wxStaticText(this, -1, "Size");
    wxBoxSizer *sizeSizer1= new wxBoxSizer(wxHORIZONTAL);
    sizeSizer1->Add(label1, 0, wxEXPAND | wxALL, 1);
    sizeSizer1->Add(sizeSlider, 1, wxEXPAND | wxALL, 0);
    glyphSizeBoxSizer->Add(sizeSizer1, 0, wxEXPAND | wxALL, 0);  

    distortionSlider = new wxSlider(this, -1, 0.001, 0, 1000, wxPoint(-1,-1), wxSize(-1,-1),
        wxSL_HORIZONTAL, wxDefaultValidator, "slider");
    randomButton = new wxButton(this, BUTTON_RANDOMVARZ, "Randomly Reorder", wxPoint(-1,-1), wxSize(-1,-1));

    wxStaticText *label2 = new wxStaticText(this, -1, "   Distortion");
    wxBoxSizer *sizeSizer2= new wxBoxSizer(wxHORIZONTAL);
//    sizeSizer2->Add(randomButton, 0, 0, 1);
    sizeSizer2->Add(randomButton, 0, 0, 0);  
    sizeSizer2->Add(label2, 0, wxEXPAND | wxALL, 1);
    sizeSizer2->Add(distortionSlider, 1, wxEXPAND | wxALL, 0);
    glyphSizeBoxSizer->Add(sizeSizer2, 1, wxEXPAND | wxALL, 0);  


    topSizer->Add(glyphSizeBoxSizer, 0, wxEXPAND | wxALL, 1); 
//    topSizer->Add(boxSizer1, 0, wxEXPAND | wxALL, 0); 
//    topSizer->Add(sliderWinSizer, 0, wxEXPAND | wxALL, 0); 

//	topSizer->Add(boxSizer14, 0, wxEXPAND | wxALL, 1); 


    /////////////////////////////////////////////
/*    wxStaticBox *box9;
    box9 = new wxStaticBox(this, -1, "X and Y Dimension", wxPoint(-1,-1), wxSize(-1,-1),
        0, "staticBox");

    wxStaticBoxSizer *boxSizer9 = new wxStaticBoxSizer (box9, wxVERTICAL);

    xDimComboBox = new wxComboBox(this, COMBO_XDIM, "", wxDefaultPosition, wxDefaultSize, 
        0, NULL, wxCB_SORT, wxDefaultValidator, "comboBox");
    boxSizer9->Add(xDimComboBox, 1, wxEXPAND | wxALL, 0);

    yDimComboBox = new wxComboBox(this, COMBO_YDIM, "", wxDefaultPosition, wxDefaultSize, 
        0, NULL, 0, wxDefaultValidator, "comboBox");
    
    boxSizer9->Add(yDimComboBox, 1, wxEXPAND | wxALL, 0);
	topSizer->Add(boxSizer9, 1, wxEXPAND | wxALL, 1); */

    /////////////////////////////////////////////
    /////////////////////////////////////////////
/*    wxString bchoices[2];
    bchoices[0] = "Non-binary mode";
    bchoices[1] = "Binary mode";
    binaryModeRadioBox = new wxRadioBox(this, RADIO_BINARYMODE, "Binary Mode",
        wxDefaultPosition, wxDefaultSize, 2, bchoices, 2, wxRA_SPECIFY_COLS, 
        wxDefaultValidator, "radioBox");

//    boxSizer15->Add(samplingModeRadioBox, 0, wxEXPAND | wxALL, 0);

//	topSizer->Add(boxSizer15, 0, wxEXPAND | wxALL, 1); 
	topSizer->Add(binaryModeRadioBox, 0, wxEXPAND | wxALL, 1); */


    /////////////////////////////////////////////
    wxStaticBox *styleBox;
    styleBox = new wxStaticBox(this, -1, "VaR Glyph Style", wxPoint(-1,-1), wxSize(-1,-1),
        0, "staticBox");
    wxStaticBoxSizer *styleBoxSizer = new wxStaticBoxSizer (styleBox, wxVERTICAL);

    wxString choices5[5];
    choices5[0] = "Pixel-Spiral";
    choices5[1] = "Pixel-Vertical";
    choices5[2] = "Pixel-Horizontal";
    choices5[3] = "Scatterplot";
    choices5[4] = "Histogram";
    VarGlyphModeRadioBox = new wxRadioBox(this, RADIO_VARGLYPHMODE, "Mode",
        wxDefaultPosition, wxDefaultSize, 5, choices5, 3, wxRA_SPECIFY_COLS, 
        wxDefaultValidator, "radioBox");

	styleBoxSizer->Add(VarGlyphModeRadioBox, 0, wxEXPAND | wxALL, 1); 

    scatterDimComboBox = new wxComboBox(this, COMBO_SCATTERDIM, "", wxDefaultPosition, wxDefaultSize, 
        0, NULL, wxCB_SORT, wxDefaultValidator, "comboBox");
    modeLabel = new wxStaticText(this, -1, "Order Dimension");
    wxBoxSizer *styleSizer1= new wxBoxSizer(wxHORIZONTAL);
    styleSizer1->Add(modeLabel, 0, wxEXPAND | wxALL, 1);
    styleSizer1->Add(scatterDimComboBox, 1, wxEXPAND | wxALL, 1);
    orderCheckBox = new wxCheckBox(this, CHECKBOX_ORDER, "Increasing Order");
    orderCheckBox->SetValue(false);

	styleSizer1->Add(orderCheckBox, 0, 0, 1); 
    styleBoxSizer->Add(styleSizer1, 0, wxEXPAND | wxALL, 1); 
    topSizer->Add(styleBoxSizer, 0, wxEXPAND | wxALL, 1); 

    wxStaticBox *box6;
    box6 = new wxStaticBox(this, -1, "Masking", wxPoint(-1,-1), wxSize(-1,-1),
        0, "staticBox");

    wxStaticBoxSizer *boxSizer6 = new wxStaticBoxSizer (box6, wxVERTICAL);

    groupDimComboBox = new wxComboBox(this, COMBO_GRPDIM, "", wxDefaultPosition, wxDefaultSize, 
        0, NULL, wxCB_SORT, wxDefaultValidator, "comboBox");

    wxStaticText *label3 = new wxStaticText(this, -1, "Dimension");
    wxBoxSizer *groupSizer1= new wxBoxSizer(wxHORIZONTAL);
    groupSizer1->Add(label3, 0, wxEXPAND | wxALL, 1);
    groupSizer1->Add(groupDimComboBox, 1, wxEXPAND | wxALL, 0);

    boxSizer6->Add(groupSizer1, 1, wxEXPAND | wxALL, 0);

    groupValueComboBox = new wxComboBox(this, COMBO_GRPVALUE, "", wxDefaultPosition, wxDefaultSize, 
        0, NULL, 0, wxDefaultValidator, "comboBox");
    wxStaticText *label4 = new wxStaticText(this, -1, "    Value");
    groupSizer1->Add(label4, 0, wxEXPAND | wxALL, 1);
    groupSizer1->Add(groupValueComboBox, 1, wxEXPAND | wxALL, 0);

    maskColorButton = new wxButton(this, BUTTON_MASKCOLOR, "Color", wxPoint(-1,-1), wxSize(-1,-1));
    maskColorButton->SetBackgroundColour(wxColor(33, 0, 0));
    wxStaticText *label6 = new wxStaticText(this, -1, "Color ");
    wxBoxSizer *groupSizer2= new wxBoxSizer(wxHORIZONTAL);
    groupSizer2->Add(label6, 0, wxEXPAND | wxALL, 1);
    groupSizer2->Add(maskColorButton, 0, 0, 1);

	maskSlider = new wxSlider(this, -1, 0.001, 0, 1000, wxPoint(-1,-1), wxSize(-1,-1),
        wxSL_HORIZONTAL, wxDefaultValidator, "slider");
    wxStaticText *label5 = new wxStaticText(this, -1, "   Transparency");
    groupSizer2->Add(label5, 0, wxEXPAND | wxALL, 1);
    groupSizer2->Add(maskSlider, 1, wxEXPAND | wxALL, 0);
    
    boxSizer6->Add(groupSizer2, 1, wxEXPAND | wxALL, 0);

	topSizer->Add(boxSizer6, 0, wxEXPAND | wxALL, 1); 

    Fit();


}

void VarControlFrame::Reset()
{
//     activeDimComboBox->Clear();
     groupDimComboBox->Clear();
     baseDimComboBox->Clear();
     scatterDimComboBox->Clear();
     groupValueComboBox->Clear();
//     xDimComboBox->Clear();
//     yDimComboBox->Clear();
     centerDimComboBox->Clear();

//     activeDimComboBox->Append("");
     baseDimComboBox->Append("");
     scatterDimComboBox->Append("");
     groupDimComboBox->Append("");
//     xDimComboBox->Append("");
//     yDimComboBox->Append("");
     centerDimComboBox->Append("");
     
     Iterator* it = docManager->GetDimensionNameIterator();
     std::string name;
     DocLabel* label;
     while (!it->IsDone())
     {
         label = (DocLabel*)(it->GetNext());
         name = label->GetText();
//         activeDimComboBox->Append(name.c_str());
         baseDimComboBox->Append(name.c_str());
         groupDimComboBox->Append(name.c_str());
//         xDimComboBox->Append(name.c_str());
//         yDimComboBox->Append(name.c_str());
         centerDimComboBox->Append(name.c_str());
         scatterDimComboBox->Append(name.c_str());
     }
     delete it;

    int i = VarGlyphModeRadioBox->GetSelection();
   // SetVarGlyphMode(i);

}


VarControlFrame::~VarControlFrame()
{
}


void VarControlFrame::OnScroll(wxScrollEvent& event)
{
    wxObject* object = event.GetEventObject();
    if (object == (wxObject *)sizeSlider)
    {
        int value = sizeSlider->GetValue();
        viewManager->VarSizeChanged(value/1000.f);
    }
    else if (object == (wxObject *)distortionSlider)
    {
        int value = distortionSlider->GetValue();
        viewManager->VarDistortionChanged(value/1000.f*5);
    }
    else if (object == (wxObject *)maskSlider)
    {
        int value = maskSlider->GetValue();
        viewManager->VarMaskAlphaChanged((1000 - value)/1000.f);
     }
    else if (object == (wxObject *)speedSlider)
    {
        int value = speedSlider->GetValue();
        docManager->SetCenterT(value/1000.f * 100);
        docManager->OnlineCreateAroundCenterPos();
        viewManager->VarPositionReset();
    }
    else if (object == (wxObject *)simiSelectSlider)
    {
        int mode = selectionModeRadioBox->GetSelection();
        if (mode != 2)
        {
            int value = simiSelectSlider->GetValue() - 1;
            if (docManager->SelectDimensionsAroundBase(value/1000.f))
               viewManager->SelectedDimensionsChanged();
        }
    }
}

void VarControlFrame::SetActiveDimension(wxCommandEvent& event)
{
    std::string str = activeDimComboBox->GetStringSelection();
     
    docManager->SetActiveDimension(str);
    viewManager->RefreshAll();

}

void VarControlFrame::SetBaseDimension(wxCommandEvent& event)
{
    std::string str = baseDimComboBox->GetStringSelection();
    int value = simiSelectSlider->GetValue() - 1;
    int i = selectionModeRadioBox->GetSelection();

    int id;
    docManager->SetBaseDimension(str, id);
    if (i != 2)
    {
        viewManager->BaseDimensionReset(id);
        docManager->SelectDimensionsAroundBase(value/1000.f);
    }
    else
    {
        viewManager->BaseDimensionReset(-1);
    }
    viewManager->SelectedDimensionsChanged();
}

void VarControlFrame::SetBaseDimension(std::string str)
{
    baseDimComboBox->SetValue(str.c_str());
    int value = simiSelectSlider->GetValue() - 1;
    int i = selectionModeRadioBox->GetSelection();
     
    int id;
    docManager->SetBaseDimension(str, id);
    if (i != 2)
    {
        viewManager->BaseDimensionReset(id);
        docManager->SelectDimensionsAroundBase(value/1000.f);
    }
    else
    {
        viewManager->BaseDimensionReset(-1);
    }
    viewManager->SelectedDimensionsChanged();
}


void VarControlFrame::SetVarGlyphBaseDimension(std::string str)
{
    int i = VarGlyphModeRadioBox->GetSelection();
    if (docManager->SetVarGlyphBaseDimension(str, i))
    {
        scatterDimComboBox->SetValue(str.c_str());
        viewManager->VarTextureChanged();
    }
}

void VarControlFrame::SetVarGlyphBaseDimension(wxCommandEvent& event)
{
    int i = VarGlyphModeRadioBox->GetSelection();
    std::string str = scatterDimComboBox->GetStringSelection();
    if (docManager->SetVarGlyphBaseDimension(str, i))
    {
        viewManager->VarTextureChanged();
    }
}

void VarControlFrame::SetGroupDimension(wxCommandEvent& event)
{
    std::string str = groupDimComboBox->GetStringSelection();
     
    if (docManager->SetGroupDimension(str))
    {
        DimensionSpace* dimSpace = docManager->GetAllDimensionSpace();
        Dimension* dim = dimSpace->GetDimension(str);
        std::string* strs;
        int* sizes;
        int groupNum;

        groupValueComboBox->Clear();
        dim->GetGroups(strs, sizes, groupNum);

        for (int i = 0; i < groupNum; i++)
        {
            std::ostringstream oss;
        	oss << strs[i] << "(" << sizes[i] << ")";

            groupValueComboBox->Append(oss.str().c_str());
        }
    }
    else 
    {
        groupValueComboBox->Clear();

    }
}

void VarControlFrame::SetSelectedGroupValue(wxCommandEvent& event)
{
    std::string str = groupValueComboBox->GetStringSelection();
    str = str.substr(0, str.rfind('('));
     
    docManager->SetGroupValue(str);
    viewManager->VarMaskChanged();
}

void VarControlFrame::SetMaskColor(wxCommandEvent& event)
{
     wxColourDialog* dlg = new wxColourDialog(this);
     if (dlg->ShowModal() == wxID_CANCEL)
     {
         delete dlg;
         return;
     }
     wxColour color = dlg->GetColourData().GetColour();
     
     maskColorButton->SetBackgroundColour(color);
     delete dlg;

     int temp;
     temp = color.Red();
     temp = color.Green();
     temp = color.Blue();
     viewManager->VarMaskColorChanged(RGBt(color.Red() / 255.f, color.Green() / 255.f, color.Blue() / 255.f, 1));

     return;

}

void VarControlFrame::SetXDimension(wxCommandEvent& event)
{
    std::string str = xDimComboBox->GetStringSelection();
     
    if (docManager->SetXDimension(str))
    {
        docManager->OnlineCreateXYDimensionPos();
        viewManager->VarPositionReset();
    }
}

void VarControlFrame::SetYDimension(wxCommandEvent& event)
{
    std::string str = yDimComboBox->GetStringSelection();
     
    if (docManager->SetYDimension(str))
    {
        docManager->OnlineCreateXYDimensionPos();
        viewManager->VarPositionReset();
    }
}

void VarControlFrame::SetCenterDimension(wxCommandEvent& event)
{
    std::string str = centerDimComboBox->GetStringSelection();
     
    if (docManager->IsEmpty())
        return;

    if (docManager->SetCenterDimension(str))
    {
        docManager->OnlineCreateAroundCenterPos();
    }
    else
    {
        docManager->CreateMDSPosForDimensions();
    }
    viewManager->VarPositionReset();
}

void VarControlFrame::SetDimensionDisplayMode(wxCommandEvent& event)
{
    int i = selectionDisplayRadioBox->GetSelection();

    docManager->ResetVarPositions(i);
    viewManager->VarDisplayModeReset(i);
//    pre_dimDisplayMode = i;
}

void VarControlFrame::SetDimensionSelectMode(wxCommandEvent& event)
{
    int i = selectionModeRadioBox->GetSelection();

    if (i == 0 || i == 1)
        selectionLabel->SetLabel("Sample Dimension");
    else
        selectionLabel->SetLabel("Select Dimension");

    docManager->SetDimensionSelectMode(i);

    if (i != 2)
    {
        int id = docManager->GetBaseDimensionID();
        if (id != -1)
        {
            std::string str = docManager->GetDimensionByID(id)->GetDimensionName();
            wxString wstr = str.c_str();
            baseDimComboBox->SetValue(wstr);
            viewManager->BaseDimensionReset(id);

            int value = simiSelectSlider->GetValue() - 1;

            if (docManager->SelectDimensionsAroundBase(value/1000.f))
               viewManager->SelectedDimensionsChanged();
        }
    }
}

void VarControlFrame::SetDimensionDistinctSelectOption(wxCommandEvent& event)
{
    int i = selectionOptionRadioBox->GetSelection();

    docManager->SetDistinctDimensionSelectOption(i);

    int value = simiSelectSlider->GetValue() - 1;

    if (docManager->SelectDimensionsAroundBase(value/1000.f))
       viewManager->SelectedDimensionsChanged();
}


void VarControlFrame::SetSamplingMode(wxCommandEvent& event)
{
    int i = samplingModeRadioBox->GetSelection();

    docManager->SetSamplingMode(i);
    viewManager->VarTextureChanged();
}

void VarControlFrame::SetBinaryMode(wxCommandEvent& event)
{
    int i = binaryModeRadioBox->GetSelection();

    docManager->SetBinaryMode(i);
    if (docManager->GetAllDimensionSpace() != NULL)
    {
        viewManager->VarTextureChanged();
    }
}

void VarControlFrame::SetVarGlyphMode(wxCommandEvent& event)
{
    int i = VarGlyphModeRadioBox->GetSelection();
    SetVarGlyphMode(i);
}

void VarControlFrame::SetVarGlyphMode(int i)
{
    Query* query = docManager->GetQuery();

    if (i == 0 || i == 1 || i == 2)
    {
        modeLabel->SetLabel("Order Dimension");
        orderCheckBox->Enable(true);
        int id = query->GetOrderByDimensionID();
        if (id == -1)
            scatterDimComboBox->SetValue("");
        else
        {
            wxString name = docManager->GetDimensionByID(id)->GetDimensionName().c_str();
            scatterDimComboBox->SetValue(name);
        }
        bool increasingOrder = query->IsIncreasingOrder();
        if (increasingOrder)
            orderCheckBox->SetValue(true);
        else 
            orderCheckBox->SetValue(false);
    }
    else if (i == 3)
    {
        modeLabel->SetLabel("X Dimension");
        orderCheckBox->Enable(false);
        int id = query->GetScatterDimensionID();
        if (id == -1)
            scatterDimComboBox->SetValue("");
        else
        {
            wxString name = docManager->GetDimensionByID(id)->GetDimensionName().c_str();
            scatterDimComboBox->SetValue(name);
        }
    }
    else if (i == 4)
    {
        modeLabel->SetLabel("Time Dimension");
        orderCheckBox->Enable(false);
        int id = docManager->GetQuery()->GetTimeDimensionID();
        if (id == -1)
            scatterDimComboBox->SetValue("");
        else
        {
            wxString name = docManager->GetDimensionByID(id)->GetDimensionName().c_str();
            scatterDimComboBox->SetValue(name);
        }
    }
    docManager->SetVarGlyphMode(i);
    viewManager->SetVarGlyphMode(i);
}

void VarControlFrame::SetLabelDisplayMode(wxCommandEvent& event)
{
    int i = labelDisplayRadioBox->GetSelection();

    docManager->ResetVarLabelingMode(i);
    viewManager->VarLabelingModeReset(i);
}


void VarControlFrame::RandomizeZ(wxCommandEvent& event)
{
     viewManager->VarZPosRandomReset();
}

void VarControlFrame::Resample(wxCommandEvent& event)
{
    docManager->GetQuery()->Resample();
    viewManager->VarTextureChanged();
}

void VarControlFrame::SetPixelIncreaseOrder(wxCommandEvent& event)
{
    docManager->GetQuery()->SetIncreasingOrder(orderCheckBox->IsChecked());
    viewManager->VarTextureChanged();
}

void VarControlFrame::SetSampleSize(wxCommandEvent& event)
{
    int size = atoi(sampleSizeTextCtrl->GetValue().c_str());
    if (size > 0)
        docManager->GetQuery()->SetApproximateSampleSize(size);
    docManager->GetQuery()->Resample();
    viewManager->VarTextureChanged();
}

void VarControlFrame::WindowClose(wxCloseEvent& event)
{
    MainFrame *frame = (MainFrame *)this->GetParent();
    frame->SetVarFrame(NULL);
    Destroy();
}
