// The interface for setting control modes and parameters of the var display

#ifndef VarControlFrame_H
#define VarControlFrame_H

#include <string>
#include <wx/wx.h>
#include <wx/filedlg.h>
#include "Frame.h"

enum  //use 401 - 499
{ 
    COMBO_ACTDIM =          401,
    COMBO_GRPDIM =          402,
    COMBO_GRPVALUE =        403,
    COMBO_ORDERBYDIM1 =     404,
    COMBO_ORDERBYDIM2 =     405,
    COMBO_ORDERBYDIM3 =     406,
    COMBO_BASEDIM =         407,
    BUTTON_MASKCOLOR =      408,

    COMBO_XDIM =            409,
    COMBO_YDIM =            410,


    COMBO_SCATTERDIM =      411,
    RADIO_DIMDISPLAYMODE =  412,
    RADIO_LABELDISPLAYMODE =413,
    RADIO_DIMSELECTMODE =   414,
    BUTTON_RANDOMVARZ =     415,
    RADIO_DIMSELECTOPTION = 416,
    RADIO_SAMPLINGMODE =    417,
    RADIO_BINARYMODE =      418,
    RADIO_VARGLYPHMODE =    419,

    COMBO_CENTERDIM =       420,

    CHECKBOX_ORDER =        421,
    TEXTCTRL_SAMPLESIZE =   422,
    BUTTON_RESAMPLE =       423,   
};


class VarControlFrame: public Frame
{
 protected:
	wxMenuBar  *menuBar;

    wxBoxSizer *topSizer;
	wxBoxSizer *quickButtonSizer;
	wxBoxSizer *majorSizer;

    wxSlider* sizeSlider;
    wxSlider* simiSelectSlider;
    wxSlider* distinctSelectSlider;
    wxSlider* distortionSlider;

    wxSlider* maskSlider;

    wxSlider* speedSlider;
    
    wxComboBox* activeDimComboBox;
    wxComboBox* baseDimComboBox;
    wxComboBox* scatterDimComboBox;
    wxComboBox* groupDimComboBox;
    wxComboBox* groupValueComboBox;

    wxComboBox* xDimComboBox;
    wxComboBox* yDimComboBox;

    wxComboBox* centerDimComboBox;
    
    wxButton* maskColorButton;
    wxButton* randomButton;
    wxButton* resampleButton;
    wxButton* selectAllButton;
    wxButton* unselectAllButton;

    wxRadioBox* selectionDisplayRadioBox;
    wxRadioBox* labelDisplayRadioBox;
    wxRadioBox* selectionModeRadioBox;
    wxRadioBox* selectionOptionRadioBox;
    wxRadioBox* samplingModeRadioBox;
    wxRadioBox* binaryModeRadioBox;
    wxRadioBox* VarGlyphModeRadioBox;

    wxStaticText *selectionLabel;
    wxStaticText *modeLabel;

    wxCheckBox *orderCheckBox;

    wxTextCtrl *sampleSizeTextCtrl;

 public:
	VarControlFrame(wxFrame *parent, int id, const wxChar *title, int xpos, int ypos, int width, int height);
	~VarControlFrame();

    void OnScroll(wxScrollEvent& event);

    void Reset();
    void SetActiveDimension(wxCommandEvent& event);
    void SetBaseDimension(wxCommandEvent& event);
    void SetGroupDimension(wxCommandEvent& event);
    void SetSelectedGroupValue(wxCommandEvent& event);
    void SetMaskColor(wxCommandEvent& event);

    void SetXDimension(wxCommandEvent& event);
    void SetYDimension(wxCommandEvent& event);

    void SetCenterDimension(wxCommandEvent& event);
    void SetVarGlyphBaseDimension(wxCommandEvent& event);
    void SetDimensionDisplayMode(wxCommandEvent& event);
    void SetLabelDisplayMode(wxCommandEvent& event);
    void SetDimensionSelectMode(wxCommandEvent& event);
    void SetBaseDimension(std::string str);
    void SetVarGlyphBaseDimension(std::string str);
    void RandomizeZ(wxCommandEvent& event);
    void SetDimensionDistinctSelectOption(wxCommandEvent& event);
    void SetSamplingMode(wxCommandEvent& event);
    void SetBinaryMode(wxCommandEvent& event);
    void SetPixelIncreaseOrder(wxCommandEvent& event);

    void SetVarGlyphMode(wxCommandEvent& event);
    void SetVarGlyphMode(int i);
    
    void SetSampleSize(wxCommandEvent& event);
    void Resample(wxCommandEvent& event);

    void WindowClose(wxCloseEvent& event);

	DECLARE_EVENT_TABLE()

};


#endif

