#include <wx/wx.h>
#pragma warning(disable:4663)    // unreferenced formal parameter
#include "Basic/macros.h"
#include "MainApplication.h"
#include <ctime>

IMPLEMENT_APP(MainApplication)


//this is the main entrance of the program 
bool MainApplication::OnInit()
{
    srand( (unsigned)time( NULL ));
    //viewManager is in charge of graph drawing in the program
    viewManager = new ViewManager();
    //docManager is in charge of data managemenet in the program
    docManager = new DocumentManager();
    //mainFrame is the main window of the program, It is in charge of the interface
    mainFrame = new MainFrame("LMDV - Semantic Image Browser", 0, 0, 700, 850);

    //the three have pointers to each other
    viewManager->SetDocumentManager(docManager);
    viewManager->SetMainFrame(mainFrame);

    docManager->SetViewManager(viewManager);
    docManager->SetMainFrame(mainFrame);
    
    mainFrame->SetViewManager(viewManager);
    mainFrame->SetDocumentManager(docManager);

    //the default is to get data from a database
    docManager->SetDBMode(true);
    //true for those binary datasets such as images and videos 
    docManager->SetBinaryMode(true);

    
    //the default display type is VaR display
    mainFrame->SetDisplayType(VAR); //The following types are available: VAR, PARCOORD, DIMSTACK or GLYPH;
	
    mainFrame->Show(true);

	return true;	
}

//this is the main exit of the program 
int MainApplication::OnExit()
{
    delete docManager;
    delete viewManager;
    //mainFrame is deleted automatically by wxWidgets. 
    return 0;
}

DocumentManager* MainApplication::GetDocumentManager()
{
    return docManager;
}

