#ifndef _VIEW_MANAGER_H
#define _VIEW_MANAGER_H

#include "Basic/CLinkList.h"
#include "Basic/LinkListIterator.h"
#include "Basic/macros.h"
#include "Basic/Vec2.h"

#include "Interface/grCanvas.h"
#include "Color/ColorManager.h"

class Drawer;
class MainFrame;
class DocumentManager;

class ViewManager
{
protected:
    DocumentManager* docManager;
    MainFrame* mainFrame;

    //color manager is in charge of color settings. Color manager is created and maintained by view manager
    ColorManager* colorManager;

    //this hash table records the canvas - drawer relationship
    PPHash canvasDrawerHash;
    //this hash table records the type of the drawers created
    IPHash typeDrawerHash;
public:
	ViewManager();
	~ViewManager();

    void SetDocumentManager(DocumentManager* documentManager) {this->docManager = documentManager;}
    DocumentManager* GetDocumentManager() {return docManager;}
    void SetMainFrame(MainFrame* mainFrame) {this->mainFrame = mainFrame;}
    MainFrame* GetMainFrame() {return mainFrame;}
    ColorManager* GetColorManager() {return colorManager;}

    //refresh all canvases managed by the view manager
    void RefreshAll();
    //the major canvas drawing function
    void RefreshCanvas(grCanvas* canvas);
    //key press handler
	void KeyPress(grCanvas* canvas, int key);
    //key release handler
	void KeyRelease(grCanvas* canvas, int key);
    //mouse button down handler. button: 0: left mouse button, 1: middle, 2: right
    void ButtonDown(grCanvas* canvas, int button, Vec2 p);                                      
    void ButtonUp(grCanvas* canvas, int button, Vec2 p);                                        
    void DoubleClick(grCanvas* canvas, int button, Vec2 p);                              
    void MouseMotion(grCanvas* canvas, Vec2 p);                                        

    //get the drawer associated with a canvas
    Drawer* GetDrawer(grCanvas* canvas);
    //given a display type, assign a drawer to a canvas
    //current, if a drawer of the type exists already, the existing drawer will be assigned
    //otherwise a new drawer will be created and assigned. In the future, we might support
    //multiple drawers with the same type thus users can have multiple different displays 
    //of the same type
    void AssignDrawerToCanvas(grCanvas* canvas, int type);
    //get a drawer of the given type
    Drawer* GetDrawer(int type);
    Drawer* CreateDrawer(int type);

    //handler for the event: the all dim space is deleted
    void AllDimensionSpaceDeleted();
    //handler for the event: the curr dim space is changed
    void CurrentDimensionSpaceChanged();

    //event handlers for the var display
    void VarSizeChanged(float value);
    void VarMaskColorChanged(RGBt color);
    void VarMaskAlphaChanged(float alpha);
    void VarPositionReset();
    void BaseDimensionReset(int id);
    void SelectedDimensionsChanged();
    void VarTextureChanged();
    void VarDisplayModeReset(int i);
    void VarLabelingModeReset(int i);
    void VarDistortionChanged(float value);
    void VarZPosRandomReset();
    void VarMaskChanged();
    void SetVarGlyphMode(int i);

    //event handlers for the glyph display
    void GlyphSizeChanged(float value);
    void GlyphPositionReset();
    void GlyphZPosRandomReset();
    void GlyphDisplayModeReset(int i);
    void GlyphDistortionChanged(float value);
    void FocusItemReset(int newID);
    void BaseItemReset(int newID);
    void SelectedItemsChanged();
    void RefreshGlyphView();

};

#endif

