#ifndef VIZ_H
#define VIZ_H

/****************************************************************/
/* headers                                                      */
/****************************************************************/
#include <stdio.h>
#include "general.h"
#include "nmatrix.h"
#include "matrix.h"
#include "animate.h"

/****************************************************************/
/* defines                                                      */
/****************************************************************/
#define MAX_NAME_LENGTH		2000		/* number of characters in dbase name */
#define MAX_DISPLAY_DIMS	8		/* number of display dimensions */
#define MAX_MEMBERS			2		/* number of subclusters per cluster */
#define	MAX_CONNECTIONS		10

#define DEFAULT_NUM_POINTS      20     /* number of points in database */
#define DEFAULT_MAX_ITER        10     /* number of iterations to animate */
#define DEFAULT_DATA_DIMS       10     /* number of dimensions of data */
#define DEFAULT_DISPLAY_DIMS    2      /* number of display dimensions */
#define DEFAULT_NUM_ATTRACTORS  2      /* number of datums that 'pull' datum */
#define DEFAULT_NUM_MEMBERS     2      /* number of subclusters per cluster */

#define NUM_INIT_CLUSTERS   4       /* number of clusters for data on init */
#define MAX_STRESS_POINTS   200

/*
 * draw graph defines
 */
#define INTEGER     0
#define LONG		1
#define FLOAT       2
#define STRING		3
#define SPECIAL		4

/*
 * graph plotting defines
 */
#define STRESS              1
#define SHEPARD             2
#define CORRELATION         4

/*
 * setup_screen_coords modes
 */
#define RANDOM		0
#define VARIANCE	1
#define DATA		2

/*
 * init modes
 */
#define	NO_DATA			0
#define	FROM_FILE		1
#define SIM_TRI			2
#define SIM_CUBE		3
#define SIM_HYPERCUBE	4
#define SIM_CLUSTER		5
#define SIM_TETRAHEDRON	6
#define FROM_ARRAY      7

/*
 * bond selection modes
 */
#define	TOP_TEN		0
#define RANDOM_ONCE	1
#define RANDOM_EACH	2

/*
 * display modes
 */
#define TEXT		1
#define GRAPHICS	2

/*
 * interact modes
 */
#define ROTATE		0
#define	PAN			1
#define	ZOOM		2
#define SELECT		3

/*
 * draw_db modes
 */
#define DOTS        1
#define LINES       2
#define TRACE       4
#define LABELS      8
#define ATTRACTIONS 16
#define LINKS       32
#define SPHERES     64
#define AXES   		128
#define UNIT   		256
#define BOUNDS   	512

/*
 * load_db modes
 */
#define LABEL		0
#define NOMINAL		1
#define NUMERIC		2

#define DB_CHECK(db)        if( !db )                                   	\
    {                                                                   	\
        print_error( "No data for MDS to run on - read in data first" );   	\
        return;                                                         	\
    }

#define DB_CHECK_DATA(db)   if( !db || !db->dat.data )                  	\
    {                                                                   	\
        print_error( "No data for MDS to run on - read in data first" );   	\
        return;                                                         	\
    }

#define NDIST_MSG   "About to recalculate %ld pairwise distance\n\
relations.  This may take a considerable \n\
amount of time.\n\
\n\
Do you wish to proceed?"

#define MEM_CHECK(s) \
    if( db && db->dat.P ) printf( "%s - %d\n", s, *((UINT *)((char *)(db->dat.P->data[1]) - sizeof(UINT))) )

/****************************************************************/
/* typedefs                                                     */
/****************************************************************/
typedef struct dim_info{
	int			type;
	char		*label;
	double		min;
	double		max;
	}DIM_INFO;

typedef struct nominal_info{
	char					*name;
	long					column;
	struct nominal_info		*next;
	}NOMINAL_INFO;
	
typedef struct datum{
	long			i;
	char			*label;
	long			num_bonds;
	double			total_stress;
	NOMINAL_INFO	*nominal_head;
	NOMINAL_INFO	*nominal_tail;
	long			sub_clusters;
	long			num_children;
	struct datum	*members[MAX_MEMBERS];
	long			num_connections;
	long			connections[MAX_CONNECTIONS];
	}DATUM;			

typedef struct mds_info{
	int				select_mode;
	long			num_bonds;
	int				norm_function;
	int				dist_func;			
	int				stress_function;
	double			step_factor;
	double			noise_factor;
	int				display_init_mode;
	char			*dist_expression;
	char			*stress_expression;
	}MDS_INFO;

typedef struct mds_stats{
	long			num_ndist;
	long			num_dropped;
	double			sum_ndist;
	double			min_ndist;
	double			max_ndist;
	double			mean_ndist;
	double			sum_sdist;
	double			max_bond_stress;
	double			max_datum_stress;
	double			start_max_bond_stress;
	double			start_max_datum_stress;
	}MDS_STATS;

typedef struct file_info{
	char            *infile;
    char            *outfile;
    char            *viewfile;
    FILE            *reportfp;
	}FILE_INFO;

typedef struct data_info{
	int				data_init_mode;
	long			num_points;
	long			num_dists;
	long			data_dims;
	DIM_INFO		*data_dim_array;
	int				*dim_mask;
	long			display_dims;
	long			num_clusters;
	long			max_children;
	double			clust_prob;
	int				num_P;
	MATRIX			*R;					/* unnormalized data matrix */
	MATRIX			*A;					/* normalized data matrix */
	MATRIX			*P;					/* display point matrix */
	MATRIX			*PO;				/* old display points matrix */
	MATRIX			*POO;				/* old old display points matrix */
	MATRIX			*POOO;				/* old old old display points matrix */
	MATRIX			*IP;				/* interpolated data points */
	MATRIX			*nD;				/* matrix of n-dimensional distances */
	MATRIX			*sD;				/* matrix of display distances */
	MATRIX			*S;					/* matrix of stresses */
	MATRIX			*O;					/* matrix of references */
	DATUM			*data;				/* pointer to list of data points */
	unsigned int	seed;
	}DATA_INFO;

typedef int PT[2];
	
typedef struct database{
	char			*name;
	int				interact_mode;
	int				output_mode;
	int				draw_mode;
	int				plot_mode;
	FILE_INFO		files;
	DATA_INFO		dat;
	MDS_INFO		mds;
	MDS_STATS		stats;
	ANIMATE			animate;
	}DB;

#endif VIZ_H
